#include <oswrapper.h>
#include "voiceSysCtl.h"
#if DECT_MODULE
//#include <adi6843.h>
#include "linux/sbi_defs.h"
void dectTasklet(unsigned long param);
DECLARE_TASKLET(dect_tasklet,dectTasklet,0);

#define DECT_RX_BUFFER	0x1534 
#define DECT_TX_BUFFER	0x150B 
#define DECT_TO_DSP_PCM_TXRX_COMPLETE_FLAG	0x1533 


extern UCHAR 		dspDataMode[MAX_VOICE_CHANNELS];
extern UINT16 		spPacketLen[];
extern UINT16 		spPacketLen_5ms[];
extern UCHAR 		gDataModeEnable;
extern DspStateInfo_t   gaDspStateInfo[MAX_VOICE_CHANNELS];
extern VoiceQ_t         *gVoiceBuf[MAX_VOICE_CHANNELS];
extern VoiceQ_t         gVoiceTxQBuffer[MAX_VOICE_CHANNELS][MAX_TX_VOICE_Q];
extern UCHAR            ucG7115msIntrFlag[MAX_VOICE_CHANNELS];
extern VoiceCirQ_t      gVoiceTxQ[MAX_VOICE_CHANNELS];
extern VoiceQ_t         *gVoiceBuf[MAX_VOICE_CHANNELS];
extern UINT32           dummy_value, *dummy_read_addr;
/* SAGEM -> [ Managed Start/Stop transcode from DECT module ] { */
extern void dspipcStopDectTransCode(unsigned portNum);
extern void dspipcStartDectTransCode(UINT16 portNum, UCHAR ucDectCodecType);
/* } */
/* SAGEM [ Add Timestamp increase ] { */
extern UINT32 ulTimeStampCntA[];
extern int dectFree(void);
/** Base statistics. */
extern struct dsp_voiceSysCtl_str dsp_voiceSysCtl;
/* SAGEM } */

dsp_regEndPoint_str     *dsp_regEndPoint = NULL;

#if DECT_DEBUG
dectDebugInfo_t		dectDebug;
/***********************************************************************
 * Function    : dectGetDebugStats.                                    *
 * Description : Function to fetch debug information related to DECT   *
 * Input       : none.                                                 *
 * Output      : None                                                  *
 * Return      : Pointer to debug structure (dectDebugInfo_t*)         *
 **********************************************************************/
dectDebugInfo_t* dectGetDebugStats(void)
{
  return &dectDebug;
}
#endif

int dectPassThrEnabled(int port)
{
  if(dsp_regEndPoint && dsp_regEndPoint->port == port &&
        dsp_regEndPoint->withoutConv.writeVoice != NULL)
    return SUCCESS;
  return FAILURE;
}

/* SAGEM -> [ Managed Start/Stop transcode from DECT module ] { */
/***********************************************************************
 * Function    : dectCmd. 			                               *
 * Description : Function allow to perform commands to DSP             *
 **********************************************************************/
int dectCmd(DSP_ENDPOINT_Cmd_e cmd, void *param)
{
  switch (cmd) {
    case DSP_ENDPOINT_Cmd_Start_transcode:
      dspipcStartDectTransCode(1, 0);
      break;
    case DSP_ENDPOINT_Cmd_Stop_transcode:
      dspipcStopDectTransCode(1);
      break;
    case DSP_ENDPOINT_Cmd_Start_FXO :
      dsp_regEndPoint->dectMode = DSP_DECT_MODE_FX0;
      break;
    case DSP_ENDPOINT_Cmd_Stop_FXO :
      dsp_regEndPoint->dectMode = DSP_DECT_MODE_FREE;

    default:
      printk(	"%s: FAILURE -> cmd(%d) unknown\n",
			__FUNCTION__,
			cmd);

      return FAILURE;
      break;
  }

  return SUCCESS;
}
/* } */

/***********************************************************************
 * Function    : dectRegisterCallBacks.                                *
 * Description : Function to register DECT call backs to voice module. *
 * Input       : dsp_regEndPoint_str                                   *
 * Output      : None                                                  *
 * Return      : None.                                                 *
 **********************************************************************/
void dectRegisterCallBacks(dsp_regEndPoint_str *callbacks) {
  dsp_regEndPoint = callbacks;

	if( dsp_regEndPoint == NULL)
		return;

/* SAGEM -> [ DEBUG ]  { */
  if ((1 << DECT_DEBUG) & gDebugLevel) {
    printk(	"%s: callback(%p)\n",
			__FUNCTION__,
			dsp_regEndPoint);

   printk(	"%s: callback(%p) -> port(%d) maxFrames(%d) ref(%p) withConv[read(%p) write(%p)] withoutConv[read(%p) write(%p)] evt(%p) cmd(%p)\n",
			__FUNCTION__,
  			dsp_regEndPoint,
			dsp_regEndPoint->port,
			dsp_regEndPoint->maxFrames,
			dsp_regEndPoint->ref,
  			dsp_regEndPoint->withConv.writeVoice,
  			dsp_regEndPoint->withConv.readVoice,
  			dsp_regEndPoint->withoutConv.readVoice,
  			dsp_regEndPoint->withoutConv.writeVoice,
  			dsp_regEndPoint->evtFromDsp,
  			dsp_regEndPoint->cmdToDsp);
  }
/* } */

/* SAGEM -> [ Managed Start/Stop transcode from DECT module ] { */
  dsp_regEndPoint->cmdToDsp = dectCmd;
/* } */
  dectInit(dsp_regEndPoint->port);
  dspipcClearQs(dsp_regEndPoint->port);
  confSetMaxFrames(dsp_regEndPoint->port,dsp_regEndPoint->maxFrames);
}
/* SAGEM -> [Export for DECT] { */
EXPORT_SYMBOL(dectRegisterCallBacks);
/* } */

/***********************************************************************
 * Function    : dectUnRegisterCallBacks.                              *
 * Description : Function to un-register DECT call backs to voice      *
 *   module.                                                           *
 * Input       : None                                                  *
 * Output      : None                                                  *
 * Return      : None.                                                 *
 **********************************************************************/
void dectUnRegisterCallBacks(void)
{
  dectFree();
  dsp_regEndPoint = NULL;
}
/* SAGEM -> [Export for DECT] { */
EXPORT_SYMBOL(dectUnRegisterCallBacks);
/* } */

/***********************************************************************
 * Function    : dectIntHandler.                                       *
 * Description : Interrupt Service Routine for DSP servicing DEC.      *
 * Return      : IRQ_HANDLED.                                          *
 **********************************************************************/
int dectIntHandler(int irq, void *dev_id, struct pt_regs *regs)
{
  /* Clear the latched PF4 bit */
  *(volatile UINT32*)DSP_LATCH_ADDR_HI = 0x01000000;
#ifdef VOICE_TASKLET_MODE
  tasklet_hi_schedule(&dect_tasklet);
#else
  dectTasklet(0);
#endif
  return IRQ_HANDLED;
}
/***********************************************************************
 * Function    : dectFree.                                             *
 * Description : This function free DECT module,                 *
 * Input       : DSP Number.                                           *
 * Return      : SUCCESS or error.                                     *
 **********************************************************************/
int dectFree(void)
{
  /* Free IRQ */  
free_irq(DSP1_FL1_INT,
				&dsp_regEndPoint->port);
/* end BUG 3120 */
  return SUCCESS;
}
/***********************************************************************
 * Function    : dectInit.                                             *
 * Description : This function initialize DECT module,                 *
 * Input       : DSP Number.                                           *
 * Return      : SUCCESS or error.                                     *
 **********************************************************************/
int dectInit(int dsp)
{
  int result;
  /* Program FL1 interrupt */
  result = request_irq(DSP1_FL1_INT,
                       dectIntHandler,
		       SA_INTERRUPT,
                       "dect",
	               &dsp_regEndPoint->port);
  switch(result)
  {
    case -EBUSY:
      printk(KERN_ERR ": IRQ %d busy\n", DSP1_FL1_INT);
      return -EBUSY;
    case -EINVAL:
      printk(KERN_ERR ": Bad irq number or handler\n");
      return -EINVAL;
  }
  DEBUG(PRINT("\n Created IRQ for DECT\r\n"),DEBUG_INFO);
#if DECT_DEBUG
  memset(&dectDebug,0,sizeof(dectDebug));
#endif
  return SUCCESS;
}

/***********************************************************************
 * Function    : dectWriteVoice.                                       *
 * Description : This function write the voice packet to DECT module,  *
      when conversion is not required.                                 *
 * Input       : Port, Voice Payload and Payload Length.               *
 * Return      : SUCCESS / FAILURE.                                    *
 **********************************************************************/
int dectWriteVoice(int port, UCHAR *data, int len)
{
  if( (dsp_regEndPoint == NULL) ||
      (port != dsp_regEndPoint->port) ||
      ((dsp_regEndPoint->withoutConv.writeVoice == NULL) && (dsp_regEndPoint->dectToneDtmf_Active == 0)) )
  {
    return FAILURE;
  }
  else if (dsp_regEndPoint->dectToneDtmf_Active == 1)
  	return SUCCESS;
#ifdef VOICE_SYSCTL_STATS
  /* Stats */
  {
    dsp_voiceSysCtl.ports[port].stats.network.external.snd.pkts	++;
    dsp_voiceSysCtl.ports[port].stats.network.external.snd.bytes	+= len;
  }
#endif

  dsp_regEndPoint->withoutConv.writeVoice(dsp_regEndPoint->ref, data, len);
#if DECT_DEBUG
  dectDebug.withConvWrite++;
#endif
  return SUCCESS;
}
/***********************************************************************
 * Function    : dectReadVoicePkt.                                     *
 * Description : This function reads the voice packet from DECT module *
 * and puts into the Transmit Q for RTP.                               *
 * Input       : port.                                                 *
 * Return      : SUCCESS / FAILURE.                                    *
 **********************************************************************/
int dectReadVoicePkt(int port)
{
  int len = 80;

  if( (dsp_regEndPoint == NULL) ||
      (port != dsp_regEndPoint->port) ||
      (dsp_regEndPoint->withoutConv.readVoice == NULL) )
  {
    return FAILURE;
  }
  /* Dont Send Any thing Out if the mode is not correct */
  if(gDataModeEnable == TRUE)
  {
    if(dspDataMode[port] == DSP_DATA_INACTIVE 
          || dspDataMode[port] == DSP_DATA_RECV_ONLY 
                      || gaDspStateInfo[port].bInitDone == FALSE)
    {
      DEBUG_ISR(PRINT("dspipcIsr => Pkt Drop, incorect mode or port is not opened[%d]\n",port), 
                     DEBUGISR);

#ifdef VOICE_DEBUG_TEST
    if (gaDspStateInfo[port].bInitDone != FALSE) {
		static int initDone = 0;

		initDone++;

		if (initDone == 200) {
			printk(	"%s: In Test initDone Port(%d) DSP->RTP with DECT with way(%d)-> ",
					__FUNCTION__,
					port,
      				dspDataMode[port]);

      		if (dspDataMode[port] == DSP_DATA_INACTIVE) {
				printk("INACTIVE\n");
			}

      		if (dspDataMode[port] == DSP_DATA_RECV_ONLY) {
				printk("RECV_ONLY\n");
			}

      		if (dspDataMode[port] == DSP_DATA_SEND_ONLY) {
				printk("SEND_ONLY\n");
			}

      		if (dspDataMode[port] == DSP_DATA_SEND_RECV) {
				printk("SEND_RECV\n");
			}

			initDone = 0;
		}
	}
#endif
    
        return SUCCESS;
      }
    }
    else 
    {
      if(gaDspStateInfo[port].bInitDone == FALSE)
      {
        DEBUG_ISR(PRINT("dspipcIsr => Pkt Drop, port is not opened[%d]\n",port),DEBUGISR);
        return SUCCESS;
      }
  }

  ulTimeStampCntA[port] += len;
 #if VOICE_DEBUG_TEST
	{
		static int initDone = 0;

		initDone++;

		if (initDone == 200) {
			printk(	"%s: DECT Port(%d) DSP->RTP\n",
					__FUNCTION__,
					port);

			initDone = 0;
		}
	}
#endif

#ifdef VOICE_SYSCTL_STATS
  /* Stats */
  {
    dsp_voiceSysCtl.ports[port].stats.network.external.rcv.pkts	++;
    dsp_voiceSysCtl.ports[port].stats.network.external.rcv.bytes	+= len;
  }
#endif

  if( (gVoiceBuf[port] == NULL) || 
      (gVoiceBuf[port]->cFrameCounter == 0) ||
      (gVoiceBuf[port]->cFrameCounter >= confGetMaxFrames(port)) )
  {
    /* Allocate a TX packet for RTP */
    gVoiceTxQ[port].writeIdx = 
           (gVoiceTxQ[port].writeIdx + 1) % MAX_TX_VOICE_Q;
    gVoiceBuf[port] = &(gVoiceTxQBuffer[port][gVoiceTxQ[port].writeIdx]);
    gVoiceBuf[port]->ucPktLen = 0;
    gVoiceBuf[port]->cFrameCounter = 0;
    gVoiceBuf[port]->ucChNum = port;
  }
  /* Call Dect Call Back to read the voice packet from DECT module */
  dsp_regEndPoint->withoutConv.readVoice(dsp_regEndPoint->ref,
       &(gVoiceBuf[port]->aucPayload[12+gVoiceBuf[port]->ucPktLen]),
       len);
#if DECT_DEBUG
  dectDebug.withConvRead++;
#endif
  gVoiceBuf[port]->ucPktLen +=len;
  gVoiceBuf[port]->ulTimeStamp = dspipcGetTimeStamp(port);
  gVoiceBuf[port]->cFrameCounter++;

  return SUCCESS;
}
/***********************************************************************
 * Function    : dectISR.                                              *
 * Description : This is the tasklet routine to handle DECT ISR this is*
 *    is mainly used for DECT <-> DSP communications for PCM samples.  *
 * Input       : None.                                                 *
 * Output      : None.                                                 *
 * Return      : None.                                                 *
 **********************************************************************/
void dectTasklet(unsigned long param)
{
  UCHAR g729TransCode[82];
  UINT16 temp_addr;
  UINT16 temp_data;
  int len = 80;
  int count;

#if DECT_DEBUG
  dectDebug.dectTaskCnt++;
#endif
  if(dsp_regEndPoint  == NULL)
    return;
if(dsp_regEndPoint->dectMode == DSP_DECT_MODE_FX0)   
{ 
  /* Read the PCM samples ( converted from FXO)  */
  if(dsp_regEndPoint->withoutConv.writeVoice != NULL)
  {
    UCHAR *dest = g729TransCode;
    temp_addr = DECT_TX_BUFFER; 
    /* Read the packet from DSP */
    for(count=0; count<len; count+=2,temp_addr++)
    {
      temp_data   = dspReadDataMem(dsp_regEndPoint->port,temp_addr);
      *(dest+count)   = (UCHAR)temp_data & 0xff;
      *(dest+count+1) = (UCHAR)((temp_data >> 8) & 0xff);
      dummy_value = *dummy_read_addr;
      dummy_value = *dummy_read_addr;
    }
#if DECT_DEBUG
    dectDebug.dspTranscodeRead++;
#endif
    /* Write the packet to the DECT */
    if(dsp_regEndPoint->withoutConv.writeVoice != NULL)
    /* Well it should not be NULL */
    {
      dsp_regEndPoint->withoutConv.writeVoice(dsp_regEndPoint->ref,
           g729TransCode,len);
#if DECT_DEBUG
      dectDebug.withConvWrite++;
#endif
    }
  }
  /* Write the data to the DSP for transcoding if required */
  if(dsp_regEndPoint->withoutConv.readVoice != NULL)
  {
    temp_addr = DECT_RX_BUFFER;
    /* Call Dect Call Back to read the voice packet from DECT module */
    dsp_regEndPoint->withoutConv.readVoice(dsp_regEndPoint->ref,g729TransCode,len);

#if DECT_DEBUG
    dectDebug.withConvRead++;
#endif
    /* Write the packet to the DSP for transcoding */
    for(count=0; count<len; count+=2,temp_addr++)
    {
      temp_data = g729TransCode[count] | (g729TransCode[count+1] << 8);	
      dummy_value = *dummy_read_addr;
      dspWriteIntoDataMem(dsp_regEndPoint->port,temp_addr, temp_data);
      dummy_value = *dummy_read_addr;
      dummy_value = *dummy_read_addr;
    }
#if DECT_DEBUG
    dectDebug.dspTranscodeWrite++;
#endif
  }
}
else
{
  /* Read the PCM samples ( converted from G-729)  */
  if(dsp_regEndPoint->withConv.writeVoice != NULL)
  {
    UCHAR *dest = g729TransCode;
    temp_addr = DECT_TX_BUFFER; 
    /* Read the packet from DSP */
    for(count=0; count<len; count+=2,temp_addr++)
    {
      temp_data   = dspReadDataMem(dsp_regEndPoint->port,temp_addr);
      *(dest+count)   = (UCHAR)temp_data & 0xff;
      *(dest+count+1) = (UCHAR)((temp_data >> 8) & 0xff);
      dummy_value = *dummy_read_addr;
      dummy_value = *dummy_read_addr;
    }
#if DECT_DEBUG
    dectDebug.dspTranscodeRead++;
#endif
    /* Write the packet to the DECT */
    if(dsp_regEndPoint->withConv.writeVoice != NULL)
#ifdef VOICE_SYSCTL_STATS
    /* Stats */
    {
      dsp_voiceSysCtl.ports[dsp_regEndPoint->port].stats.endPoint.external.snd.pkts	++;
      dsp_voiceSysCtl.ports[dsp_regEndPoint->port].stats.endPoint.external.snd.bytes	+= len;
    }
#endif

    /* Well it should not be NULL */
    {
      dsp_regEndPoint->withConv.writeVoice(dsp_regEndPoint->ref,
           g729TransCode,len);
#if DECT_DEBUG
      dectDebug.withConvWrite++;
#endif
    }
  }
  /* Write the data to the DSP for transcoding if required */
  if(dsp_regEndPoint->withConv.readVoice != NULL)
  {
    temp_addr = DECT_RX_BUFFER;

#ifdef VOICE_SYSCTL_STATS
    /* Stats */
    {
      dsp_voiceSysCtl.ports[dsp_regEndPoint->port].stats.endPoint.external.rcv.pkts	++;
      dsp_voiceSysCtl.ports[dsp_regEndPoint->port].stats.endPoint.external.rcv.bytes	+= len;
    }
#endif

    /* Call Dect Call Back to read the voice packet from DECT module */
    dsp_regEndPoint->withConv.readVoice(dsp_regEndPoint->ref,g729TransCode,len);

#if DECT_DEBUG
    dectDebug.withConvRead++;
#endif
    /* Write the packet to the DSP for transcoding */
    for(count=0; count<len; count+=2,temp_addr++)
    {
      temp_data = g729TransCode[count] | (g729TransCode[count+1] << 8);	
      dummy_value = *dummy_read_addr;
      dspWriteIntoDataMem(dsp_regEndPoint->port,temp_addr, temp_data);
      dummy_value = *dummy_read_addr;
      dummy_value = *dummy_read_addr;
    }
#if DECT_DEBUG
    dectDebug.dspTranscodeWrite++;
#endif
  }
}	  
  /* Tell DSP, we are done */
  dspWriteIntoDataMem(dsp_regEndPoint->port,DECT_TO_DSP_PCM_TXRX_COMPLETE_FLAG, 0x1);
}

#if DECT_DEBUG
void dectPrintDectStat(void)
{
  printk("withConvWrite : %d \r\n withConvRead : %d \r\nwithoutConvWrite : %d \r\nwithoutConvRead : %d\r\ndectTaskCnt : %d\r\ndspTranscodeRead : %d\r\ndspTranscodeWrite : %d\r\n",dectDebug.withConvWrite,dectDebug.withConvRead,dectDebug.withoutConvWrite,dectDebug.withoutConvRead,dectDebug.dectTaskCnt,dectDebug.dspTranscodeRead,dectDebug.dspTranscodeWrite);
}
#endif
#endif
