#include <oswrapper.h>

// SAGEM
#include <dsp_util.h>
#include <voiceSysCtl.h>

#ifdef VOICE_SYSCTL_STATS
/** Base statistics. */
extern struct dsp_voiceSysCtl_str dsp_voiceSysCtl;
#endif

#if KERNEL_RTP_INTERFACE
dsp_regNetwork_t *pNWCallBack = NULL;
extern UINT32 ulTimeStampCntA[MAX_VOICE_CHANNELS];
/**********************************************************************************
 *  Function Name  : dsp_get_nw_pkt.                                              *
 *  Description    : Function to get voice packet within the kernel.              *
 *  Input          : Channel.                                                     *
 *  Output         : Voice Packet.                                                *
 *  Return Value   : SUCCESS / FAILURE.                                           *
 *********************************************************************************/
int dsp_get_nw_pkt(VoiceQ_t **pVoice, int channel)
{
  *pVoice = dspipcGetVoicePkt(channel, DSP_MEDIA_AUDIO);
  if(*pVoice == NULL) {
#ifdef VOICE_SYSCTL_STATS
    /* Stats */
    dsp_voiceSysCtl.ports[channel].stats.network.rtp.error.sndVoiceNull++;
#endif

    return FAILURE;
  }

#ifdef VOICE_SYSCTL_STATS
  /* Stats */
  {
    dsp_voiceSysCtl.ports[channel].stats.network.rtp.snd.pkts	++;
    dsp_voiceSysCtl.ports[channel].stats.network.rtp.snd.bytes	+= (*pVoice)->ucPktLen;
  }
#endif

  return SUCCESS;
}
/**********************************************************************************
 *  Function Name  : dsp_write_nw_pkt.                                            *
 *  Description    : Function to write voice packet from kernel.                  *
 e  Input          : Voice Packet.                                                *
 *  Output         : None.                                                        *
 *  Return Value   : SUCCESS / FAILURE.                                           *
 *********************************************************************************/
int dsp_write_nw_pkt(VoiceNWPkt_t *voice,unsigned char *pPayload)
{
  WtRtpPktType_ jitPkt;
  if(voice->channel > MAX_VOICE_CHANNELS) {
#ifdef VOICE_SYSCTL_STATS
    /* Stats */
    dsp_voiceSysCtl.stats.network.rcvBadChannel++;
#endif

    return FAILURE;
  }

#ifdef VOICE_SYSCTL_STATS
  /* Stats */
  {
    dsp_voiceSysCtl.ports[voice->channel].stats.network.rtp.rcv.pkts	++;
    dsp_voiceSysCtl.ports[voice->channel].stats.network.rtp.rcv.bytes	+= voice->FrameLen;
  }
#endif

#if DECT_TESTING
  voice->channel = 1;
#endif

  /* Fill AJB Parameters */
  jitPkt.CodecType       = voice->CodecType;
  jitPkt.SeqNum          = voice->SeqNum;
  jitPkt.SR_TIMESTAMP    = voice->timeStamp;
  jitPkt.RR_TIMESTAMP    = dspipcGetTimeStamp(voice->channel);
  jitPkt.FrameLen        = voice->FrameLen;

/* SAGEM -> [ DEBUG ] { */
  if ((1 << DEBUG_KRTP) & gDebugLevel) {
    printk(	"%s: Port(%d) codec(%d) SeqNum(%d) SR_TimeStamp(%lu) RR_TimeStamp(%lu) FrameLen(%d)\n",
			__FUNCTION__,
			voice->channel,
			jitPkt.CodecType,
			jitPkt.SeqNum,
			jitPkt.SR_TIMESTAMP,
			jitPkt.RR_TIMESTAMP,
			jitPkt.FrameLen);
  }
/* } */

  /* Write Packet to AJB Module */
  WtJitBuffer(&jitPkt,voice->channel,pPayload);
  return SUCCESS;
}

/* SAGEM -> [Add for managed event/command] { */
/**********************************************************************************
 *  Function Name  : dsp_cmd. 		                                          *
 *  Description    : Function allow to activate commands                          *
 e  Input          : port.                                                        *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
int dsp_evt(DSP_NETWORK_Evt_fromNetwork_e evt, int channel, void *param)
{
  switch(evt) {
    case DSP_NETWORK_Evt_fromNetwork_digit_start_report:
      break;

    case DSP_NETWORK_Evt_fromNetwork_digit_stop_report:
      break;

    case DSP_NETWORK_Evt_fromNetwork_digit_received:
      {
		DSPGenToneInfo_t	_toneInfo;

#ifdef VOICE_SYSCTL_STATS
          /* Stats */
          dsp_voiceSysCtl.ports[channel].stats.network.rtp.rcv.dtmfs++;
#endif

		/* Set informations */
		{
			_toneInfo.portNum				= channel;
			_toneInfo.toneType				= DSP_LOCAL_TONE;
			_toneInfo.tone.local.toneId		= dsp_dtmfConv_Char2DspTone((int)param);
			_toneInfo.tone.local.timeout		= 30;
		}

		printk(	"%s: Dtmf(%c) on channel(%d)\n",
				__FUNCTION__,
				(int)param,
				channel);
		
          dspipcGenTone(	_toneInfo.portNum,
					_toneInfo.tone.local.toneId,
					_toneInfo.tone.local.timeout);
      }
      break;

   default:
     printk(	"\n\n\n%s: Don't knwon dsp_evt(%d)\n\n\n",
			__FUNCTION__,
			evt);

     return(FAILURE);
     break;
  }

  return(SUCCESS);
}

/**********************************************************************************
 *  Function Name  : dsp_cmd. 		                                          *
 *  Description    : Function allow to activate commands                          *
 e  Input          : port.                                                        *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
int dsp_cmd(DSP_NETWORK_Cmd_toDsp_e cmd, int channel, void *param)
{
  switch(cmd) {
    case DSP_NETWORK_Cmd_toDsp_get_timeStamp:
        *((int *)param) = ulTimeStampCntA[channel];  
	 break;

    case DSP_NETWORK_Cmd_toDsp_get_frames_per_packet:
        *((int *)param) = (int)(confGetMaxFrames(channel));  
	 break;

    case DSP_NETWORK_Cmd_toDsp_clear_buffer:
      dspipcClearQs(channel);
      break;

    case DSP_NETWORK_Cmd_toDsp_mode_data_enable:
#ifdef VOICE_SYSCTL_STATS
      /* Stats */
      dsp_voiceSysCtl.stats.network.rcvModeDataEnable++;
#endif

      dspipcSetDataMode((int)param);
      break;

   case DSP_NETWORK_Cmd_toDsp_mode_data:
     {
       DSPDataMode_t *pDspDataMode = (DSPDataMode_t *)param;
	  int _mode = pDspDataMode->mode;

#if 0
	  printk(	"%s: Modify Data Mode(%d) on port(%d)",
			__FUNCTION__,
			_mode,
       		pDspDataMode->portNum);
#endif

    	  if (_mode == DSP_DATA_INACTIVE) {
#ifdef VOICE_SYSCTL_STATS
          /* Stats */
          dsp_voiceSysCtl.ports[pDspDataMode->portNum].stats.network.rtp.rcv.modeData.inactive++;
#endif

#if 0
		printk("INACTIVE\n");
#endif
	  }

    	  if (_mode == DSP_DATA_RECV_ONLY) {
#ifdef VOICE_SYSCTL_STATS
          /* Stats */
          dsp_voiceSysCtl.ports[pDspDataMode->portNum].stats.network.rtp.rcv.modeData.recv_only++;
#endif

#if 0
		printk("RECV_ONLY\n");
#endif
	  }

    	  if (_mode == DSP_DATA_SEND_ONLY) {
#ifdef VOICE_SYSCTL_STATS
          /* Stats */
          dsp_voiceSysCtl.ports[pDspDataMode->portNum].stats.network.rtp.rcv.modeData.send_only++;
#endif

#if 0
		printk("SEND_ONLY\n");
#endif
	  }

    	  if (_mode == DSP_DATA_SEND_RECV) {
#ifdef VOICE_SYSCTL_STATS
          /* Stats */
          dsp_voiceSysCtl.ports[pDspDataMode->portNum].stats.network.rtp.rcv.modeData.send_recv++;
#endif

#if 0
		printk("SEND_RECV\n");
#endif
	  }

       dspipcSetDSPDataMode(pDspDataMode->portNum,pDspDataMode->mode);
     }
     break;

   default:
     printk(	"\n\n\n%s: Don't knwon dsp_cmd(%d)\n\n\n",
			__FUNCTION__,
			cmd);

     return(FAILURE);
     break;
  }

  return(SUCCESS);
}
/* } */

/**********************************************************************************
 *  Function Name  : dsp_registerNetwork.                                         *
 *  Description    : Function to register call backss for KERNEL_RTP_INTERFACE.   *
 *                   functionality.                                               *
 e  Input          : dsp_regNetwork_str.                                          *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dsp_registerNetwork(dsp_regNetwork_t *_pRegNetwork)
{
  /* Initialize call back functions */
  _pRegNetwork->writeVoice = dsp_write_nw_pkt;
  _pRegNetwork->readVoice  = dsp_get_nw_pkt;
/* SAGEM -> [Add for managed event/command] { */
  _pRegNetwork->cmdToDsp  = dsp_cmd;
  _pRegNetwork->evtFromNetwork  = dsp_evt;
/* } */
  pNWCallBack = _pRegNetwork;
}
/* SAGEM -> [Export for RTP] { */
EXPORT_SYMBOL(dsp_registerNetwork);
/* } */

/**********************************************************************************
 *  Function Name  : dsp_unregisterNetwork.                                       *
 *  Description    : Function to unregister call backs for KERNEL_RTP_INTERFACE   *
 *                   functionality.                                               *
 e  Input          : None.                                                        *
 *  Output         : None.                                                        *
 *  Return Value   : None.                                                        *
 *********************************************************************************/
void dsp_unregisterNetwork(void)
{
  pNWCallBack = NULL;
}
/* SAGEM -> [Export for RTP] { */
EXPORT_SYMBOL(dsp_unregisterNetwork);
/* } */
#endif

