/*======================================================================
 *
 * Copyright (C) 2000 Chiplogic Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive
 * property of Chiplogic and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 *======================================================================
 */

/*======================================================================
 * File Name	:	voiceopt.c
 * Description	:	functions for setting voice options.
 * CVS ID	:	$Id: voiceopt.c,v 1.3 2006/07/18 21:43:26 asingh Exp $
 *======================================================================
 */

/*======================================================================
 * History
 *
 * Date		Author			Comment
 * 10/23/2001   Malati Chavva       	Initial version
 *======================================================================
 */

/* include files
 */
#include<oswrapper.h>
#include <voiceopt.h>

/*======================================================================
 * Function Name	: setEncoderSpeechPktSize
 * Description  	: set the encoder speech pkt size with the given value 
 *======================================================================
 */
void setEncoderSpeechPktSize(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->encoder_opt[0], DSP_ENC_SPEECH_PKT_SIZE, value);
}

/*======================================================================
 * Function Name	: setEncoderAgc
 * Description  	: set the encoder AGC bit with given value
 *======================================================================
 */
void setEncoderAgc(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->encoder_opt[0], DSP_ENC_AGC_EN, value);
}

/*======================================================================
 * Function Name	: setEncoderDtmf
 * Description  	: set the encoder DTMF bit with given value
 *======================================================================
 */
void setEncoderDtmf(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->encoder_opt[0], DSP_ENC_DTMF_EN, value);
}

/*======================================================================
 * Function Name	: setEncoderSport
 * Description  	: set the encoder Sport type with given value
 *======================================================================
 */
void setEncoderSport(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->encoder_opt[0], DSP_ENC_SPORT_TYPE, value);
}

/*======================================================================
 * Function Name	: setEncoderType
 * Description  	: set the given encoder type
 *======================================================================
 */

void setEncoderType(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->encoder_opt[0], DSP_ENCODER_TYPE, value);
}

/*======================================================================
 * Function Name	: getEncoderType
 * Description  	: get the given encoder type
 *======================================================================
 */

unsigned char getEncoderType(VoiceOptions *vcopt)
{
    return GETFIELD(vcopt->encoder_opt[0], DSP_ENCODER_TYPE);
}

/*======================================================================
 * Function Name	: setEncoderActive
 * Description  	: set the encoder active/inactive bit with given value
 *======================================================================
 */
void setEncoderActive(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->encoder_opt[0], DSP_ENCODER_ACTIVE, value);
}

/*======================================================================
 * Function Name	: setEncoderG168Ec
 * Description  	: set the encoder G168_EC enable/ disable bit
 *======================================================================
 */
void setEncoderG168Ec(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->encoder_opt[1], DSP_ENC_G168_EC_EN, value);
}

/*======================================================================
 * Function Name	: setEncoderVadPktSize
 * Description  	: set the encoder VAD pkt size
 *======================================================================
 */
void setEncoderVadPktSize(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->encoder_opt[1], DSP_ENC_VAD_PACKET_SIZE, value);
}

/*======================================================================
 * Function Name	: setEncoderVadType
 * Description  	: set the encoder VAD type 
 *======================================================================
 */
void setEncoderVadType(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->encoder_opt[1], DSP_ENC_VAD_TYPE, value);
}

/*======================================================================
 * Function Name	: setEncoderG168TailLen
 * Description  	: set the encoder G168 tail length
 *======================================================================
 */
void setEncoderG168TailLen(VoiceOptions *vcopt, unsigned int value)
{
    SETFIELD(vcopt->encoder_opt[2], DSP_ENC_G168_TAIL_LENGTH, value);
}

/*======================================================================
 * Function Name	: setDecoderSpeechPktSize 
 * Description  	: set the decoder speech pkt size 
 *======================================================================
 */
void setDecoderSpeechPktSize(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->decoder_opt[0], DSP_DEC_SPEECH_PKT_SIZE, value);
}

/*======================================================================
 * Function Name	: setDecoderToneGen
 * Description  	: set the decoder Tone generation enable/ disable bit
 *======================================================================
 */
void setDecoderToneGen(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->decoder_opt[0], DSP_DEC_TONE_GEN_EN, value);
}

/*======================================================================
 * Function Name	: setDecoderPlc
 * Description  	: set the decoder PLC enable/disable bit
 *======================================================================
 */
void setDecoderPlc(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->decoder_opt[0], DSP_DEC_PLC_EN, value);
}

/*======================================================================
 * Function Name	: setDecoderSport
 * Description  	: set the decoder Sport type
 *======================================================================
 */
void setDecoderSport(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->decoder_opt[0], DSP_DEC_SPORT_TYPE, value);
}

/*======================================================================
 * Function Name	: setDecoderType  
 * Description  	: set the decoder type
 *======================================================================
 */
void setDecoderType(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->decoder_opt[0], DSP_DECODER_TYPE, value);
}

/*======================================================================
 * Function Name	: getDecoderType  
 * Description  	: get the decoder type
 *======================================================================
 */
unsigned char getDecoderType(VoiceOptions *vcopt)
{
    return GETFIELD(vcopt->decoder_opt[0], DSP_DECODER_TYPE);
}

/*======================================================================
 * Function Name	: setDecoderActive  
 * Description  	: set the decoder active/inactive bit 
 *======================================================================
 */
void setDecoderActive(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->decoder_opt[0], DSP_DECODER_ACTIVE, value);
}

/*======================================================================
 * Function Name	: setDecoderVadPktSize 
 * Description  	: set the decoder VAD pkt size with the given value
 *======================================================================
 */
void setDecoderVadPktSize(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->decoder_opt[1], DSP_DEC_VAD_PACKET_SIZE, value);
}

/*======================================================================
 * Function Name	: setDecoderVadCng  
 * Description  	: set the decoder VAD CNG
 *======================================================================
 */
void setDecoderVadCng(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->decoder_opt[1], DSP_DEC_VAD_CNG, value);
}


/*======================================================================
 * Function Name	: setDecoderVadCng  
 * Description  	: set the decoder VAD CNG
 *======================================================================
 */
void setDecoderVarGainCntl(VoiceOptions *vcopt, unsigned char value)
{
    SETFIELD(vcopt->decoder_opt[1], DSP_DEC_VAR_GAIN_CNTL, value);
}


// outband or  rfc2833 rejection
void setEncoderDtmfRej(VoiceOptions *vcopt, unsigned char value)
{
    SETBIT(vcopt->encoder_opt[1], DSP_ENC_DTMF_REJ_EN, value);
}


