

/***********************************************************************

  File Name : AddHist.c 

************************************************************************/
#include <oswrapper.h>
#include "G711PLCCons.h"
#include "G711PLCBasic_op.h"
#include "G711pLcProto.h"
#include "G711pLcVar.h"
extern Word16 plcincr;
#if G711_PLC

/**************************************************************************
Function Name	:   G711PLC_addtohistory()										  
Description  	:   This function will be called for good frame cases to update History Buffers	                
		    The functional flow has got two cases :
		     -  First Good frame after the  Erasure case. 
		     -  Good frames not following erase case.						 
     Case 1 : 
     - Get Synthetic speech samples from pitch Buffer of Length --> oLength
       into  overLapBuffer  
     - ovelap add both Current frame speech samples and samples in the overLapBuffer.                					           
     case 2 :
     -  Update History Buffer. 					  
Input   	:
			
		 - EraseCnt
		 - Pitch_overlap (  Pitch / 4 )
		 - pointer to present speech frame 80 samples  Buffer
Output 		: 														  
	         - After Updation of History Buffer
		   Current Frame speech  taken from History Buffer with 3.75ms add-end delay
		
Functions called  :
	           getfespeech
	           overlapaddatend
	           savespeech
***************************************************************************/

void  G711PLC_addtohistory()
{
	
	if ( EraseCnt != 0) 
	{
	/*************************************************************
             longer erasures require longer overlaps to smooth the 
	     transition between the synthetic  and real signal.
	**************************************************************/
		oLength = Pitch_overlap + (EraseCnt - 1) * EOVERLAPINCR; // EOVERLAPINCR = 32 i.e 4ms
			
		if (oLength > FRAME_LEN)  //--> if   ( olen > 10 ms )
		oLength = FRAME_LEN;
		/*******************************************************************
			   
		   |<-----------History Buffer------------------------------>|
					    |<-- Current Pitch Length------->|
				            |<--PitchOffset-->|<----->|
					                        olen
			   	Pitch_overlap = Pitch_Period / 4 ; 
			        Where Pitch_period is pitch cal in first lost frame.
	        *********************************************************************/
		BuffPtr1 = TempBuff;			
		Temp1_plc = oLength;
		getfespeech();
            //-->Three Arguemts 1-->Speech Buffer/2--> TempBuff/ 3--> Temp1
			
		Temp1_plc = oLength;
		overlapaddatend();
		EraseCnt = 0;    //--> Reset the Erase Count Value			
	}
	savespeech();
}

#endif
