
/************************************************************************

File Name : Pitch.c
Purpose   : To calculate Short Term Pitch 
  
************************************************************************/
#include <oswrapper.h>
#include "G711PLCCons.h"
#include "G711PLCBasic_op.h"
#include "G711pLcProto.h"
#include "G711pLcVar.h"

#if G711_PLC

/**************************************************************************
Function Name	:     findpitch
Description	:     Estimate the pitch.
			 l - pointer to first sample in last 20 ms of speech.
			 r - points to the sample PITCH_MAX before l	  
Input Parameters:													   
			none 
Return Values   :										  
			pitch													
Functions called :
			none
***************************************************************************/

Word16 findpitch()
{
   Word16 i,j,k;


/***************************************************************
        Noramalization of Pitch Buffer
*****************************************************************/
	     //--> CORR_BUFF_LEN  20 ms + PITCH_MAX ( 15 ms ) = 35ms = 280 Buffer
#ifdef NORMALISATION
		max = 0;
		for(i = 0;i < CORR_BUFF_LEN;i++)
		{
			temp[i] =  PitchBuffPtr[i+110];
			if ( max < abs_s_plc(temp[i]) )
                            max = temp[i];

		}
		normvalue = norm_s_plc(max);

		//vector normalisation of input

		/*
		for (i = 0; i< 280; i++)
		{
			temp[i] = shl( temp[i],(Word16) (normvalue-2) );
		}
		*/

		tempend = &temp[280];
		normvalue = 2* (norm_s_plc (max) - 2);  //-->added on 30'th/Oct
		
		BuffPtr1 = tempend - CORRLEN;       // --> temp  119  
		BuffPtr2 = tempend - CORRBUFLEN;	 // --> temp   0   
		//l = tempend - CORRLEN;       // --> temp  119  
		//r = tempend - CORRBUFLEN;	 // --> temp   0   
		//normvalue = 0;
#else

/****************************************************************

*****************************************************************/
		normvalue = 2* (norm_s_plc (max) - 2);  //-->added on 30'th/Oct
     
         

        	BuffPtr1 = PitchBuffPtr + HISTORYLEN - CORRLEN;
		BuffPtr2 = PitchBuffPtr + HISTORYLEN - CORRBUFLEN;
		
		//l = SMM_PitchBufEndPtr - CORRLEN;
		//r = SMM_PitchBufEndPtr - CORRBUFLEN;
#endif

		bestmatch = 0;
#if 1	
/***************************************************************
             Normalization of Pitch Buffer
*****************************************************************/
		BuffPtr3 = BuffPtr2;
		//rp = r;
		corr = 0;
		energy = 0;

		// Find current frame cross correlation and auto correlation energy.

		for (i = 0; i < CORRLEN; i += NDEC) 
		{
			//rp[i]  = 8192;
			energy = L_add_plc(energy, L_shr( L_mult_plc( BuffPtr3[i] , BuffPtr3[i]),(3-normvalue)));
			
			corr = L_add_plc(corr , L_shr( L_mult_plc( BuffPtr3[i] , BuffPtr1[i]),(3-normvalue)));
		}

		

		if (corr >0)		 //---> to take care of -ve corr
		{			
		    scale = energy;				            				 
			if ( scale < CORRMINPOWER )		
			    //scal16 = SQRT_CORRMINPOWER; //-->>
                // else
                scale = CORRMINPOWER; 
			    scal16 = Sqrt_lbc(scale);				  
			  
			    corr16 = extract_h(corr);			

			//---> to take care of division errors.
			if (corr16 < scal16)
				corr1 = div_s_plc(corr16, scal16);	 
			else
				corr1 = 32767;

		    bestcorr = corr1;	

		}
		else
		{
			bestcorr = 0;
		}
		
		/* finally assign 16-bit corr1 to bestcorr */
		
		bestmatch = 0;

/***************************************************************
			//----> Coarse Search
*****************************************************************/
		for (j = NDEC; j <= PITCHDIFF; j += NDEC)
		{
			// update autocorrelation energy of the frame scrolled by 0.25 ms
			energy = L_sub_plc(energy,L_shr(L_mult_plc(BuffPtr3[0],BuffPtr3[0]),(3-normvalue)));
							
			energy = L_add_plc(energy, L_shr( L_mult_plc(BuffPtr3[CORRLEN],BuffPtr3[CORRLEN]),(3-normvalue)));

			// scroll the frame by 0.25 ms
			BuffPtr3 += NDEC;
			corr = 0;

			// Find cross correlation energy 
			for (i = 0; i < CORRLEN; i += NDEC) 
				corr = L_add_plc(corr , L_shr( L_mult_plc( BuffPtr3[i] , BuffPtr1[i]), (3-normvalue)) );				
			

			//---> to take care of -ve corr
			if (corr >0)					
			{
				scale = energy;
				
				if (scale < CORRMINPOWER)
				//scal16 = SQRT_CORRMINPOWER;
				//else
				scale  = CORRMINPOWER;
				scal16 = Sqrt_lbc(scale);

				corr16 = extract_h(corr);		

				//---> to take care of division errors.
				if (corr16 > scal16)
					corr1 = 32767;

				else
					corr1 = div_s_plc(corr16, scal16);

				//--> Update bestcorr and bestmatch
				if (corr1 > bestcorr)
				{
					bestcorr = corr1;
					bestmatch = j;
				}

			}			
			
		}		/* end of j-loop */

/***************************************************************
			//----> End of Coarse Search
*****************************************************************/


/***************************************************************
			//----> Fine Search
*****************************************************************/
		j = bestmatch - (NDEC - 1);

		if (j < 0)
			j = 0;

		k = bestmatch + (NDEC - 1);

		if (k > PITCHDIFF) //--> 80
			k = PITCHDIFF;

		BuffPtr3 = BuffPtr2+j;
		energy = 0;
		corr = 0;

			// Find current frame cross correlation and auto correlation energy.
		for (i = 0; i < CORRLEN; i++)
		{
			energy = L_add_plc(energy,L_shr( L_mult_plc( BuffPtr3[i],BuffPtr3[i] ),(4-normvalue)));
			corr = L_add_plc(corr,L_shr( L_mult_plc( BuffPtr3[i],BuffPtr1[i]),(4-normvalue)));
		} 		

		
		//---> to take care of -ve corr
		if (corr >0)
		{
			
			scale = energy;
			
			if (scale < CORRMINPOWER)
			 //scal16 = SQRT_CORRMINPOWER;
             //else
			 scale = CORRMINPOWER;
			 scal16 = Sqrt_lbc(scale);
			
			 corr16 = extract_h(corr);			

			//---> to take care of division errors.
			if (corr16 > scal16)
				 corr1 = 32767;
			else
				corr1 = div_s_plc(corr16, scal16);
            
			bestcorr = corr1;		    
		}
		else
		{
			bestcorr = 0;
		}	

		bestmatch = j;

			/* second sample search */

		for (j++; j <= k; j++)
		{

				// update autocorrelation energy of the frame scrolled by 0.125 ms
			energy = L_sub_plc(energy,L_shr( L_mult_plc( BuffPtr3[0],BuffPtr3[0]),(4-normvalue)));
			energy = L_add_plc(energy, L_shr( L_mult_plc( BuffPtr3[CORRLEN],BuffPtr3[CORRLEN]),(4-normvalue)));
			BuffPtr3++;
			corr = 0;

			for (i = 0; i < CORRLEN; i++)
				corr = L_add_plc(corr,L_shr( L_mult_plc( BuffPtr3[i],BuffPtr1[i]) ,(4-normvalue)));						

				//---> to take care of -ve corr
			if (corr >0)
			{				
                scale = energy;

				if (scale < CORRMINPOWER)
				 // scal16 = SQRT_CORRMINPOWER;
				//else
				  scale =  CORRMINPOWER;
				  scal16 = Sqrt_lbc(scale);

				corr16 = extract_h(corr);		

				//---> to take care of division errors.
				if (corr16 > scal16)
					corr1 = 32767;
				else
					corr1 = div_s_plc(corr16, scal16);			

			   if (corr1 > bestcorr)
			   {
			    	bestcorr = corr1;
			 	    bestmatch = j;
			   }
			}
		}		
#endif
/***************************************************************
			//----> End of Fine Search
*****************************************************************/
		return PITCH_MAX - bestmatch;
}


#endif
