/******************************************************************************
	File Name	    :   Tonegenparam.h
	Description     :   This file contains look up tables that are used in 
			            Tone Generation module
	Version Number	:	1.0	
*****************************************************************************/
#include "ToneGenCons.h"

#if TONE_GEN

/**********************************************************************
    The parameters for each tone are stored in the following format
    {ON Time - OFF Time - Phase1 - Phase2 - Amp1 - Amp2}
	
    Phase1,Phase2 are phase values of each DUAL tone
    Formula for Phase = (65536*F/Fs)  (q.15 format conversion)
    Where Fs = 8000 Hz Sampling Rate 
          F  = frequency of the tone

    Amplitude is given as Amp = 

	ToneArray[MAX_NUM_COUNTRY][][]
	{
          {USA};
	  {FRANCE};
	  {KOREA};
	  {JAPAN};
	  {ROMANIA};
	}
***********************************************************************/
//Word16 ToneArray[MAX_NUM_COUNTRY][MAX_NUM_TONES][6] = {
Word16 ToneArray[5][MAX_NUM_TONES][6] = {
  {
	//US Country Tones specifications
	{0xFFF, 0x0, 0xb33, 0xe14, 0x400, 0x400}, //DialBuff[]  //continuous
	{0x7D0, 0xFA0, 0xe14, 0xf5c, 0x400, 0x400}, //RingBackBuff[]
	{0x1F4, 0x1F4, 0xf5c, 0x13d7, 0x400, 0x400}, //Busy1Buff[]
	{0xFA, 0xFA, 0xf5c, 0x13d7, 0x400, 0x400}, //Busy2Buff[]
	{0x12c,0x12c,0xe14,0,0x800,0},		//CallWaitingBuff[] //0.3 ON, 0.3 OFF
	{0xFA,0xFA,0xf5c,0x13d7,0x200,0x200}, 	//ReorderBuff[]
	{0x64,0x64,0xb33,0xe14,0x400,0x400}, 	//StutterBuff[]
	{0xFA,0xFA,0x2dda,0x5041,0x400,0x400}, //OffHookWarnBuff[]
	{0x01f4,0x0,0x0ae8,0x15d8,0x200,0x200}, //MelodyToneBuff1[]
	{0x01f4,0x0,0x2bc8,0x5800,0x200,0x200}, //MelodyToneBuff2[]
	{0x01f4,0x0,0x0dc0,0x41c0,0x200,0x200}, //MelodyToneBuff3[]
	{0x01f4,0x0,0x3130,0x62e0,0x200,0x200}, //MelodyToneBuff4[]
	{0x01f4,0x0,0x49e0,0x5dc0,0x200,0x200}, //MelodyToneBuff5[]
	{0x01f4,0x0,0x0dc0,0x1b88,0x200,0x200}, //MelodyToneBuff6[]

	//DTMF Tone Specifications
	//-->SPEC:-<LOW:-13dBV to -8.5dBV>-<HIGH:-11.5 to -7dBV>-<TW: 1-4 >
        //-->------<LOW:-11dBV----------->-<HIGH:-9dBV >-<TW:2 >-----------
        {0x50,0x50,0x1e1c,0x2ac0,0xCB3,0xFFD}, //Word16 Dig0Buff[6]	//941 * 1336
	{0x50,0x50,0x164d,0x26b0,0xCB3,0xFFD}, //Word16 Dig1Buff[6] //697 * 1209 
	{0x50,0x50,0x164d,0x2ac0,0xCB3,0xFFD}, //Word16 Dig2Buff[6]	//697 * 1336
	{0x50,0x50,0x164d,0x2f43,0xCB3,0xFFD}, //Word16 Dig3Buff[6]	//697 * 1477
	{0x50,0x50,0x18a3,0x26b0,0xCB3,0xFFD}, //Word16 Dig4Buff[6] //770 * 1209
	{0x50,0x50,0x18a3,0x2ac0,0xCB3,0xFFD}, //Word16 Dig5Buff[6]	//770 * 1336
	{0x50,0x50,0x18a3,0x2f43,0xCB3,0xFFD}, //Word16 Dig6Buff[6]	//770 * 1477
	{0x50,0x50,0x1b43,0x26b0,0xCB3,0xFFD}, //Word16 Dig7Buff[6]	//852 * 1209
	{0x50,0x50,0x1b43,0x2ac0,0xCB3,0xFFD}, //Word16 Dig8Buff[6]	//852 * 1336
	{0x50,0x50,0x1b43,0x2f43,0xCB3,0xFFD}, //Word16 Dig9Buff[6]	//852 * 1477

	{0x50,0x50,0x164d,0x3441,0xCB3,0xFFD}, //Word16 DigABuff[6]	//697 * 1633
	{0x50,0x50,0x18a3,0x3441,0xCB3,0xFFD}, //Word16 DigBBuff[6]	//770 * 1633
	{0x50,0x50,0x1b43,0x3441,0xCB3,0xFFD}, //Word16 DigCBuff[6]	//852 * 1633
	{0x50,0x50,0x1e1c,0x3441,0xCB3,0xFFD}, //Word16 DigDBuff[6]	//941 * 1633

	{0x50,0x50,0x1e1c,0x26b0,0xCB3,0xFFD}, //Word16 DigStarBuff[6]//941 * 1209
	{0x50,0x50,0x1e1c,0x2f43,0xCB3,0xFFD}, //Word16 DigHashBuff[6]//941 * 1477
	
	//---->80ms cont, 2130Hz and 2750Hz, -8dbm
	{0x50,0x0,0x4429,0x57ff,0x850,0x850} //Word16 CASBuff[6]

   },	//end of US Tones
	
   {
	//France Country Tones specifications
	{0xfff ,0x0,0xe14,0x0,0xE00,0x0}, //DialBuff[6]   //continuous
	{0x5dc,0xdac,0xe14,0x0,0x850,0x0},//RingBackBuff[6]
	{0x1f4,0x1f4,0xe14,0x0,0x850,0x0}, //Busy1Buff[6]
	{0x1f4,0x1f4,0xe14,0x0,0x850,0x0}, //ErrorToneBuff[6]
	{0x12c,0x0,0xe14,0x0,0x8cf,0x0}, //CallWaitingBuff[6]
//	{0,0,0,0,0,0}, //dummy
	{0xfff,0xfff,0x164d,0x26b0,0x659,0x7FE}, //Dig1Buff[6] //697 * 1209 Continous
	{0xfff,0x0,0xe14,0xa8f,0x850,0xc71}, //SpecialDialBuff[6]  //continuous
//	{0xFA, 0xFA, 0x2dda,0x5041,0x400,0x400 }, //Word16 OffHookWarnBuff[6]
	{0x1f4,0x1f4,0xe14,0x0,0x850,0x0}, //OffHookWarnBuff[6]

	/////// 0x8 - 0xd Melody Tone
	/////// 0xe - 0x1e DTMF Tones, CPE Alerting Tone
	{0x01f4,0x0,0x0ae8,0x15d8,0x200,0x200}, //MelodyToneBuff1[]
	{0x01f4,0x0,0x2bc8,0x5800,0x200,0x200}, //MelodyToneBuff2[]
	{0x01f4,0x0,0x0dc0,0x41c0,0x200,0x200}, //MelodyToneBuff3[]
	{0x01f4,0x0,0x3130,0x62e0,0x200,0x200}, //MelodyToneBuff4[]
	{0x01f4,0x0,0x49e0,0x5dc0,0x200,0x200}, //MelodyToneBuff5[]
	{0x01f4,0x0,0x0dc0,0x1b88,0x200,0x200}, //MelodyToneBuff6[]
	//DTMF Tone Specifications
	//-->SPEC:-<LOW:-13dBV to -8.5dBV>-<HIGH:-11.5 to -7dBV>-<TW: 1-4 >
        //-->------<LOW:-11dBV----------->-<HIGH:-9dBV >-<TW:2 >-----------
	{0x50,0x50,0x1e1c,0x2ac0,0x659,0x7FE}, //Word16 Dig0Buff[6]	//941 * 1336
	{0x50,0x50,0x164d,0x26b0,0x659,0x7FE}, //Word16 Dig1Buff[6] //697 * 1209 
	{0x50,0x50,0x164d,0x2ac0,0x659,0x7FE}, //Word16 Dig2Buff[6]	//697 * 1336
	{0x50,0x50,0x164d,0x2f43,0x659,0x7FE}, //Word16 Dig3Buff[6]	//697 * 1477
	{0x50,0x50,0x18a3,0x26b0,0x659,0x7FE}, //Word16 Dig4Buff[6] //770 * 1209
	{0x50,0x50,0x18a3,0x2ac0,0x659,0x7FE}, //Word16 Dig5Buff[6]	//770 * 1336
	{0x50,0x50,0x18a3,0x2f43,0x659,0x7FE}, //Word16 Dig6Buff[6]	//770 * 1477
	{0x50,0x50,0x1b43,0x26b0,0x659,0x7FE}, //Word16 Dig7Buff[6]	//852 * 1209
	{0x50,0x50,0x1b43,0x2ac0,0x659,0x7FE}, //Word16 Dig8Buff[6]	//852 * 1336
	{0x50,0x50,0x1b43,0x2f43,0x659,0x7FE}, //Word16 Dig9Buff[6]	//852 * 1477

	{0x50,0x50,0x164d,0x3441,0x659,0x7FE}, //Word16 DigABuff[6]	//697 * 1633
	{0x50,0x50,0x18a3,0x3441,0x659,0x7FE}, //Word16 DigBBuff[6]	//770 * 1633
	{0x50,0x50,0x1b43,0x3441,0x659,0x7FE}, //Word16 DigCBuff[6]	//852 * 1633
	{0x50,0x50,0x1e1c,0x3441,0x659,0x7FE}, //Word16 DigDBuff[6]	//941 * 1633

	{0x50,0x50,0x1e1c,0x26b0,0x659,0x7FE}, //Word16 DigStarBuff[6]//941 * 1209
	{0x50,0x50,0x1e1c,0x2f43,0x659,0x7FE}, //Word16 DigHashBuff[6]//941 * 1477

	//---->80ms cont, 2130Hz and 2750Hz, -8dbm
	{0x3c,0x0,0x4429,0x57ff,0x7da,0x7da}, //Word16 CASBuff[6]

	////////////////////////////////////////////////
	{0x12c,0x1e,0x1e66,0x0,0x850,0x0}, //SplInfoToneA[6]
	{0x12c,0x1e,0x2ccc,0x0,0x850,0x0},//SplInfoToneB[6]
	{0x12c,0x3e8,0x3999,0x0,0x850,0x0}, //SplInfoToneC[6]
	
	{0x1f4,0x1f4,0xe14,0x11ba,0x8cf,0x382},//RocToneA[6]
	{0x1f4,0x1f4,0xe14,0x179d,0x8cf,0x382},//RocToneB[6]
	{0x1f4,0x1f4,0xe14,0x1516,0x8cf,0x382},//RocToneC[6]
	{0x1f4,0x1f4,0xe14,0x0fce,0x8cf,0x382}, //RocToneD[6]
	//Added from us 
	

	//------>Peak is taken as -8dbm
	{0x28,0, 0xe14,0xa8f,0x850,0x16F0}, //Word16  MesgWatingIndicator1[6]
	{0x8c,0, 0xe14,0x0,0x850,0x0}, //Word16  MesgWatingIndicator2[6]
	{0x122,0, 0xe14,0x0,0x850,0x0}, //Word16  MesgWatingIndicator3[6]

	{0x1f4,0, 0xe14,0x0,0x500,0x0}, //Word16  VOIP_DIAL_TONEA[6]
	{0xfa,0, 0x2000,0x0,0x500,0x0}, //Word16  VOIP_DIAL_TONEB[6]
	{0xfa,0, 0x17AE,0x0,0x500,0x0}, //Word16  VOIP_DIAL_TONEC[6]
	{0x7d0,0, 0xe14,0x0,0x500,0x0},//Word16  VOIP_DIAL_TONED[6]

	{0xfff,0, 0xe14,0x0,0x500,0x0} //Word16  TEST_TONE[6]

   },	//end of France Tones

   {
        //Korea Country Tones specifications
        {0xFFF, 0x0, 0xe14, 0xf5c, 0x2d8, 0x2d8}, //Word16 DialBuff[] /* 440+480,continuos, -15 dBm0 */
        {0x3e8, 0x7d0, 0xe14, 0xf5c, 0x2d8, 0x2d8}, //Word16 RingBackBuff[]     /* 440+480,1 ON - 2 OFF, -15 dBm0 */
        {0x1f4, 0x1f4, 0xf5c, 0x13d7, 0x199, 0x199}, //Word16 BusyBuff[] /* 480+620,0.5 ON - 0.5 OFF, -20 dBm0 */
        {0x12c, 0xc8, 0xf5c, 0x13d7, 0x199, 0x199},//Word16 CongestBuff[]/* 480+620,0.3 ON - 0.2 OFF, -20 dBm0 */
        {0xfa, 0xfa, 0xB33, 0xe14, 0x50f, 0x50f},//Word16 CallWaitingBuff1[]    /* 350+440,0.25 ON - 0.25 OFF, -10 dBm0 */
        {0xfa, 0xcb2, 0xB33, 0xe14, 0x50f, 0x50f},//Word16 CallWaitingBuff2[]/* 350+440,0.25 ON - 3.25 OFF, -10 dBm0 */
        {0x3e8, 0xfa,0xB33, 0xe14, 0x50f, 0x50f}, //Word16 InternDialBuff[]     /* 350+440,1 ON -0.25 OFF, -10 dBm0 */
        {0xFFF, 0x0, 0xB33, 0xe14, 0x50f, 0x50f}, //Word16 ImmRingBackBuff[]/* 350+440,continuous, -10 dBm0 */
        //////// 0x8 - 0xd Melody Tone
        //////// 0xe - 0x1e DTMF Tones, CPE Alerting Tone
        {0x01f4,0x0,0x0ae8,0x15d8,0x200,0x200}, //MelodyToneBuff1[]
        {0x01f4,0x0,0x2bc8,0x5800,0x200,0x200}, //MelodyToneBuff2[]
        {0x01f4,0x0,0x0dc0,0x41c0,0x200,0x200}, //MelodyToneBuff3[]
        {0x01f4,0x0,0x3130,0x62e0,0x200,0x200}, //MelodyToneBuff4[]
        {0x01f4,0x0,0x49e0,0x5dc0,0x200,0x200}, //MelodyToneBuff5[]
        {0x01f4,0x0,0x0dc0,0x1b88,0x200,0x200}, //MelodyToneBuff6[]
        //DTMF Tone Specifications
        //-->SPEC:-<LOW:-13dBV to -8.5dBV>-<HIGH:-11.5 to -7dBV>-<TW: 1-4 >
        //-->------<LOW:-11dBV----------->-<HIGH:-9dBV >-<TW:2 >-----------
	{0x50,0x50,0x1e1c,0x2ac0,0xCB3,0xFFD}, //Word16 Dig0Buff[6]     //941 * 1336
        {0x50,0x50,0x164d,0x26b0,0xCB3,0xFFD}, //Word16 Dig1Buff[6] //697 * 1209
        {0x50,0x50,0x164d,0x2ac0,0xCB3,0xFFD}, //Word16 Dig2Buff[6]     //697 * 1336
        {0x50,0x50,0x164d,0x2f43,0xCB3,0xFFD}, //Word16 Dig3Buff[6]     //697 * 1477
        {0x50,0x50,0x18a3,0x26b0,0xCB3,0xFFD}, //Word16 Dig4Buff[6] //770 * 1209
        {0x50,0x50,0x18a3,0x2ac0,0xCB3,0xFFD}, //Word16 Dig5Buff[6]     //770 * 1336
        {0x50,0x50,0x18a3,0x2f43,0xCB3,0xFFD}, //Word16 Dig6Buff[6]     //770 * 1477
        {0x50,0x50,0x1b43,0x26b0,0xCB3,0xFFD}, //Word16 Dig7Buff[6]     //852 * 1209
        {0x50,0x50,0x1b43,0x2ac0,0xCB3,0xFFD}, //Word16 Dig8Buff[6]     //852 * 1336
        {0x50,0x50,0x1b43,0x2f43,0xCB3,0xFFD}, //Word16 Dig9Buff[6]     //852 * 1477

	{0x50,0x50,0x164d,0x3441,0xCB3,0xFFD}, //Word16 DigABuff[6]     //697 * 1633
        {0x50,0x50,0x18a3,0x3441,0xCB3,0xFFD}, //Word16 DigBBuff[6]     //770 * 1633
        {0x50,0x50,0x1b43,0x3441,0xCB3,0xFFD}, //Word16 DigCBuff[6]     //852 * 1633
        {0x50,0x50,0x1e1c,0x3441,0xCB3,0xFFD}, //Word16 DigDBuff[6]     //941 * 1633

        {0x50,0x50,0x1e1c,0x26b0,0xCB3,0xFFD}, //Word16 DigStarBuff[6]//941 * 1209
        {0x50,0x50,0x1e1c,0x2f43,0xCB3,0xFFD}, //Word16 DigHashBuff[6]//941 * 1477

        //---->80ms cont, 2130Hz and 2750Hz, -8dbm
        {0x50,0x0,0x4429,0x57ff,0x850,0x850}, //Word16 CASBuff[6]
        /////////////////////////////////////////////////

        {0x1f4, 0x1f4, 0xe14, 0xf5c, 0x50f, 0x50f},//Word16 CallHoldBuff1[]/* 440+480,0.5 ON - 0.5 OFF, -10 dBm0 */
        {0x1f4, 0x9c4, 0xB33, 0xe14, 0x50f, 0x50f},//Word16 CallHoldBuff2[] /* 350+440,0.5 ON - 2.5 OFF, -10 dBm0 */
        {0x7d, 0xfa, 0xB33, 0xe14, 0x50f, 0x50f},//Word16 InterceptBuff1[]/* 350+440,0.125 ON - 0.25 OFF, -10 dBm0 */
        {0x7d, 0x5dc, 0xB33, 0xe14, 0x50f, 0x50f},//Word16 InterceptBuff2[]/* 350+440,0.125 ON - 1.5 OFF, -10 dBm0 */
        {0xFFF, 0x0, 0x12c8, 0x0, 0xA1e, 0x0},  //Word16 BitBuff[]/* 587,Continuous, -10 dBm0 */
        {0x3e8, 0x3e8, 0xe14, 0xf5c, 0x50f, 0x50f},//Word16 ReserveBuff[] /* 440+480, 1 ON - 1 OFF, -10 dBm0 */
        {0x12c, 0xc8, 0xf5c, 0x13d7, 0x199, 0x199},//Word16 FailureBuff[]/* 480+620, 0.3 ON - 0.2 OFF, -20 dBm0 */
        {0xfa, 0xfa, 0xFCE, 0x0, 0xA1e, 0x0},   //Word16 HoldReleaseBuff[]/* 494, 0.25 ON - 0.25 ON, -10 dBm0 */
        {0x64, 0x64, 0x2ccc, 0x41eb, 0x392, 0x392},//Word16 HowlerBuff1[]/* 1400+2060,0.1 ON - 0.1 OFF, -7dBm0 */
        {0x64, 0x64, 0x4e66, 0x5333, 0x392, 0x392},//Word16 HowlerBuff2[]/* 2450+2600,0.1 ON - 0.1 OFF, -7dBm0 */

        /* Yet to be Implemented
        1. Confirm Tone         --> ON-OFF Times not clearly mentioned
        2. Auto Answer Tone --> ON-OFF Times not mentioned (given in Korean language)
        */
        {0,0,0,0,0,0}, //ConfirmTone1
        {0,0,0,0,0,0}, //ConfirmTone2
        {0,0,0,0,0,0}, //ConfirmTone3
        {0,0,0,0,0,0} //Auto Answer Tone

   },    //end of Korea Tones
	
   {
	//Japan Country Tones specifications
	/* ON Time -- OFF Time -- Phase1 -- Phase2 -- Amp1 -- Amp2 */
	{0xFFF, 0x0, 0xccc, 0x0,0x28a,0x0}, //Word16 DialBuff[]  	/* 400Hz,continuos,   -22 dBm0 */
	{0x3e8,0x7d0,0xd4f,0xc49,0x91,0x91}, //Word16 RingingBuff[]	/* 400*16Hz(400+16,400-16),1.0 ON,2.0 OFF, -29 dBm0 */
	{0x1f4,0x1f4,0xccc,0x0,0x122,0x0}, //Word16 BusyBuff[]	/* 400Hz, 0.5 ON, 0.5 ON, -29 dBm0 */
	{0x1f4,0x1f4,0xccc,0x0,0x122,0x0}, //Word16 CongestionBuff[]/* 400Hz, 0.5 ON, 0.5 ON, -29 dBm0 */
	/* 400Hz,0.5 ON, 0.0~4.0 OFF, 0.05 ON,0.45 OFF,0.05 ON,3.45 OFF,0.05 ON,0.45 OFF,0.05 ON,3.45 OFF, -22 dBm0 */
	{0x1f4,0xfa0,0xccc,0x0,0x122,0x0}, //Word16 CallWaitingBuff_I_1[]
	{0x32,0x1c2,0xccc,0x0,0x122,0x0}, //Word16 CallWaitingBuff_I_2[]
	{0x32,0xd7a,0xccc,0x0,0x122,0x0}, //Word16 CallWaitingBuff_I_3[]
	{0xFA,0xFA,0xccc,0x0,0x28a,0x0}, //Word16 DialBuffPABX[]   	/* 400Hz,0.25 ON , 0.25 OFF, -22 dBm0  */
	{0x01f4,0x0,0x0ae8,0x15d8,0x200,0x200}, //Word16 MelodyToneBuff1[]
	{0x01f4,0x0,0x2bc8,0x5800,0x200,0x200}, //Word16 MelodyToneBuff2[]
	{0x01f4,0x0,0x0dc0,0x41c0,0x200,0x200}, //Word16 MelodyToneBuff3[]
	{0x01f4,0x0,0x3130,0x62e0,0x200,0x200}, //Word16 MelodyToneBuff4[]
	{0x01f4,0x0,0x49e0,0x5dc0,0x200,0x200}, //Word16 MelodyToneBuff5[]
	{0x01f4,0x0,0x0dc0,0x1b88,0x200,0x200}, //Word16 MelodyToneBuff6[]

	//DTMF Tone Specifications
	//-->SPEC:-<LOW:-13dBV to -8.5dBV>-<HIGH:-11.5 to -7dBV>-<TW: 1-4 >
        //-->------<LOW:-11dBV----------->-<HIGH:-9dBV >-<TW:2 >-----------
	{0x50,0x50,0x1e1c,0x2ac0,0xCB3,0xFFD}, //Word16 Dig0Buff[6]	//941 * 1336
	{0x50,0x50,0x164d,0x26b0,0xCB3,0xFFD}, //Word16 Dig1Buff[6] //697 * 1209 
	{0x50,0x50,0x164d,0x2ac0,0xCB3,0xFFD}, //Word16 Dig2Buff[6]	//697 * 1336
	{0x50,0x50,0x164d,0x2f43,0xCB3,0xFFD}, //Word16 Dig3Buff[6]	//697 * 1477
	{0x50,0x50,0x18a3,0x26b0,0xCB3,0xFFD}, //Word16 Dig4Buff[6] //770 * 1209
	{0x50,0x50,0x18a3,0x2ac0,0xCB3,0xFFD}, //Word16 Dig5Buff[6]	//770 * 1336
	{0x50,0x50,0x18a3,0x2f43,0xCB3,0xFFD}, //Word16 Dig6Buff[6]	//770 * 1477
	{0x50,0x50,0x1b43,0x26b0,0xCB3,0xFFD}, //Word16 Dig7Buff[6]	//852 * 1209
	{0x50,0x50,0x1b43,0x2ac0,0xCB3,0xFFD}, //Word16 Dig8Buff[6]	//852 * 1336
	{0x50,0x50,0x1b43,0x2f43,0xCB3,0xFFD}, //Word16 Dig9Buff[6]	//852 * 1477

	{0x50,0x50,0x164d,0x3441,0xCB3,0xFFD}, //Word16 DigABuff[6]	//697 * 1633
	{0x50,0x50,0x18a3,0x3441,0xCB3,0xFFD}, //Word16 DigBBuff[6]	//770 * 1633
	{0x50,0x50,0x1b43,0x3441,0xCB3,0xFFD}, //Word16 DigCBuff[6]	//852 * 1633
	{0x50,0x50,0x1e1c,0x3441,0xCB3,0xFFD}, //Word16 DigDBuff[6]	//941 * 1633

	{0x50,0x50,0x1e1c,0x26b0,0xCB3,0xFFD}, //Word16 DigStarBuff[6]//941 * 1209
	{0x50,0x50,0x1e1c,0x2f43,0xCB3,0xFFD}, //Word16 DigHashBuff[6]//941 * 1477
	
	//---->80ms cont, 2130Hz and 2750Hz, -8dbm
	{0x50,0x0,0x4429,0x57ff,0x850,0x850}, //Word16 CASBuff[6]
	/////////////////////////////////////////////////


	{0x7d,0x7d,0xccc,0x0,0x28a,0x0}, //Word16 SecondDialBuff_I[]  	/* 400Hz,0.125 ON, 0.125 OFF,-22 dBm0 */
	{0x7d,0x7d,0xe14,0x0,0x28a,0x0}, //Word16 SecondDialBuff_II[] 	/* 440Hz,0.125 ON, 0.125 OFF,-22 dBm0 */
	{0x96,0x96,0xccc,0x0,0x28a,0x0}, //Word16 SecondDialBuff_III[]	/* 400Hz,0.15 ON ,0.15 OFF, -22 dBm0 */
	{0xfff,0x0,0xb33,0xe14,0x145,0x145}, //RecallDialBuff[]	/* 350+440Hz,continuous, -22 dBm0 */		
	{0x3e8,0x7d0,0xd70,0xc28,0x91,0x91}, //RingingBuffPABX[]/* 400*20Hz(400+20,400-20),1.0 ON,2.0 OFF, -29 dBm0 */
	{0x64,0x64,0xccc,0x0,0x28a,0x0}, //SpecialInfoBuff[]/* 400Hz, 0.1 ON, 0.1 OFF, -22 dBm0 */

	{0x64,0x64,0xe14,0x0,0x28a,0x0}, //Word16 WaitingBuff_I_1[]	/* 440Hz,0.1ON,0.1OFF,0.1ON,1.0OFF,-22 dBm0*/
	{0x64,0x3e8,0xe14,0x0,0x28a,0x0}, //Word16 WaitingBuff_I_2[]/* 440Hz, 0.1 ON, 0.1 OFF, 0.1 ON, 1.0 OFF, -22 dBm0*/

	{0x1f4,0x1f4,0xd4f,0xc49,0x91,0x91}, //WaitingBuff_II_1[] /* 400*16/400Hz,0.5ON,0.5OFF,0.5ON,2.5OFF,-29 dBm0 */
	{0x1f4,0x9c4,0xd4f,0xc49,0x91,0x91}, //WaitingBuff_II_2[]/* 400*16/400Hz,0.5ON,0.5OFF,0.5ON,2.5OFF,-29 dBm0 */

	{0x1f4,0x1f4,0xccc,0x0,0x122,0x0},//WaitingBuff_II_3[] /* 400*16/400Hz,0.5ON,0.5OFF,0.5ON,2.5OFF,-29 dBm0 */
	{0x1f4,0x9c4,0xccc,0x0,0x122,0x0},//WaitingBuff_II_4[] /* 400*16/400Hz,0.5ON,0.5OFF,0.5ON,2.5OFF,-29 dBm0 */


	/* 400x16Hz,0.5 ON, 0.0~4.0 OFF, 0.05 ON,0.45 OFF,0.05 ON,3.45 OFF,0.05 ON,0.45 OFF,0.05 ON,3.45 OFF, -22 dBm0 */
	{0x1f4,0xfa0,0xd4f,0xc49,0x91,0x91},//Word16 CallWaitingBuff_II_1[]
	{0x32,0x1c2,0xd4f,0xc49,0x91,0x91}, //Word16 CallWaitingBuff_II_2[]
	{0x32,0xd7a,0xd4f,0xc49,0x91,0x91}, //Word16 CallWaitingBuff_II_3[]

	{0x3e8,0x0,0x800,0x0,0x28a,0x0}, //PayToneBuff_I[]/* 250Hz, 1.0 ON , -22 dBm0*/
	{0x1f4,0x0,0x800,0x0,0x28a,0x0}, // PayToneBuff_II[]/* 250Hz, 0.5 ON , -22 dBm0*/

	{0x7d,0x7d,0xccc,0x0,0x28a,0x0}, //PositiveIndBuff_1[]/* 400Hz,0.125 ON, 0.125 OFF, 0.125 ON, 0.625 OFF, -22 dBm0 */
	{0x7d,0x271,0xccc,0x0,0x28a,0x0} //PositiveIndBuff_2[]/* 400Hz,0.125 ON, 0.125 OFF, 0.125 ON, 0.625 OFF, -22 dBm0 */

   },	//end of Japan Tones

   {
	// ROMANIA Country Tones specifications
	{0xfff, 0x0, 0xe66, 0x0, 0xe04, 0x00}, //DialBuff[]  //continuous
	{ 0x96, 0x15e,0xe66, 0x0, 0xe04, 0x00}, //RingBackBuff[]
	{ 0x32, 0x32,0xe66, 0x0, 0xe04, 0x00},  //Busy1Buff[]
	{ 0xa7, 0xa7,0xe66, 0x0, 0xe04, 0x00}, //congestion tone
	{0xaf,0xaf,0x2ccc,0x0,0x3f3,0x00}, //CallwaitBuff1
	{0xaf,0x15e,0x2ccc,0x0,0x3f3,0x00}, //CallwaitBuff2
		
	{ 0x64, 0x64, 0xe66, 0x0,0xe04, 0x00},	 //Speacialdialtonebuff1
	{ 0x64, 0x64, 0xe66, 0x0,0xe04, 0x00},	 //Speacialdialtonebuff2
       { 0x64, 0x64, 0xe66, 0x0,0xe04, 0x00},	 //Speacialdialtonebuff3
	{ 0xfff, 0x0, 0xe66, 0x0,0xe04, 0x00},       //Speacialdialtonebuff4
	
	{ 0x28, 0x28, 0xe66, 0x0,0xe04, 0x00}, 	//Special confirmation Tone
	
	{0x14a,0x3e8,0x1686,0x2ccc,0xe14,0xe14},//Special information Tone
	{0x14a,0x3e8,0x3999,0x0,0xe14,0xe14},
	

	//DTMF Tone Specifications
	//-->SPEC:-<LOW:-13dBV to -8.5dBV>-<HIGH:-11.5 to -7dBV>-<TW: 1-4 >
        //-->------<LOW:-11dBV----------->-<HIGH:-9dBV >-<TW:2 >-----------
	{0x50,0x50,0x164d,0x26b0,0xCB3,0xFFD}, //Word16 Dig1Buff[6] //697 * 1209 
	{0x50,0x50,0x164d,0x2ac0,0xCB3,0xFFD}, //Word16 Dig2Buff[6]	//697 * 1336
	{0x50,0x50,0x164d,0x2f43,0xCB3,0xFFD}, //Word16 Dig3Buff[6]	//697 * 1477
	{0x50,0x50,0x164d,0x3441,0xCB3,0xFFD}, //Word16 DigABuff[6]	//697 * 1633

	{0x50,0x50,0x18a3,0x26b0,0xCB3,0xFFD}, //Word16 Dig4Buff[6] //770 * 1209
	{0x50,0x50,0x18a3,0x2ac0,0xCB3,0xFFD}, //Word16 Dig5Buff[6]	//770 * 1336
	{0x50,0x50,0x18a3,0x2f43,0xCB3,0xFFD}, //Word16 Dig6Buff[6]	//770 * 1477
	{0x50,0x50,0x18a3,0x3441,0xCB3,0xFFD}, //Word16 DigBBuff[6]	//770 * 1633

	{0x50,0x50,0x1b43,0x26b0,0xCB3,0xFFD}, //Word16 Dig7Buff[6]	//852 * 1209
	{0x50,0x50,0x1b43,0x2ac0,0xCB3,0xFFD}, //Word16 Dig8Buff[6]	//852 * 1336
	{0x50,0x50,0x1b43,0x2f43,0xCB3,0xFFD}, //Word16 Dig9Buff[6]	//852 * 1477
	{0x50,0x50,0x1b43,0x3441,0xCB3,0xFFD}, //Word16 DigCBuff[6]	//852 * 1633

	{0x50,0x50,0x1e1c,0x26b0,0xCB3,0xFFD}, //Word16 DigStarBuff[6]//941 * 1209
	{0x50,0x50,0x1e1c,0x2ac0,0xCB3,0xFFD}, //Word16 Dig0Buff[6]	//941 * 1336
	{0x50,0x50,0x1e1c,0x2f43,0xCB3,0xFFD}, //Word16 DigHashBuff[6]//941 * 1477
	{0x50,0x50,0x1e1c,0x3441,0xCB3,0xFFD}, //Word16 DigDBuff[6]	//941 * 1633
	//---->80ms cont, 2130Hz and 2750Hz, -8dbm
	{0x50,0x0,0x4429,0x57ff,0x850,0x850} //Word16 CASBuff[6]

   }	//end of Romania Tones

};

#endif //end of #ifdef TONE_GEN


