/* ============================================================================= *
 * Copyright (C) 2004 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ===========================================================================*/                                                                                

/*********************************************************************
*	File Name	: processor.h				     *
*	Description	: This file contains the addresses of the    *
*                         Fusiv registers used in voice drivers.     *
*********************************************************************/

#ifndef __VOICE_PROCESSOR_H
#define __VOICE_PROCESSOR_H


/*########################################################################################*/
/* GPIO Register */

#define GPIO_DIR_REG            K0_TO_K1(0x19040000)    /* Direction Register           */
#define GPIO_DIR_EXT_REG        K0_TO_K1(0x190b0000)    /* Direction Register           */
#define GPIO_FLAG_SET           K0_TO_K1(0x19040006)    /* Flag Lower Half Register     */
#define GPIO_FLAG_EXT_SET       K0_TO_K1(0x190b0006)    /* Flag Lower Half Register     */
#define GPIO_FLAG_RESET         K0_TO_K1(0x19040004)    /* Flag Upper Half Register     */
#define GPIO_FLAG_EXT_RESET     K0_TO_K1(0x190b0004)    /* Flag Upper Half Register     */

#define FXS_EXT_LINE            3     /* FXS switch bit (RG1 and RG2) */      // Dubois

/*########################################################################################*/

/*########################################################################################*/
/* SPI Registers */
#define  SPI_BASE_ADDRESS       K0_TO_K1(0x19030000)       /* Base Address              */
#define  SPI_CTRL_ADDRESS       (SPI_BASE_ADDRESS + 0x00)  /* Control Register          */
#define  SPI_FLAG_ADDRESS       (SPI_BASE_ADDRESS + 0x04)  /* Flag Register             */
#define  SPI_STATUS_ADDRESS     (SPI_BASE_ADDRESS + 0x08)  /* Status Register           */
#define  SPI_XMIT_BUF_ADDRESS   (SPI_BASE_ADDRESS + 0x0C)  /* TX Register               */
#define  SPI_RECV_BUF_ADDRESS   (SPI_BASE_ADDRESS + 0x10)  /* RX Register               */
#define  SPI_BAUD_ADDRESS       (SPI_BASE_ADDRESS + 0x14)  /* Baud Rate Register        */
#define  SPI_RDBR_SHDW_ADDRESS  (SPI_BASE_ADDRESS + 0x18)  /* Shadow Register           */
                                                                                                  
/* SPI Control Register Bits  */
#define SPI_TIMOD_FST       0		/* Initialization Mode and Interrupt Generation */
#define SPI_TIMOD_LEN       2		/* Length                                       */
#define SPI_SZ_BIT          2           /* Send ZERO or last word when TDBR empty       */
#define SPI_GM_BIT          3           /* Overwrite or Discard Data when RDBR is full  */
#define SPI_PSSE_BIT        4           /* Emable Select Select.                        */
#define SPI_EMISO_BIT       5           /* Enable MISO Pin as Output.                   */
#define SPI_SIZE_BIT        8           /* Word Length 0 => 8 bits, 1 => 16bits.        */
#define SPI_LSBF_BIT        9		/* Data Format 0 -> BIG Endian, 1 -> Little     */
#define SPI_CPHA_BIT       10           /* Clock Phase                                  */
#define SPI_CPOL_BIT       11		/* Clock Polarity. 0->Active High,1->Active Low */ 
#define SPI_MSTR_BIT       12           /* SPI as Master, 1-> Master , 0 -> Slave.      */
#define SPI_WOM_BIT        13           /* Open Drian Data output enabled.              */
#define SPI_SPE_BIT        14           /* SPI Module Enable (1) /Disable (0)           */

/* SPI Status Register Bits */
#define SPI_SPIF_BIT        0		/* Set, when single word Tx is complete.        */
#define SPI_MODF_BIT        1           /* Mode Fault Error.                            */
#define SPI_TXE_BIT         2           /* Transmission Error.                          */
#define SPI_TXS_BIT         3           /* TDBR Data buffer Status. 0->Empty, 1->Full.  */
#define SPI_RBSY_BI         4           /* Receive Error.                               */
#define SPI_RXS_BIT         5           /* Rx data buffer status. 0->Empty, 1->Full.    */
#define SPI_TXCOL_BIT       6           /* Transmit Collision Error.                    */

/*#######################################################################################*/

#define DSP_SPORT0	    K0_TO_K1(0x19000080) /* DSP Sport0                         */  
#define MISC_CTRL_REG	    K0_TO_K1(0x19000088) /* Misc register                      */        
#define CPE_PER_INT         K0_TO_K1(0x19050300) /* CPE Peripheral Interrupt Register. */
#define CPE_PR_REG0         K0_TO_K1(0x19050100) /* CPE Priority Register 0.           */
#define CPE_PR_REG1         K0_TO_K1(0x19050104) /* CPE Priority Register 1.           */
#define CPE_PR_REG2         K0_TO_K1(0x19050108) /* CPE Priority Register 2.           */
#define CPE_PR_REG3         K0_TO_K1(0x1905010C) /* CPE Priority Register 3.           */
#define CPE_PR_REG4         K0_TO_K1(0x19050110) /* CPE Priority Register 4.           */
#define CPE_PR_REG5         K0_TO_K1(0x19050114) /* CPE Priority Register 5.           */
#define CPE_PR_REG6         K0_TO_K1(0x19050118) /* CPE Priority Register 6.           */
#define CPE_PR_REG7         K0_TO_K1(0x1905011C) /* CPE Priority Register 7.           */
#define CPE_PR_REG8         K0_TO_K1(0x19050120) /* CPE Priority Register 8.           */
#define SPI_CS1_EN 25
#define SPI_CS2_EN 26
#define SPI_CS3_EN 27
#define DSP_LATCH_ADDR_HI    K0_TO_K1(0x19050634)
#define DSP_LATCH_ADDR_LOW    K0_TO_K1(0x19050638)
#define DSP1_FL0_INTTERUPT 31
#define GETBIT(var, bit)        (((var) >> bit ## _BIT) & 0x1)


#define  hp_addr_latch_value1  0xb91c0000;
#define  pm_data_latch_value1  0xb91c0004;
#define  dm_data_latch_value1  0xb91c0008;
                                                                                
#define  hp_addr_latch_value2  0xb91c0010;
#define  pm_data_latch_value2  0xb91c0014;
#define  dm_data_latch_value2  0xb91c0018;

#define DSP1_DEVICE_NAME "dsp"
#define DSP2_DEVICE_NAME "dsp1"

/* These addresses are the Dsp's I/O mapped host port register addresses
 */
#define HPCR_ADDR               0x3802
#define HPCR_BWS_BIT            0
#define HPCR_AMS_BIT            6
                                                                                
#define HPPR_ADDR               0x3804
#define HPPR_MEMORY_SPACE_BIT   0
#define HPPR_DATA_TYPE_BIT      1
#define HPPR_DTYPE_16           0
#define HPPR_DTYPE_24           1
#define HP_BOOT_SPACE           1
#define HP_MEMORY_SPACE         0
                                                                                
#define HPDMA_SMPHA_ADDR        0x39F8
#define HPDMA_SMPHB_ADDR        0x39FA
#define HPDMA_PTR_ADDR          0x3A00
                                                                                
#define HPDMA_CFG_ADDR          0x3A02
#define HPDMA_EN_BIT            0
#define HPDMA_DIR_BIT           1
#define HPDMA_DATATYPE_BIT      3
#define HPDMA_OWNER_BIT         15
#define HPDMA_MEM_READ          0
#define HPDMA_MEM_WRITE         1
#define HP_DATA_16              0
#define HP_DATA_24              1
                                                                                
#define HPDMA_SRP_ADDR          0x3A04
#define HPDMA_CP_ADDR           0x3A0A
#define HPDMA_CPR_ADDR          0x3A0C
#define HPDMA_IRQ_ADDR          0x3A0E
                                                                                
#define DSP_FXO_BUFFER_ADDR  0x155c

#define DSP_SWRST_ADDR          0x0404
                                                                                
#define DM_DMA_DESCRIPTOR_ADDR  0xe000
                                                                                
#define SHIFT_ADDR(x)        (x = x << 1)
                                                                                
#define DM_MEM                  0
#define PM_MEM                  1
#define HP_REG                  2
                                                                                
void dspWriteMem(int memType, unsigned short addr, unsigned short value);
void dspWriteRead(int memType, unsigned short addr, unsigned short *value);
void ad2191dn(void);
                                                                               



#endif
