/*======================================================================
 *
 * Copyright (C) 2000 Chiplogic Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive
 * property of Chiplogic and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 *======================================================================
 */


/*     Include files
 */

#ifndef _SLICAPI_H_
#define _SLICAPI_H_

#ifdef __cplusplus
extern "C"{
#endif



#define CODEC_LE88221                           0x0
#define CODEC_LE88241                           0x1
#define CODEC_LE88111                           0x2
#define CODEC_LE88131                           0x3
#define CODEC_LE78d11				0x4
#define CODEC_SILABS				0x5


#define SLIC_STANDBY    0
#define SLIC_ACTIVE     1
#define SLIC_RING       2
#define SLIC_RINGOFF    3
#define SLIC_RING_LOWBSEL       4
#define SLIC_LOOPBACK   5
#define SLIC_REVERSE_ACTIVE     6
#define SLIC_REVERSE_RINGOFF    7

#define IDLE_POL	0
#define REVERSE_POL	1

// gain control on receive pass
#define REC_GAIN_VERY_SOFT   0
#define REC_GAIN_SOFT	     1
#define REC_GAIN_NORMAL      2
#define REC_GAIN_LOUD        3
#define REC_GAIN_VERY_LOUD   4


#ifdef FIRMWARE_CONF_COUNTRY_NL
// FXS line voltage monitoring
#define LINE_MONITORING_YES	0
#define LINE_MONITORING_NO	1

extern unsigned char voltmonitor;
#endif
void codecInit(int codecNo);
void slicModeSetting(int codecNo, int chNo, int mode);
unsigned short codecReadStatus(int codeNo);

/* Function Prototypes */
void slicInit(unsigned short numPorts);
void slicActivate(unsigned short portNum);
void slicIdlePol(unsigned short portNum);
void slicRing(unsigned short portNum);
void slicReverseActivate(unsigned short portNum);
void slicRingOff(unsigned short portNum);
void slicStandby(unsigned short portNum);
void slicLoopBack(unsigned short portNum);
void slicCodecInit(unsigned short codec); 
void slicIsr(void);
void slicGainCtrl(unsigned char portNum, unsigned int level);
void slicRegGetHookstate(void (*f)(unsigned short portNum, int newHookState));
typedef void (*slicCommandHandler)(int codecNo,int uiChannel,int mode);
typedef unsigned short (*codecStatusHandler)(int codecNo);

typedef struct
{
  UCHAR codecType;
  slicCommandHandler slicCmdHandler;
  codecStatusHandler codecStatus;
  UCHAR codecCnt;
  UCHAR ucTimeSlot0;
  UCHAR ucTimeSlot1;
  UCHAR ucPcmMode;
  UCHAR ucClock;
}slicHandlers_t;


typedef struct 
{
  int        ucNumRings;
  UCHAR      ucSlicState;
  BOOL       bFirstCadence;
  UINT16     uiT2SCntr;
  UINT16     uiT4SCntr;  
  UCHAR      slicStateChanged;
  INT16      slicCountOffhookDebounce;
  INT16      slicCountOnhookDebounce;
  INT16      slicOffTheHook;
  UCHAR       slicPolarity;
}SLICInfo_t;
   

/* Host based commands */
typedef struct
{
  int cmd;
  UCHAR port;
}SLICHostCmd_t;


void slicDeviceInitialize(void);
void slicResetRingDSPStateInfo(UCHAR ucPort);
void slicVoiceRing(UCHAR ucPort, int ringCnt);
void slicVoiceRingOff(UCHAR ucPort, BOOL bActivate);
void slicFXOCmd(UCHAR ucFXSPort, UCHAR cmd);
void slicGetHookStateInfo(SlicStateInfo_t *pHookInfo);
void slicModeSetting_88111(int codecNo,int chNo,int mode);
void slicModeSetting_88221(int codecNo,int chNo,int mode);
void slicModeSetting_78d11(int codecNo,int chNo,int mode);
void slicModeSetting_silabs(int codecNo,int chNo,int mode);
void slicUserControlRing(UCHAR port);
void slicUserControlRingOff(UCHAR port);
unsigned short codecReadStatus_88111(int codecNo);
unsigned short codecReadStatus_88221(int codecNo);
unsigned short codecReadStatus_78d11(int codecNo);
unsigned short codecReadStatus_silabs(int codecNo);
int si3216_initialize(unsigned char codecNo);
void legerity88111_init(int codecNo);
void legerity78d11_init(int codecNo);
void legerity88221_init(int codecNo);
void read2codecArray(int codecNo, unsigned char addr,unsigned char *data, int count);
int si3050_initialize(void);
void si3050PCMSlotChange(UCHAR ucFXSPort);
void si3050_cmd(int cmd);
void si3050_isr(void);
int si3050_getFXOState(void);
int slicGetFXOState(void);
void si3050_reset_polarity(void);
// BDt Boxer : Flash Hook
void si3050_FlashHook(unsigned char delay);
void si3050_Gain(int value);
void si3216_gain_ctrl(unsigned char port, unsigned char option); 
#ifdef __cplusplus
}
#endif

#endif  /*  _SLICAPI_H_ */


