/******************************************************************************
*
*   Copyright (C) 2002 Analog Devices India (P) Ltd. All rights are reserved.
*
*   The information and source code contained herein is the exclusive
*   property of Analog Devices India (P) Ltd. and may not be disclosed,
*   examined or reproduced in whole or in part without explicit written
*   authorization from the compnay.
*
******************************************************************************/
#ifndef _VOICELIB_H
#define _VOICELIB_H

#include "voice_typedef.h"
#include "voiceconf.h"
#include "common.h"
#include "../dsplib/ToneGenCons.h"
#include "voiceopt.h"

#define MAX_VOICE_OPT           4
#define ON "ON"
#define OFF "OFF"

#define AJB 1
#define FJB 0


#define V29      1
#define V27      2
#define V17      3


#define VOICE_FAIL  0
#define VOICE_SUCCESS 1
#define NM_API_MAX_LEN 100

#define DSP_IOCTL_MAJIC			0xdd	/* Un Used One */

/* DSP Port Commands */
#define DSP_INIT_SYSTEM     _IOWR(DSP_IOCTL_MAJIC,0,int)  /* Initilize DSP System.         */
#define DSP_INIT_PORT       _IOWR(DSP_IOCTL_MAJIC,1,int)  /* Initialize DSP Port           */
#define DSP_GEN_TONE        _IOWR(DSP_IOCTL_MAJIC,2,int)  /* Generate Tone                 */
#define DSP_QUIT_TONE       _IOWR(DSP_IOCTL_MAJIC,3,int)  /* Quit Tone.                    */
#define DSP_CLOSE_PORT      _IOWR(DSP_IOCTL_MAJIC,5,int)  /* Close DSP Port.               */
#define DSP_SLIC_RING       _IOWR(DSP_IOCTL_MAJIC,6,int)  /* Provide Ring to DSP Port.     */
#define DSP_SLIC_RING_OFF   _IOWR(DSP_IOCTL_MAJIC,7,int)  /* Stop Ring to the DSP Port.    */
#define DSP_SLIC_STAND_BY   _IOWR(DSP_IOCTL_MAJIC,8,int)  /* STandby the Slic Port.        */
#define DSP_SLIC_ACTIVATE   _IOWR(DSP_IOCTL_MAJIC,9,int)  /* Activae the Slic Port.        */
#define DSP_SLIC_REVERSE_ACT  _IOWR(DSP_IOCTL_MAJIC,60,int)  /* Reverse Activate the Slic Port.   */
#define DSP_SLIC_REC_GAIN  _IOWR(DSP_IOCTL_MAJIC,62,int)  /* change Receive gain on Slic Port.   */


#define DSP_SET_VOICEOPT    _IOWR(DSP_IOCTL_MAJIC,10,int)/* DSP Set voice options      */
#define DSP_SET_JBPARAMS    _IOWR(DSP_IOCTL_MAJIC,11,int)/* DSP set Jitter Buffer Type */
#define DSP_SET_DWNLOADOPTS _IOWR(DSP_IOCTL_MAJIC,12,int)/* DSP Download Options.      */
#define DSP_CALLERID        _IOWR(DSP_IOCTL_MAJIC,13,int)/* DSP Display CallerId       */
#define DSP_GET_JBPARAMS    _IOWR(DSP_IOCTL_MAJIC,14,int)/* DSP Get Jitter Buffer Type */
#define DSP_GET_VOICEOPT    _IOWR(DSP_IOCTL_MAJIC,15,int)/* DSP Get Voice Options.     */
#define DSP_GET_DWNLOADOPTS _IOWR(DSP_IOCTL_MAJIC,16,int)/* DSP Get Download Options.  */
#define DSP_CLEAR_VOICE_BUFFER _IOWR(DSP_IOCTL_MAJIC,61,int)/* DSP Clear Data Queues.  */
#define DSP_FAR_END_TONE_GENERATE_ON _IOWR(DSP_IOCTL_MAJIC,17,int)/* Far End Tone Generation On.   */
#define DSP_FAR_END_TONE_GENERATE_OFF _IOWR(DSP_IOCTL_MAJIC,18,int)/* Far End Tone Generation Off. */
#define DSP_SET_PCMMODE    _IOWR(DSP_IOCTL_MAJIC,19,int)/* Set SPORT PCMMODE           */
#define DSP_GET_PCMMODE    _IOWR(DSP_IOCTL_MAJIC,20,int)/* Get SPORT PCMMODE           */
#define DSP_QUIT_FSK_PLAY  _IOWR(DSP_IOCTL_MAJIC,21,int)/* To quit FSK generation      */
#define DSP_GAIN_ADJUST    _IOWR(DSP_IOCTL_MAJIC,22,int)/* Gain Adjustments.           */

#define DSP_SET_GCPARAM   _IOWR(DSP_IOCTL_MAJIC,23,int) /* Configure Gain Control        */
#define DSP_NEXT_GCPARAM  _IOWR(DSP_IOCTL_MAJIC,24,int)  /* Get Next GC Parameter        */
#define DSP_SET_COUNTRYOPT _IOWR(DSP_IOCTL_MAJIC,25,int)/* Set Country Option            */
#define DSP_GET_COUNTRYOPT _IOWR(DSP_IOCTL_MAJIC,26,int)/* Get Country Option            */
#define DSP_SAVE_CONF     _IOWR(DSP_IOCTL_MAJIC,27,int) /* Configure DSP with Saved.     */
#define DSP_GET_CONF      _IOWR(DSP_IOCTL_MAJIC,28,int) /* Command to read DSP Configs   */
#define DSP_SET_DYN_DTMF_REJ _IOWR(DSP_IOCTL_MAJIC,29,int) /* DSP DTMF Reject.           */
#define DSP_SET_DATA_MODE    _IOWR(DSP_IOCTL_MAJIC,30,int)/* DSP Set Data Mode.          */
#define DSP_SET_MODIFYCODEC  _IOWR(DSP_IOCTL_MAJIC,31,int)/* DSP Set Modify Codec        */

#define DSP_SET_DATA_MODE_ENABLE _IOWR(DSP_IOCTL_MAJIC,32,int)/* Enable DSP Data mode settings */
#define DSP_SET_VAD	_IOWR(DSP_IOCTL_MAJIC,33,int) /* Enable / Disable VAD    */
#define DSP_GET_VAD     _IOWR(DSP_IOCTL_MAJIC,34,int) /* Get VAD Configurations  */
#define DSP_SET_PLC     _IOWR(DSP_IOCTL_MAJIC,35,int) /* Enable / Disable PLC    */
#define DSP_GET_PLC     _IOWR(DSP_IOCTL_MAJIC,36,int) /* Get PLC Configurations  */
#define DSP_SET_EC      _IOWR(DSP_IOCTL_MAJIC,37,int) /* Enable / Disable EC     */
#define DSP_GET_EC      _IOWR(DSP_IOCTL_MAJIC,38,int)  /* Get EC Configurations   */

#define DSP_MEM_READ      _IOWR(DSP_IOCTL_MAJIC,39,int)  /* MEMORY Read   */
#define DSP_MEM_WRITE      _IOWR(DSP_IOCTL_MAJIC,40,int)  /* MEMORY write   */


#define DSP_SET_DEBUG_LEVEL _IOWR(DSP_IOCTL_MAJIC,41,int)  /* Set Debug Levels   */
#define DSP_GET_DEBUG_LEVEL _IOWR(DSP_IOCTL_MAJIC,42,int)  /* Get Debug Levels   */


#define DSP_SWITCHTO_PASSTHRU _IOWR(DSP_IOCTL_MAJIC,43,int)  /* Set Debug Levels   */
#define T38_RESET_QUEUE  _IOWR(DSP_IOCTL_MAJIC,44,int)  /* T.38 Clear Q     */
#define DSP_SWITCHTO_T38 _IOWR(DSP_IOCTL_MAJIC,45,int)  /* Switch to T.38   */
#define DSP_GET_FAX_MODE _IOWR(DSP_IOCTL_MAJIC,46,int)  /* Get T38 FAX Mode */
#define DSP_SET_FAX_MODE _IOWR(DSP_IOCTL_MAJIC,47,int)  /* Set FAX mode   */
// Sagem ICC : Bug 3242 2006/01/09
#define DSP_MWIIND _IOWR(DSP_IOCTL_MAJIC,48,int)  /* Display VMWI   */

#if FXO_MODULE
#define FXO_DSP_SET_CONNECTION _IOWR(DSP_IOCTL_MAJIC,50,int)  /* FXO-FXS Connect   */
#define FXO_DSP_QUIT_CONNECTION _IOWR(DSP_IOCTL_MAJIC,51,int) /* FXO-FXS Disconnect*/
#define FXO_DSP_SLIC_CMD       _IOWR(DSP_IOCTL_MAJIC,52,int)  /* FXO Commands.     */
#define FXS_GET_SLICSTATE      _IOWR(DSP_IOCTL_MAJIC,53,int)  /* Get FXS State info*/
#define FXO_DSP_DETECT_TONE    _IOWR(DSP_IOCTL_MAJIC,54,int)  /* Enable FXO Tone Detection */
#define FXO_DSP_CALLERID_DETECT _IOWR(DSP_IOCTL_MAJIC,55,int) /* Enable FXO CID Detection */
#define FXO_LINE_STATUS        _IOWR(DSP_IOCTL_MAJIC,56,int)  /* Enable FXO Tone Detection */
#define FXO_RESET_POLARITY     _IOWR(DSP_IOCTL_MAJIC,57,int)  /* Reset reference polarity (ghost incomming call) */
#define FXO_FLASHHOOK          _IOWR(DSP_IOCTL_MAJIC,58,int)  /* Command a flash hook on FXO */
#define FXO_GAIN_DECT          _IOWR(DSP_IOCTL_MAJIC,59,int)  /* change gain on FXO for DECT connection*/
#endif

#if THREEWAY_CONFERENCE
#define DSP_START_CONF         _IOWR(DSP_IOCTL_MAJIC,70,int)  /* Start Conference */
#define DSP_STOP_CONF          _IOWR(DSP_IOCTL_MAJIC,71,int)  /* Stop Conference  */
#define DSP_SET_CONFERENCE     _IOWR(DSP_IOCTL_MAJIC,72,int)  /* Set Conference   */
#define DSP_GET_CONFERENCE     _IOWR(DSP_IOCTL_MAJIC,73,int)  /* Get Conference   */

#endif


#if DRV_DEBUG_STAT
#define DSP_GET_VOICE_STAT _IOWR(DSP_IOCTL_MAJIC,204,int) /* Get DSP Debug Levels*/
typedef struct
{
  unsigned long intCount;
  unsigned long taskletCount;
  unsigned long ajbReadCnt;
  unsigned long ajbSpeechPkt;
  unsigned long ajbNULLPkt;
  unsigned long ajbOutofOrder;
  unsigned long ajbEnqCnt;
  unsigned long ajbReadNULLCnt;
  unsigned long ajbReadSpeechCnt;
  unsigned long rtpPktRcv;
  unsigned long rtpPktSend;
  unsigned long rtpPktErr;
  unsigned char bLog;
}VoiceDebugStat_t;
#endif

/* User Controlled SLIC Rings */
#define SLIC_RING_USER_CONTROLLED _IOWR(DSP_IOCTL_MAJIC,80,int) /* SLIC User Controlled Ring */
#define SLIC_RING_OFF_USER_CONTROLLED _IOWR(DSP_IOCTL_MAJIC,81,int) /* SLIC User Controlled RingOff */

#if AJB_ENABLE_STATISTICS
#define AJB_DUMP_DEBUG_INFO    _IOWR(DSP_IOCTL_MAJIC,90,int)
#endif

#if KRTP
#define DSP_RTP_CMD            _IOWR(DSP_IOCTL_MAJIC,100,int)
#endif

#if DECT_MODULE
#define DSP_START_TRANSCODE    _IOWR(DSP_IOCTL_MAJIC,101,int)
#define DSP_STOP_TRANSCODE     _IOWR(DSP_IOCTL_MAJIC,102,int)
#endif

#define FAX_T38_METHOD 1
#define FAX_PASSTHRU_METHOD 2


#define DSP_MEDIA_AUDIO 0x01
#define DSP_MEDIA_DATA 0x02


/* DSP Data Modes */
#define DSP_DATA_INACTIVE               1
#define DSP_DATA_SEND_ONLY              2
#define DSP_DATA_RECV_ONLY              3
#define DSP_DATA_SEND_RECV              4

#define DSP_FOUR_CADENCE_TONE 		1
#define DSP_LOCAL_TONE			2
#define SLIC_LOCAL_TONE                 3

#define NON_CONTINUOUS_TONE             0x0
#define CONTINUOUS_TONE                 0x2
#define PLAY_ONLY_ONCE                  0x3

#define MAX_T38_Q_LEN	200
#define T38_PAYLOAD_LEN 80


typedef struct
{
  unsigned short portNum;
  unsigned char  mode;
}DSPDataMode_t;


/* Circular Queue Structure */
typedef struct
{
  UINT16  writeIdx;
  UINT16  readIdx;
} VoiceCirQ_t;


typedef struct
{
  unsigned char          ucChNum;               /* Port Number.             */
  unsigned char          ucCodec;               /* Codec Type.              */
  unsigned char          cFrameCounter;         /* Frame Counters           */
  unsigned short         ucPktLen;              /* Length of the packet     */
  unsigned short         uiSeqNo;               /* Sequence Number.         */
  unsigned int           ulTimeStamp;           /* Time Stamp.              */
  unsigned char          aucPayload[350];       /* Actual Payload.          */
}VoiceQ_t;

/* T.38 Data Structure */
typedef struct
{
  unsigned char    ucChNum;                     /* Port Number */
  unsigned short   ucPktLen;                    /* Length of the packet */
  unsigned char    aucPayload[T38_PAYLOAD_LEN]; /* Actual Payload */
}t38Pkt_t;


/* Structure holds Voice Option Parameters */
typedef struct
{
   char   bVad;
   char   bPLC;
   char   bEC;
   unsigned char  ucFaxMode;
   int  lGtx;
   int  lGrx;
   int  lGec;
   unsigned char  ucPort;
   unsigned short uiTailLen; /*b/w 1 & 32, default: 12.5*/
   //VAD2
   unsigned char  ucVadType;
   char   bVadConfig;
   char   bPlcConfig;
   char   bEcConfig;
   char   bTlConfig;
   unsigned char  ucPcmMode; /* Mainly Used for Legirity Only */
#if FXO_MODULE
   /* Caller ID and Tone Detection Flags */
   unsigned char  fxoToneDetect;
   unsigned char  fxoCallerIdDetect;
#endif
}VoiceOpt_t;


typedef struct _voiceOptions_
{
   unsigned char   bVad;
   unsigned char   bPLC;
   unsigned char   bEC;
   unsigned char   ucGain;
   int             uiTailLen;
//VAD2
   unsigned char   ucVadType;
   unsigned short  encoder_opt[3];
   unsigned short  decoder_opt[2];
   int maxFrames;
}VoiceOptions;

typedef struct
{
  VoiceOptions voiceOptions;
  unsigned char ucPort;
}PortVoiceOpts_t;



#define MAX_CID_STRING 50
#define DTMF_CID	0
#define FSK_CID		1
typedef struct
{
  unsigned short  uiCurChanNum;
  unsigned char   CidTxFlag;
  unsigned char   aucRemoteUserNum[MAX_CID_STRING];
  unsigned char   aucRemoteUserName[MAX_CID_STRING];
  unsigned char   aucDateBuffStr[MAX_CID_STRING];
  unsigned char   CidType;
  unsigned char   CidTxMode;		// FSK or DTMF CallerId
  unsigned char   bFirstRingOff;
}CallerIdInfo_t;


/* This structure is mainly ported fromm VxWorks code just to maintain portability
   aspects */
typedef struct
{
  char   bErr;
  int  lIndex;
  char   bVad;
  char   bPLC;
  char   bEC;
  unsigned char   ucFaxMode;
  short  uiTailLen; /*b/w 1 & 32, default: 12.5*/
  //VAD2
   unsigned char  ucVadType;

  unsigned char  ucGain;
  int  lGtx;
  int  lGrx;
  int  lGec;
  unsigned char  ucPortNumber;

  char   bJBType;
  unsigned char  uiThreshold;

  unsigned char   ucDspNum;
  unsigned char   ucDownloadOpts;

  unsigned char  ucCountryOpt;

  char   bMode;
  char   bRate;
  unsigned char  uLSRed;  //Low Speed Redundancy for T.38
  unsigned char  uHSRed;  //High Speed Redundancy for T.38
  unsigned char ucPcmMode;
  short uiPassThrThreshold;
  unsigned char bConference;
}VOICELS;



typedef struct
{
  unsigned short ulCadenceOn; 		// Candence on period (Unit: 10ms)
  unsigned short ulCadenceOff; 		// Candence off period (Unit: 10ms)
  unsigned short ulCadenceFrq1;		// Candence first frequency
  unsigned short ulCadenceFrq2; 	// Candence second frequency
  unsigned short ulCadenceAmp1;  	// Candence amplification
  unsigned short ulCadenceAmp2;
}CANDENCE_TYPE;

typedef struct
{
  unsigned short ulToneId; 		// Tone Id
  unsigned short ulCadenceCount; 	// Cadence Number
  CANDENCE_TYPE stCandence[4];  // Cadence information
  unsigned short ulCadenceDuration;     /* Unit: 10ms Maxmun is 1 minutes '0' means 1 minutes
                                   (Need Max 4 candences, Min 1 candence ) */
}FOURCADENCE_TONE_TYPE;

typedef struct
{
  unsigned short timeout;       /* Timeout in case of DSP_LOCAL_TONE */
  unsigned short toneId;        /* Tone Id 			     */
}DSPLocalTone_t;

typedef struct
{
  unsigned short portNum;	/* Port Number */
  unsigned char  toneType;      /* Type of Tone DSP_FOUR_CADENCE_TONE OR DSP_LOCAL_TONE */
  union
  {
    DSPLocalTone_t local; /* For Local Tone 					*/
    FOURCADENCE_TONE_TYPE forCadInfo; /* Four Cadance Tone Information 		*/
  }tone;
}DSPGenToneInfo_t;


/* DSP command ACKs */
typedef enum
{
    DSPIPC_RESET_ALL_PORTS_DONE,
    DSPIPC_INIT_PORT_DONE,
    DSPIPC_MODIFY_PORT_CODEC_DONE,
    DSPIPC_CLOSE_PORT_DONE,
    DSPIPC_GEN_TONE_DONE,
    DSPIPC_QUIT_TONE_DONE,
    DSPIPC_SYNC_ERROR,

    DSPIPC_DIGIT_RECEIVED,

    DSPIPC_CNG_TONE_REPORT,
    DSPIPC_mMODEMREADY,
    DSPIPC_SIN_FAXTONE_DETECTED, /** Without Phase reversal tone at receive side **/
    DSPIPC_RIN_FAXTONE_DETECTED, /** Without Phase reversal tone at sending side **/
    DSPIPC_V21_PREAMBLE_DETECTED,  /**  V21 Preamble Detection event at Receive side **/
    DSPIPC_FAX_DEBUG,
    DSPIPC_RING_EXPIRED,
    DSPIPC_TONE_EXPIRED,
    DSPIPC_INTER_DIGIT_EXPIRED,
    SLIC_OFFHOOK_EVENT,
    SLIC_ONHOOK_EVENT,
    SLIC_FLASHHOOK_EVENT,
    DSPIPC_SIN_MODEMTONE_DETECTED,  /** With Phase reversal tone aat receive side **/
    DSPIPC_RIN_MODEMTONE_DETECTED,  /** With Phase reversal tone aat receive side **/
    DSPIPC_CALLERID_PLAY_DONE,
    DSPIPC_VMWI_PLAY_DONE,
    DSPIPC_DIGIT_TONE_START_REPORT,
    DSPIPC_DIGIT_TONE_END_REPORT,
    DSPIPC_DIGIT_TONE_CONT_REPORT,
    DSPIPC_INIT_THREE_WAY_CONFERENCE_DONE,
    DSPIPC_QUIT_THREE_WAY_CONFERENCE_DONE,
    DECT_RING_REMOVE_DONGLE	,				// Mbe,
    DECT_ONHOOK_EVENT_REMOVE_DONGLE,		// Mbe
    DSPIPC_ADD_DONGLE,
    DSPIPC_REMOVE_DONGLE,					// Mbe
    DSPIPC_PRESENT_DONGLE					// Mbe
}DspipcConfirmCode;

/* DSP DIgit Info Structure */
typedef struct CmdDigitInfo
{
  unsigned char               ucChannelNo;
  unsigned char               dialledDigit;
  unsigned char               ucSeqNo;
  DspipcConfirmCode           ConfirmCode; // The data type is declared in dspipc.h
  unsigned short              uiPowerLvl;
  unsigned int                ulDigDuration;
  unsigned int                ulTimeStamp;
  unsigned char               fxsNo;
#if FXO_MODULE
  UINT16 		      cptr_FXO;
#if CID_RX_DET
  CallerIdInfo_t              fxoCallerIdDet;
#endif
#endif
}CmdDigitInfo_t;


typedef struct
{
  unsigned int dspNum;
  unsigned int address;
  unsigned int value;
  unsigned int type;
}VoiceMem_t;

typedef struct
{
  unsigned short portNumber;	/* Port Number			*/
  short  timeout;               /* Ring Timeout			*/
}DSPRingInfo_t;


/* Codec Types Enum */
typedef enum
{
  PCMU = 0,CODEC_1016,PCMA=8,G722=9,G726_32=2,VAD_CNG = 13,G729=18,CODEC_EXT=15,G723AH = 96,
  G723AL,G723H,G723L,G729A,G729B,G723=4,TELEPHONE_EVENT = 103,G726_16 = 108, G726_24, G726_40
}Codec_e;

/* SLIC Hook State */
typedef enum
{
  SLIC_ONHOOK=0,      /* Hook is ON-HOOK   */
  SLIC_OFFHOOK        /* Hook is OFF-HOOK  */
}SLICHOOKSTATE_e;

/* Slic Ring State */
typedef enum
{
  SLIC_RINGING_ON = 1,		/* ringing-on         */
  SLIC_RINGING_OFF,             /* ringing-off        */
  SLIC_NORING_OFFHOOK,          /* No Ring Off Hook.  */
  SLIC_NORING_ONHOOK            /* No Ring On Hook.   */
}SLICRINGSTATE_e;


/* SLIC State Information Structure. */
typedef struct
{
  unsigned char      ucPortNum;    /* Port Number.     */
  unsigned char      slicState;    /* Ringing State.   */
  unsigned char      hookState;    /* Hook State.      */
}SlicStateInfo_t;


#if FXO_MODULE
#define PSTNMAX_DIAL 20

/* FXO Line States */
typedef enum
{
  FXO_LINE_FREE = 5,                    /* Line is Free.       */
  FXO_RING_1_ON_DETECT = 6,             /* Ring1 ON detect.    */
  FXO_RING_1_OFF_DETECT = 7,            /* Ring1 OFF detect.   */
  FXO_RING_2_ON_DETECT = 8,             /* Ring2 ON detect.    */
  FXO_RING_ON_DETECT = 9,               /* Ring ON Detect.     */
  FXO_RING_OFF_DETECT = 10,             /* Ring OFF Detect.    */
  FXO_LINE_BUSY = 11,                   /* Line is Busy.       */
  FXO_LINE_OUT = 12,                    /* No Line.            */
  FXO_LINE_INVERSE_POLARITY = 13,       /* Polarity Changed    */
  FXO_LINE_INVERSE_POLARITY_BUSY = 14,  /* Line is Busy and with Polarity Changed */
  FXO_DIGIT_GEN_TONE_DONE,              /* FXO Digit generated */
  FXO_TONE_DETECTED,                    /* Tone Detected on FXO */
  FXO_CALLERID_DETECTED,
  FXO_Q23_INCALL			/* Polarity Changed incoming call (holland) */
}fxoLineStates;


/* FXO Commands Enumeriations.*/
typedef enum
{
   FXO_ONHOOK = 0,          /* ON HOOK      */
   FXO_OFFHOOK,             /* OFF HOOK     */
   FXO_ONHM_ENA,            /* Enable CID   */
   FXO_ONHM_DIS             /* Disable CID  */
} FXOCMD_e;

/* FXO Connection Type */
typedef enum
{
   TIMESLOT,            /* Time Slot        */
   TONE_DSP,            /* Tone Detection   */
   DTMF_DSP,            /* DTMF Detection   */
   TS_FXO2FXS_LB,       /* FXO-FXS Loopback */
   TS_FXO2DECT,			/* FXO-DECT Timeslot*/
}cnxType_e;

typedef struct
{
  unsigned char  ucPortNo;     /* Port Number.      */
  cnxType_e  ConnectType;      /* Connection Type   */
  unsigned short  uiLen;       /* Length            */
  unsigned char  uiCmdParams[PSTNMAX_DIAL]; /* Commands */
}pstnDspCmdInfo_t;

#endif

/* Configuration Module Function Prototypes  */
void confInitGainControl(UCHAR ucPort);
void voiceCliConfigStored(void);
short voiceCliCmd(VOICELS*, unsigned int cmd);
unsigned char confGetJitterThreshold(void);
unsigned short confFirstCadenceOn(unsigned short uiCountry );
unsigned short confFirstCadenceOff(unsigned short uiCountry );
unsigned short confCadenceOn(unsigned short uiCountry );
unsigned short confCadenceOff(unsigned short uiCountry );
void confSetMaxFrames(unsigned short portNum,unsigned short maxFrame);
unsigned short confGetMaxFrames(unsigned short portNum);
void confGetCountryOpts(unsigned short *puiCountry);
void confSetCountryOpts(unsigned short *puiCountry);
void confSetJitterThreshold(int Threshold);
UCHAR confGetFAXMode(UINT16 uiPort);
VOID  confSetPLC(UINT16 uiPort,UCHAR plcMode);
UCHAR confGetPLC(UINT16 uiPort);
UCHAR confGetPCMMode(UINT16 uiPort);
//VAD2
UCHAR confGetG711VadType(UINT16 uiPort);
UCHAR confGetConference(UINT16 dspNum);

#endif
