#include <voice_typedef.h>
#include <voicelib.h>
#if KRTP
#include <common.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <voice_bitmap.h>
#include <JitVar.h>
#include <JitProto.h>
#include <linux/skbuff.h>
#include <linux/udp.h>
#include <dspipc.h>
#include "rtpgdef.h"
#include "rtpmsg.h"
dsp_regNetwork_str netCallBack;
#if DRV_DEBUG_STAT
extern VoiceDebugStat_t VoiceDebugStat[];
#endif
RTPMessageInfo_t rtpControlMsgBuffer[RTP_CNTRL_MSG_CNT];
VoiceCirQ_t      rtpControlCmd;
struct nf_hook_ops rtpNFHook;
extern wait_queue_head_t  voice_wait_queue;
extern RTPInfo_t       gaRTPInfo[MAX_RTP_SESSIONS];
extern UCHAR           aucSITBitMask[BitmapSize(MAX_SOURCES)];
extern SrcInfoTable_t  gaSrcInfoTable[MAX_SOURCES];
extern UCHAR           gucTotalActiveSess;
extern UCHAR           gaucActiveRTPSess[MAX_RTP_SESSIONS];
/*************************************************************************
 * Function Name        :       rtpHandleMsg.                            *
 * Description          :       Function to process RTP packet.          *
 * Input                :       socket, data , length of data.           *
 * Output               :       None.                                    *
 * Return Value         :       SUCCESS / FAILURE.                       *
 ************************************************************************/
INT32 rtpHandleMsg(RTPInfo_t *pRtpInfo, UCHAR *pcRTPData, UINT16 uiLen)
{
  if(pRtpInfo->ucMode == RTP_RECVONLY ||
                          pRtpInfo->ucMode == RTP_SENDRECV)
  {
    VoiceNWPkt_t  WtRTPPkt;
    if(rtpParseMsg(pRtpInfo, pcRTPData, uiLen, &WtRTPPkt) == SUCCESS)
    {
#if DRV_DEBUG_STAT
      VoiceDebugStat[pRtpInfo->ucEpIndex].rtpPktRcv++;
#endif
      if(netCallBack.writeVoice != NULL)
        netCallBack.writeVoice(&WtRTPPkt,pcRTPData);
    }
#if DRV_DEBUG_STAT
    else 
    {
      VoiceDebugStat[pRtpInfo->ucEpIndex].rtpPktErr++;
    }
#endif
  }
  return SUCCESS;
}
#if KRTCP_ENABLED
/*************************************************************************
 * Function Name        :       rtcpHandleMsg.                           *
 * Description          :       Function to process RTCP packet.         *
 * Input                :       socket, data , length of data.           *
 * Output               :       None.                                    *
 * Return Value         :       SUCCESS / FAILURE.                       *
 ************************************************************************/
INT16 rtcpHandleData(RTPInfo_t *pRtpInfo, UCHAR *pcRTCPData, UINT16 uiLen)
{
  if(uiLen > 0)
  {
    if(rtcpHeaderValidate(pcRTCPData,uiLen/4,pRtpInfo) != SUCCESS) 
    {
      DEBUG(PRINT("failed in rtcp header validation\n"), DEBUG_EXCEPTION);
      return(FAILURE);
    }
    return SUCCESS; 
  }
  return FAILURE;
}
#endif
/****************************************************************
 *  Function Name : rtpProcessControlMessage.                   *
 *  Description   : Function to process applictaion control msg.*
 *  Input         : Control Message.                            *
 *  Output        : None.                                       *
 *  Return Value  : 0 on Success else -EFAULT                   *
 ***************************************************************/
int rtpProcessControlMessage(unsigned long param)
{
  RTPMessageInfo_t *pMsg;
  rtpControlCmd.writeIdx = (rtpControlCmd.writeIdx + 1) % RTP_CNTRL_MSG_CNT;
  pMsg = &rtpControlMsgBuffer[rtpControlCmd.writeIdx];
  copy_from_user((UCHAR*)pMsg,(UCHAR*)param,sizeof(RTPMessageInfo_t));
  wake_up_interruptible(&voice_wait_queue);
  return SUCCESS;
}

int rtpProcessControlCmd(void)
{
  RTPMessageInfo_t *pMsg;
#if KRTP_RFC_2833
  RTPInfo_t     *pRtpInfo   = NULL;
#endif
  SessInfo_t    *pSessInfo  = NULL;
  rtpControlCmd.readIdx = (rtpControlCmd.readIdx + 1) % RTP_CNTRL_MSG_CNT; 
  pMsg = &rtpControlMsgBuffer[rtpControlCmd.readIdx];
  pSessInfo = (SessInfo_t *)pMsg->acMtext;
  DEBUG(PRINT("RTP Command Received : %d \n",pMsg->ulMtype), DEBUG_KRTP);
  switch(pMsg->ulMtype)
  {
    case RTP_CMD:
      if(pSessInfo->rtp_cmd != CREATE_SESSION &&
                               pSessInfo->rtp_cmd != SEND_RTCP_DATA)
      {
        if((pSessInfo->iSessId = rtpGetSessId(pSessInfo)) == FAILURE)
          return FAILURE;
      }
      switch(pSessInfo->rtp_cmd)
      {
#if KRTCP_ENABLED
        case SEND_RTCP_DATA:
          sendRTCPData(&gaRTPInfo[pSessInfo->iSessId]);
          break;
#endif
        case CREATE_SESSION:
          DEBUG(PRINT("RTP Create Session Command \n"), DEBUG_KRTP);
          rtpCreateSession(pSessInfo);
          break;
        case MODIFY_SESSION:
          DEBUG(PRINT("RTP Modify Session Command \n"), DEBUG_KRTP);
          rtpModifySession(pSessInfo);
          break;
        case DELETE_SESSION:
          DEBUG(PRINT("RTP Delete Session Command \n"), DEBUG_KRTP);
          rtpDeleteSession(pSessInfo->iSessId);
          break;
        case MODIFY_SES_MODE:
          DEBUG(PRINT("RTP Modify Session Mode Command \n"), DEBUG_KRTP);
          rtpModifySessMode(pSessInfo->iSessId,pSessInfo->ucMode,
                                                   pSessInfo->ucEpIndex);
          break;
        case MODIFY_SES_CODEC:
          DEBUG(PRINT("RTP Modify Session Command Command \n"), DEBUG_KRTP);
          rtpModifySessCodec(pSessInfo->iSessId,pSessInfo->uiPayloadType,
                                                   pSessInfo->ucEpIndex);
          break;
        default:
           printk("processRTPMessage: Unknown event [%d]\r\n",
                                                  pSessInfo->rtp_cmd);
           break;
      }
      break;
#if KRTP_RFC_2833
    case RFC2833_CMD:
      //printk("\r\nprocessRTPMessage(): RFC2833_DATA_INFO\r\n");
      pRfc2833Info = (rfc2833Data_t *)pMsg->aucRfc2833Info;
      pRtpInfo = &gaRTPInfo[gaucActiveRTPSess[pSessInfo->ucSigSessId]];
      pRtpInfo->bRfc2833 = TRUE;
      if(rtpSendData(pRtpInfo, pRfc2833Info->aucData,
             pRfc2833Info->uiLen, pRfc2833Info->ulTimeStamp,0) == FAILURE)
      {
          printk("\r\nprocessRTPMessage(): rtpSendData failed\r\n");
      }
      break;
#endif

    default:
      printk("default:processRTPMessage: Unknown event\r\n");
      break;
  }
  return SUCCESS;
}
/****************************************************************
 *  Function Name : rtp_check_hook_func.                        *
 *  Description   : Net filter Hook to process RTP packet.      *
 *  Input         : NF Hook Parameters.                         *
 *  Output        : None.                                       *
 *  Return Value  : NF_DROp on Success else NF_ACCEPT           *
 ***************************************************************/
unsigned int rtp_check_hook_func(unsigned int hooknum, 
                           struct sk_buff **skb,
                           const struct net_device *in, 
                           const struct net_device *out,
                           int (*okfn)(struct sk_buff *))
{
  struct sk_buff *sb = (struct sk_buff *)*skb;
  UINT16 udp_sport, ulen, udp_dport;
  BOOL bRTPPacaket = FALSE;
  UCHAR ucSessIndex;
  RTPInfo_t *pRtpInfo;
  UCHAR *udata;
  struct udphdr *udp_header;
  struct iphdr *ip;
  int ihl;
  /* Look for the active session */
  if(!gucTotalActiveSess)
    return NF_ACCEPT;
  /* Do sanity Checks on the packet */
  if(sb == NULL)
    return NF_ACCEPT;
  ip = sb->nh.iph;
  if(ip  == NULL)
  {
    return NF_ACCEPT;
  }
  if(ip->protocol != IPPROTO_UDP)
  {
    return NF_ACCEPT;
  }
  ihl = ip->ihl*4;
  if (!pskb_may_pull(sb, ihl))
    return NF_ACCEPT;
  udp_header = (struct udphdr *)(sb->data+ihl);
  udp_dport = ntohs(udp_header->dest);
  udp_sport = ntohs(udp_header->source);
  for(ucSessIndex = 0; ucSessIndex < gucTotalActiveSess ;ucSessIndex++)
  {
    pRtpInfo = &gaRTPInfo[gaucActiveRTPSess[ucSessIndex]];
    if(pRtpInfo->ulSrcIPAddr == sb->nh.iph->daddr)
    {
      if((pRtpInfo->uiRTPPort == udp_dport)|| 
                            (pRtpInfo->uiRTCPPort == udp_dport))
      {
        if(skb_pull(sb , ihl + sizeof(struct udphdr)) == NULL)
          return NF_ACCEPT;
        udata = sb->data;
        ulen = sb->tail - sb->data;
        if(pRtpInfo->uiRTPPort == udp_dport)
          rtpHandleMsg(pRtpInfo, udata, ulen);
#if KRTCP_ENABLED
        else
          rtcpHandleData(pRtpInfo, udata, ulen);
#endif
        bRTPPacaket = TRUE;
      }
    }
  }
  if(bRTPPacaket)
    return NF_DROP;  // can use NF_STOLEN
  return NF_ACCEPT;
}
void rtpWakeup(unsigned int channel)
{
  wake_up_interruptible(&voice_wait_queue);
}
/*************************************************************************
 * Function Name        :       rtpDaemon.                               *
 * Description          :       Main RTP Kernel Thread Function.         *
 * Input                :       None.                                    *
 * Output               :       None.                                    *
 * Return Value         :       SUCCESS / FAILURE.                       *
 ************************************************************************/
int rtpDaemon(void *cpu_pointer)
{
  struct sched_param param;
  RTPInfo_t *pRtpInfo = NULL;
  int iLenSent;
  int port;
  VoiceQ_t *pVoice;
  /* Make this thread as real-time thread. */
  lock_kernel();
  daemonize("krtpd");
  current->flags |= PF_NOFREEZE;
  strcpy(current->comm, "krtpd");
  param.sched_priority = MAX_USER_RT_PRIO-1;
  sched_setscheduler(current,SCHED_FIFO,&param);
  unlock_kernel();
  rtpControlCmd.readIdx = rtpControlCmd.writeIdx = 0;
  netCallBack.voiceReady = rtpWakeup;
  dsp_registerNetwork(&netCallBack);
  printk("RTP Kernel Thread Started \r\n");
  while(1)
  {
    if(rtpControlCmd.readIdx != rtpControlCmd.writeIdx)
    {
      rtpProcessControlCmd();
    } 
    /* Wait for the packet from the media framework */
    interruptible_sleep_on(&voice_wait_queue);
    /* We have packet */
    for(port = 0; port < MAX_VOICE_CHANNELS; port++)
    {
      if(netCallBack.readVoice && netCallBack.readVoice(&pVoice,port) == SUCCESS)
      {
        /* Frame RTP Packet */
#if DECT_TESTING
        if(port == 1)
        {
           if(rtpFrameMsg(pVoice,0) != SUCCESS)
             continue;
        }
        else
#endif
        if(rtpFrameMsg(pVoice,port) != SUCCESS)
          continue;
        pRtpInfo = &gaRTPInfo[gaucActiveRTPSess[pVoice->ucChNum]];
        if( (pRtpInfo->rtp_sock != NULL) && 
             (pRtpInfo->ucMode == RTP_SENDONLY || pRtpInfo->ucMode == RTP_SENDRECV))
        {
          iLenSent = rtpSendData(pRtpInfo->rtp_sock,
                           pVoice->aucPayload,
                           (pVoice->ucPktLen+(sizeof(RTPHdr_t))),
                           pRtpInfo->ulDestIPAddr,
                           pRtpInfo->uiDestRTPPort);
          if(iLenSent < 0)
          {
            printk("rtpSend: error in sendto \n");
          }
          else
          {
#if DRV_DEBUG_STAT
            VoiceDebugStat[pRtpInfo->ucEpIndex].rtpPktSend++;
#endif
            pRtpInfo->ulTotalPacketsSent++;
            pRtpInfo->ulTotalOctetsSent += pVoice->ucPktLen;
            pRtpInfo->bPktSentInRTCPInterval = TRUE;
            pRtpInfo->bPktSentInLast2Intervals = TRUE;
          }
          pRtpInfo->ulTimestamp = pVoice->ulTimeStamp;
        }
      }
    }
  }
}
/****************************************************************
 *  Function Name : rtpInit.                                    *
 *  Description   : Function to initialize RTP.                 *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
int rtpInit(void)
{
  pid_t rtp_pid;
  printk("Starting Kernel RTP Thread \n");
  /* Register the Netfilter Hook for UDP */
  rtpNFHook.hook     = rtp_check_hook_func; /* Handler function */
  rtpNFHook.hooknum  = NF_IP_LOCAL_IN;
  rtpNFHook.pf       = PF_INET;
  rtpNFHook.priority = NF_IP_PRI_LAST;
  /* Start kernel RTP Thread */
  rtp_pid = kernel_thread(rtpDaemon, (void *) NULL, CLONE_FS | CLONE_FILES);
  if(rtp_pid < 0)
  {
    printk("Could not start RTP Kernel Thread \n");
    return FAILURE;
  }
  dspipcSetDataMode(1);
  return 0;
}
#endif
