#include <voice_typedef.h>
#include <voicelib.h>
#if KRTP
#include <linux/netfilter.h>
#include <voice_bitmap.h>
#include <dspipc.h>
#include <JitVar.h>
#include "rtpgdef.h"
#include "rtpmsg.h"
extern RTPInfo_t       gaRTPInfo[MAX_RTP_SESSIONS];
extern UCHAR           aucSITBitMask[BitmapSize(MAX_SOURCES)];
extern SrcInfoTable_t  gaSrcInfoTable[MAX_SOURCES];
extern UCHAR           gucTotalActiveSess;
extern UCHAR           gaucActiveRTPSess[MAX_RTP_SESSIONS];
/***********************************************************************
 * Function    : initSeq.                                              *
 * Description : Initialises the source table.                         *
 * Input       : PerSrcInfo_t pSrc,UINT16 uiSeq                        *
 **********************************************************************/
VOID initSeq(PerSrcInfo_t *pSrc,UINT16 uiSeq)
{
  pSrc->ulBaseSeqNo = uiSeq;  
  pSrc->uiMaxSeqNo  = uiSeq;
  pSrc->ulLastBadSeqNo=RTP_SEQ_MOD +1;
  pSrc->ulCycles = 0;
  pSrc->ulPktsReceived=0;
  pSrc->ulPktsReceivedAtLastInterval=0;
  pSrc->ulPktsExpectedAtLastInterval=0;
  pSrc->ulProbation=PROBATION;
}
/***********************************************************************
 * Function    : updateSeq.                                            *
 * Description : This function ensures that a source is declared valid *
 * only after MIN_SEQUENTIAL packets have been received i sequence.    *
 * It also validates the sequence number of a newly  received  packet  *
 * and updates the sequence state for the packet's source in the       *
 * structure.                                                          *
 * Input       : PerSrcInfo_t *pSrc,UINT16                             *
 * Return      : SUCCESS / FAILURE.                                    *
 **********************************************************************/
INT32 updateSeq(PerSrcInfo_t *pSrc,UINT16 uiSeq)
{
  UINT16 uiDelta = uiSeq - pSrc->uiMaxSeqNo;
  /* Source is not valid until MIN_SEQUENTIAL packets with sequential
     sequence numbers have been received */
  if(pSrc->ulProbation)
  { 
    /* Packet is in sequence */
    if(uiSeq == pSrc->uiMaxSeqNo +1 )
    {
      pSrc->ulProbation--;
      pSrc->uiMaxSeqNo = uiSeq;
      if(pSrc->ulProbation == 0)
      {
        initSeq(pSrc,uiSeq);
        pSrc->ulPktsReceived++;
        return(SUCCESS);
      }
    }
    else
    {
      pSrc->ulProbation = MIN_SEQUENTIAL - 1; 
      pSrc->uiMaxSeqNo = uiSeq;
    }
    if(pSrc->ulPktsReceived == 0) 
      return SUCCESS;
    return(FAILURE);
  }
  else
  {
    if(uiDelta < MAX_DROPOUT) /* In order with permissible gap */
    {
      if(uiSeq < pSrc->uiMaxSeqNo )
        pSrc->ulCycles += RTP_SEQ_MOD;
      pSrc->uiMaxSeqNo = uiSeq;
    }
    else
    {
      if(uiDelta <= RTP_SEQ_MOD - MAX_MISORDERED)
      {
        if(uiSeq == pSrc->ulLastBadSeqNo) /*If other side restarted without
                                              telling us, so just re-sync */
          initSeq(pSrc,uiSeq);
        else
        {
          pSrc->ulLastBadSeqNo = (uiSeq + 1) & (RTP_SEQ_MOD-1);
          return(FAILURE);
        }
      }
      else
      {
        //printk("\n\r Duplicate packets");
        /* Duplicate packets */
      }
    }
  }
  pSrc->ulPktsReceived++;
  return(SUCCESS);
}
/***********************************************************************
 * Function        : rtpParseMsg.                                      *
 * Description     : The function receives RTP packet for specified    *
 *      session.The RTP packet is received on  the port corresponding  *
 *      to the SessionId. The received packet header is also validated.*
 * Input           : INT16 iIndex  - session index                     *
 *                   WtRtpPktType_ - packet buffer                     *
 * Output          : None.                                             *
 * Return          : SUCCESS or FAILURE                                *
***********************************************************************/
INT32 rtpParseMsg(RTPInfo_t *pRtpInfo, UCHAR *pRTPData,
                                  UINT16 uiLen, VoiceNWPkt_t *pJitPkt)
{
  RTPHdr_t *pcPkt=(RTPHdr_t*)pRTPData;
  UINT16 uiHdrLength;
  INT16 iCount;
  UINT16 uiExtLen;
  INT32 lFreeIndex=0;  //???
#if KRTCP_ENABLED
  UINT32 ulTransit=0,ulArrival=0,ulTempArrivSec=0, ulTempArrivFrac=0;
  INT32 lDiff=0;
#endif
  BOOL bSSRCFound = FALSE;
  INT32 iOctRecvd=0;
  INT16 iIndex = pRtpInfo->iSessionIndex;


  if(uiLen <= 0)
  {
    printk("RTP Receiving Error");
    return(FAILURE);
  }
  if(gaRTPInfo[iIndex].ucSessType == DSP_MEDIA_DATA)  // T.38
     return SUCCESS;
#if KRTCP_ENABLED
  if(rtpGetTimeOfDay(&ulTempArrivSec, &ulTempArrivFrac) != SUCCESS)
  {
    printk("Can not get time in RTP");
    return(FAILURE);
  }
#endif
  /* Check validity of RTP Header */
  if(pcPkt->uiVersion != 2 )
  {
    printk("Version != 2\n");
    return(FAILURE);
  }
  pJitPkt->CodecType = pcPkt->uiPt;
  pJitPkt->SeqNum = pcPkt->uiSeq;
  pJitPkt->timeStamp = pcPkt->ulTs;
  pJitPkt->channel  = pRtpInfo->ucEpIndex;
  /* If padding bit is set,check that last octet of packet is valid count*/
  /* Should be less than pkt size - rtp header size */
  uiHdrLength = 12 + (pcPkt->uiCC*4);
  /***********radded**********************/

  /* If extension bit is set, extension length field must be valid */
  if(pcPkt->uiP == 1)
  {
    if(pRTPData[uiLen-1] >= 4)
    {
      printk(" extension length invalid 1 \r\n");
      return(FAILURE);
    }
    if(pRTPData[uiLen-1] > (uiLen - uiHdrLength) )
    {
      printk(" extension length invalid 2 \r\n");
      return(FAILURE);
    }
  }
  if(pcPkt->uiX == 1)
  {
    uiExtLen= pRTPData[uiHdrLength + 2];
    uiExtLen=(uiExtLen << 8) | (pRTPData[uiHdrLength + 3]);
    if((uiExtLen*4) < (uiLen - uiHdrLength - (pRTPData[uiLen-1])))
    {
      printk(" extension length invalid 3 \r\n");
      return(FAILURE);
    }
    uiHdrLength += (uiExtLen*4)+4;
  }
  pJitPkt->FrameLen = uiLen - uiHdrLength;
  if(pcPkt->uiP == 1)
     pJitPkt->FrameLen -= pRTPData[uiLen-1];
  iOctRecvd = pJitPkt->FrameLen;
  /* Check if an RTP packet is already received from this SSRC */
  for(iCount=0;iCount < MAX_SOURCES ;iCount++)
  {
    if(CheckElement(aucSITBitMask, MAX_SOURCES, iCount) == ELE_ALLOCATED)
    {
      /* June 24 2004. Second check is needed if multiple RTP streams have
         same SSRC value. This is done by verifying the session index */
      if(pcPkt->ulSSRC == gaSrcInfoTable[iCount].ulSSRC &&
                gaSrcInfoTable[iCount].iSessionIndex == iIndex )
      {
        bSSRCFound=TRUE;
        break;
      }
    }
  }
  /* If known SSRC fill the SrcInfo table  */
  if(bSSRCFound == TRUE)
  {
    if(updateSeq(&gaSrcInfoTable[iCount].PerSrcInfo,pcPkt->uiSeq) == FAILURE)
    {
      /* I am putting here the probabtion period here, we need to re-visit it 
         for FAX, if required */
      return(FAILURE);  
    }
    gaSrcInfoTable[iCount].bPktRecvdInRTCPInterval=TRUE;
    gaSrcInfoTable[iCount].ulLastRecvdSeqNo = pcPkt->uiSeq;
    gaSrcInfoTable[iCount].PerSrcInfo.ulOctetsReceived += iOctRecvd;
  }
  else
  {
#if SNMP_RTP
    /* This is the new sender to RTP Sess */
    gaRTPInfo[iIndex].ulSenderJoins++;
    gaRTPInfo[iIndex].ulRcvrJoins++;
#endif
    /* If first RTP packet, allocate a new element */
    if(GetFreeElement(aucSITBitMask,MAX_SOURCES,&lFreeIndex) == FAILURE)
    {
      printk("FAILED in geting SIT Bitmap");
      return(FAILURE);
    }
    memset(&gaSrcInfoTable[lFreeIndex],0,sizeof(SrcInfoTable_t));
    /* Update the table */
    gaSrcInfoTable[lFreeIndex].bValid = TRUE;
    gaSrcInfoTable[lFreeIndex].PerSrcInfo.ulOctetsReceived += iOctRecvd;
    gaSrcInfoTable[lFreeIndex].ulSSRC = pcPkt->ulSSRC;
    gaSrcInfoTable[lFreeIndex].iSessionIndex = iIndex;
    gaSrcInfoTable[lFreeIndex].PerSrcInfo.ulProbation = PROBATION;
    gaSrcInfoTable[lFreeIndex].ulLastRecvdSeqNo = pcPkt->uiSeq;
    initSeq(&gaSrcInfoTable[lFreeIndex].PerSrcInfo,pcPkt->uiSeq);
#if SNMP_RTP
    gaSrcInfoTable[lFreeIndex].ulStartTime = rtpGetCurrTime();
#endif
    if(updateSeq(&gaSrcInfoTable[lFreeIndex].PerSrcInfo,pcPkt->uiSeq)
                                                           == FAILURE)
    {
       return(FAILURE);
    }
    iCount = lFreeIndex;
  }
#if KRTCP_ENABLED
  /* Calculate transit time and Interarrival jitter */
  ulArrival = (ulTempArrivSec * 8000) - gaRTPInfo[iIndex].ulOffset;
  ulTransit = ulArrival - pcPkt->ulTs;
  lDiff=ulTransit - gaSrcInfoTable[iCount].PerSrcInfo.ulTransitTimeforPrevPkt;
  gaSrcInfoTable[iCount].PerSrcInfo.ulTransitTimeforPrevPkt = ulTransit;
  lDiff=abs(lDiff);
  /* refer to A.8 in RTP RFC 1889. To keep Jitter estimates in Integer this
  * expression is used. */
  gaSrcInfoTable[iCount].PerSrcInfo.ulJitter += ((UINT32)lDiff) -
              ((gaSrcInfoTable[iCount].PerSrcInfo.ulJitter + 8) >> 4);
#endif
  return(SUCCESS);
}
/***********************************************************************
 * Function    : rtpFrameMsg.                                          *
 * Description : The function frames RTP header and send sends the RTP * 
 *     packet. The function also fills the report table which will be  *
 *     used by RTCP.                                                   *
 * Input       : RTPInfo_t *pRtpInfo - Pointer to session.             *
 *               CHAR *pcData        - Data to be sent out.            *
 *               INT16 iLen          - Lengthe of data.                *
 *               UINT32 ulTimestamp  - Time Stamp.                     *
 *               BOOL bMarkerBit     - Marker Bit.                     *
 * Output      : None.                                                 *
 * Return      : 0 on SUCCESS and -1 on FAILURE                        *
 **********************************************************************/
INT32 rtpFrameMsg(VoiceQ_t *pVoice, UCHAR ucEpIndex)
{
  INT16 iPadLen;
  INT16 iLen = pVoice->ucPktLen;
  RTPHdr_t  *RTPHdr;
#if KRTCP_ENABLED
  UINT32  ulCurrentNTPTimeSec;
  UINT32  ulCurrentNTPTimeFrac;
#endif
  INT32   iOctetsToSent = 0;
  INT32 iRet;
  RTPInfo_t *pRtpInfo = NULL;
  UCHAR ucSessIndex;
  UCHAR bMarkerBit = 0;
  
  if(gucTotalActiveSess == 0)
  {
    return FAILURE;
  }
  for(ucSessIndex = 0; ucSessIndex<gucTotalActiveSess ;ucSessIndex++)
  {
    pRtpInfo = &gaRTPInfo[gaucActiveRTPSess[ucSessIndex]];
    if(pRtpInfo->ucEpIndex == ucEpIndex)
    {
      if(pRtpInfo->ucMode == RTP_SENDONLY || pRtpInfo->ucMode == RTP_SENDRECV)
      {
        if(pRtpInfo->ucSessType == DSP_MEDIA_AUDIO)
        {
          RTPHdr = (RTPHdr_t*)pVoice->aucPayload;
          RTPHdr->uiVersion=RTP_VERSION;
          RTPHdr->uiP=0;
          RTPHdr->uiX=0;
          RTPHdr->uiCC=0;
          RTPHdr->uiM = ((bMarkerBit == TRUE) ? 1: 0);
          if(pRtpInfo->bRfc2833 == TRUE)
          {
            pRtpInfo->bRfc2833 = FALSE;
            iRet = pRtpInfo->uiDynPayloadType;
          }
          else
            iRet = pRtpInfo->uiPayloadType;
          if(iRet == FAILURE)
          {
            printk("Failed in geting rtp PT corresponding to = %d",
                                      pRtpInfo->uiPayloadType);
            return FAILURE;
          }
          RTPHdr->uiPt= iRet;
          RTPHdr->uiSeq = ++(pRtpInfo->uiSeqNum);
          RTPHdr->ulTs = pVoice->ulTimeStamp;  
          RTPHdr->ulSSRC = pRtpInfo->ulSSRC;
#if KRTCP_ENABLED
          if(rtpGetTimeOfDay((INT32*)&ulCurrentNTPTimeSec,
                 (INT32*)&ulCurrentNTPTimeFrac) == FAILURE)
          {
            printk("rtpSendData: Error in rtpGetTimeOfDay\n");
            return(FAILURE);
          }
          /* offset = (in32)(current_NTP_timestamp * 
            RTP_timestamp_clock_rate) - RTP_timestamp */
          pRtpInfo->ulOffset=(INT32)(ulCurrentNTPTimeSec * 8000)-RTPHdr->ulTs;
#endif
          iOctetsToSent = iLen;
          /* Find the number of octets to be padded */
          iPadLen = 0; //((iLen % 4) == 0) ? 0 : (4 - (iLen % 4));
          if(iPadLen > 0)   /* If data length is not multiple of 4 */
          {
            iLen += iPadLen;
            /* Last octet should contain Padlen */
            *(&pVoice->aucPayload[sizeof(RTPHdr_t) + (iLen - 1)]) = iPadLen;
            RTPHdr->uiP=1; // Set padding flag
          }
          /* Same Socket can't be used here for Sending as well as
             receiving RTP. Receiving RTP Socekt gets closed upon 
             closing the Reverse Logical Channel.
          */
        } 
        /* Replace ucChNum with RTP Session */
        pVoice->ucChNum = ucSessIndex;
      }
    }
  }
  return(SUCCESS);
}
#endif
