/* ============================================================================
 *
 * Copyright (C) 2001 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ============================================================================
 */
#include <oswrapper.h>
#include <linux/config.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include "dspdrv.h"
#if FXO_MODULE
#include "../dsplib/CidRxCon.h"
#endif
#if KRTP
#include "../krtp/rtpgdef.h"
#endif

/* LUA MODIF FR Bug 1108 { */
#include <linux/vmalloc.h>
/* { */

#include "voiceSysCtl.h"

#if DRV_DEBUG_STAT
VoiceDebugStat_t VoiceDebugStat[MAX_VOICE_CHANNELS];
#endif
/** Base statistics. */
extern struct dsp_voiceSysCtl_str dsp_voiceSysCtl;

#define DSP_DEVICE	"/dev/dsp"
unsigned char gDebugLevel  ;
CallerIdInfo_t gDspCallerIdInfo;
/* Driver file operations */
static int dsp_open(struct inode * inode, struct file * file);
static ssize_t dsp_write(struct file * file,
                         const char *buffer, 
                         size_t count, 
                         loff_t *ppos);
static ssize_t dsp_read(struct file * file,
                        char * buffer, 
                        size_t count, 
                        loff_t *ppos);
static int dsp_ioctl(struct inode *inode, 
                     struct file *filp, 
                     unsigned int cmd,
                     unsigned long param);
static unsigned int dsp_poll(struct file *file, 
                             poll_table * wait);
static int dsp_close(struct inode * inode, struct file * file);

/* SAGEM -> [ Start/ Stop DECT ] { */
#if DECT_MODULE
extern void dspipcStartDectTransCode(UINT16 portNum, UCHAR ucDectCodecType);
extern void dspipcStopDectTransCode(unsigned portNum);
#endif
extern SLICInfo_t gaSlicInfo[MAX_SLIC_CHANNELS];
extern VoiceCirQ_t gVoiceTxQ[MAX_VOICE_CHANNELS];
extern VoiceCirQ_t gaDsp2HostQ;
extern slicHandlers_t slicHandlers;
int g_initCompleted=0;
int g_nbDspOpen=0;
int g_nbVoiceOpen=0;
int g_nbFxoOpen=0;
/* } */

Word16 mesgWaitingIndMain(uWord16 CurChanNum,CidParams_t *gMwiParams);
void dsp_init_part2(void);

struct file_operations dsp_fops = {
        open: dsp_open,
        write: dsp_write,
        read:  dsp_read,
        ioctl: dsp_ioctl,
 	release: dsp_close,
        poll:  dsp_poll
};
wait_queue_head_t  voice_wait_queue;
static  wait_queue_head_t  dsp_wait_queue;
#if DYNAMIC_CODEC_DOWNLOAD
UINT16  *pFaxOnlyImage;
UINT16  *pG723OnlyImage;
UINT16  *pG729OnlyImage;
UINT16  *pG711Vad2OnlyImage;
UINT16  *p3wayOnlyImage;
extern UCHAR dspDynDownloadStart;
#endif

#if T38
static  wait_queue_head_t  t38_wait_queue;
#endif

#if FXO_MODULE
static  wait_queue_head_t  fxo_wait_queue;
#endif

#if DECT_TESTING
UCHAR dectFlag[MAX_VOICE_CHANNELS] = {0,0};
#endif
#if CALLER_IDTx
 extern void dtmfCidInit(UCHAR *aucStr,Word16 portNum);
#endif

extern UINT16 dspTimeSlot0,dspTimeSlot1;
/****************************************************************
 *  Function Name : dsp_ioctl.                                  *
 *  Description   : DSP Module ioctl Function.                  *
 *  Output        : Output data, if any                         *
 *  Return Value  : Number of bytes read.                       *
 ***************************************************************/
static int dsp_ioctl(struct inode *inode, 
                     struct file *filp, 
                     unsigned int cmd, 
                     unsigned long param)
{
  unsigned int minor = MINOR(filp->f_dentry->d_inode->i_rdev);
  
  if(minor == VOICE_MINOR)
  {
    switch(cmd)
    {
#if KRTP
      case DSP_RTP_CMD:
        rtpProcessControlMessage(param);
        break;
#else
      case DSP_SET_DATA_MODE:
      {
        DSPDataMode_t dspMode;
      	copy_from_user(&dspMode,(UCHAR*)param,sizeof(dspMode));
        if(dspMode.mode >= 1 && dspMode.mode <= 4)
          dspipcSetDSPDataMode(dspMode.portNum, dspMode.mode);
        DEBUG(PRINT("DSP_SET_DATA_MODE => %d   %d\n",dspMode.mode,dspMode.portNum),
                                                                     DEBUG_COMMANDS);
        break;
      }
      case DSP_CLEAR_VOICE_BUFFER:
        dspipcClearQs((UCHAR)param);
        DEBUG(PRINT("DSP_CLEAR_VOICE_BUFFER => %u\n",(int)param),DEBUG_COMMANDS);
        break;
      case DSP_SET_DATA_MODE_ENABLE:
        dspipcSetDataMode((UCHAR)param);
	DEBUG(PRINT("DSP_SET_DATA_MODE_ENABLE => %u \n",(int)param),DEBUG_COMMANDS);
      break;
#endif
      default:
        DEBUG( PRINT("NOT A VALID COMMAND on VOICE DEVICE : %x\n",cmd), 
                                        DEBUG_COMMANDS | DEBUG_EXCEPTION );
	return -1;
    }
    return 0; 
  }
  else if(minor == DSP_MINOR)
  {
    switch(cmd)
    {
/* SAGEM -> [ Start/ Stop DECT Transcoding ] { */
#if DECT_MODULE
      case DSP_START_TRANSCODE:
        dspipcStartDectTransCode(1, 0);
        break;
      case DSP_STOP_TRANSCODE:
       dspipcStopDectTransCode(1);
        break;
#endif
/* } */
#if DRV_DEBUG_STAT
      case DSP_GET_VOICE_STAT:
        copy_to_user((UCHAR*)param,VoiceDebugStat,sizeof(VoiceDebugStat));
        break;
#endif
      case DSP_SET_DEBUG_LEVEL: /* set debug levels */
        gDebugLevel = (unsigned char)param;
        break;
      case DSP_GET_DEBUG_LEVEL: /* Get debug levels */
        copy_to_user((UCHAR*)param,&gDebugLevel,sizeof(gDebugLevel));
        break;
      case DSP_INIT_SYSTEM: /* DSP Download Image */
        dsp_init_system();
        DEBUG( PRINT("DSP System Init \n"), DEBUG_COMMANDS);
        break;
      case DSP_INIT_PORT: /* Initialize DSP Port */
        {
        PortVoiceOpts_t voiceopts;
/* SAGEM -> [Warning] { */
#if DECT_TESTING
        PortVoiceOpts_t voiceopts1;
#endif
        copy_from_user(&voiceopts,(char*) param, sizeof(voiceopts));
#if DECT_TESTING
        if( (voiceopts.ucPort == 0) && (voiceopts.voiceOptions.encoder_opt[0] != 0x8d00))
        {
          voiceopts1.voiceOptions.encoder_opt[0] = 0x8d00;
          voiceopts1.voiceOptions.encoder_opt[1] = 0;
          voiceopts1.voiceOptions.encoder_opt[2] = 0;
	  voiceopts1.voiceOptions.decoder_opt[0] = 0x8c80;
	  voiceopts1.voiceOptions.decoder_opt[1] = 0;
          voiceopts1.voiceOptions.maxFrames = 1;
          dspipcInitPort(0,&voiceopts1.voiceOptions);
          dspipcInitPort(1,&voiceopts.voiceOptions);
          dspipcSetDataMode(1);
          dspipcSetDSPDataMode(1,DSP_DATA_SEND_RECV);
          dspipcClearQs(1);
          dspipcStartDectTransCode(1,1);
          dectFlag[1] = 1;
          dectStartTrans();
          printk("Started Transcoding \r\n");
        }
        else
#endif
        dspipcInitPort(voiceopts.ucPort, &voiceopts.voiceOptions);  
        DEBUG(PRINT(" DSP Init Port :  \nPort: %d\nEncoder Options: %x %x %x\n Decoder Options: %x %x\nPkt Period: %d\r\n",
                      voiceopts.ucPort,
                      voiceopts.voiceOptions.encoder_opt[0],
                      voiceopts.voiceOptions.encoder_opt[1],
                      voiceopts.voiceOptions.encoder_opt[2],
                      voiceopts.voiceOptions.decoder_opt[0],
		      voiceopts.voiceOptions.decoder_opt[1],
                      voiceopts.voiceOptions.maxFrames),DEBUG_COMMANDS);
        }
        break;
      case DSP_GEN_TONE:
        {
        DSPGenToneInfo_t toneInfo;
        copy_from_user(&toneInfo, (UCHAR*)param, sizeof(DSPGenToneInfo_t));
        if(toneInfo.toneType == DSP_FOUR_CADENCE_TONE)
        {
           dspipcToneCfg(toneInfo.portNum,&toneInfo.tone.forCadInfo);
           dspipcGenToneSpec(toneInfo.portNum,
                    toneInfo.tone.forCadInfo.ulToneId,toneInfo.tone.forCadInfo.ulCadenceDuration);
           DEBUG(PRINT("DSP_GEN_TONE [Cadence Tone] => \nPort: %d\n Tone-ID %d\n Duration: %d\n",toneInfo.portNum,
			toneInfo.tone.forCadInfo.ulToneId,toneInfo.tone.forCadInfo.ulCadenceDuration), DEBUG_COMMANDS);
        }
        else if(toneInfo.toneType == DSP_LOCAL_TONE)
        {
          dspipcGenTone(toneInfo.portNum, toneInfo.tone.local.toneId,
                                               toneInfo.tone.local.timeout);
          DEBUG(PRINT("DSP_GEN_TONE [local] => \n Port: %d\n ToneID: %d\n Timeout: %d\n",
                       toneInfo.portNum,
                       toneInfo.tone.local.toneId,
                       toneInfo.tone.local.timeout), DEBUG_COMMANDS);
        }
         else if(toneInfo.toneType == SLIC_LOCAL_TONE)
        {
          fxsGenTone(toneInfo.portNum, BUSY1_TONE );
          DEBUG(PRINT("DSP_GEN_TONE [local] => \n Port: %d\n ToneID: %d\n Timeout: %d\n",
                       toneInfo.portNum,
                       toneInfo.tone.local.toneId,
                       toneInfo.tone.local.timeout), DEBUG_COMMANDS);
        }
        else 
        {
          DEBUG(PRINT("DSP_GEN_TONE => Invalid Type pf Tone [%d]\n", toneInfo.toneType),
				DEBUG_EXCEPTION);
          return -1;
        }
        }
        break;
      case DSP_QUIT_TONE:
        {
        UCHAR aucTemp[4];
        copy_from_user(aucTemp, (UCHAR *)param, 4);
        dspipcQuitTone(aucTemp[0], aucTemp[1]);
        DEBUG(PRINT("DSP_QUIT_TONE => \n Port : %d\n ToneType: %d\n",aucTemp[0],aucTemp[1]),
                     DEBUG_COMMANDS);
        }
        break;
      case DSP_SET_MODIFYCODEC:
        {
        PortVoiceOpts_t voiceopts;
        copy_from_user(&voiceopts,(char*) param, sizeof(voiceopts));
        dspipcModifyPortCodec(voiceopts.ucPort, &voiceopts.voiceOptions);  
        DEBUG(PRINT(" DSP Modify Port :  \nPort: %d\nEncoder Options: %x %x %x\n Decoder Options: %x %x\nPkt Period: %d\r\n",
                      voiceopts.ucPort,
                      voiceopts.voiceOptions.encoder_opt[0],
                      voiceopts.voiceOptions.encoder_opt[1],
                      voiceopts.voiceOptions.encoder_opt[2],
                      voiceopts.voiceOptions.decoder_opt[0],
		      voiceopts.voiceOptions.decoder_opt[1],
                      voiceopts.voiceOptions.maxFrames),DEBUG_COMMANDS);
        }
        break;
      case DSP_QUIT_FSK_PLAY:
#if CALLER_IDTx
        resetCidTxPlay((UCHAR) param);
        dspipcCidTxPlayDone((UCHAR) param);
	DEBUG(PRINT("DSP_QUIT_FSK_PLAY => \nPort Number: %d\n",(int)param),DEBUG_COMMANDS);
#else
        DEBUG(PRINT("Caller-ID module is not enabled in driver \n"), DEBUG_EXCEPTION);
#endif
        break;
      case  DSP_GAIN_ADJUST:
        { 
	UINT16 aucMsgBuff[3], ucPort, ucGain,ucBlkGain;
        copy_from_user((UCHAR*)aucMsgBuff, (UCHAR*)param, 6);
        ucPort = aucMsgBuff[0];
        ucGain = aucMsgBuff[1];
        ucBlkGain = aucMsgBuff[2];
        dspipcGainCntrl(ucPort,ucGain,ucBlkGain);
        DEBUG(PRINT("DSP_GAIN_ADJUST => \n Port: %d\n Gain: %x\n BlkGain: %x\n",
			ucPort,ucGain,ucBlkGain), DEBUG_COMMANDS);
        break;
        }   
      case DSP_SWITCHTO_PASSTHRU:
        switchJitterBuffToFjb((UCHAR)param);
        confSetPLC(((UCHAR) param),0);
        break;
      case DSP_CLOSE_PORT:
        DEBUG(PRINT("DSP_CLOSE_PORT \n Port: %d\n",(int)param),DEBUG_COMMANDS);
        dspipcClosePort((UCHAR) param);
#if DECT_TESTING
	if( (param == 0) && (dectFlag[1] == 1) )
        {
          dspipcStopDectTransCode(1); 
          dspipcClosePort(1);
          dectStopTrans();
          printk("Stopped Transcoding \r\n");
	  dectFlag[1] = 0;
        }
#endif
        break;
      case DSP_SLIC_RING:
#if SLIC_MODULES
        {
          DSPRingInfo_t ring;
          copy_from_user((UCHAR*)&ring,(UCHAR*)param,sizeof(ring));
          slicVoiceRing(ring.portNumber, ring.timeout);
	  DEBUG(PRINT("DSP_SLIC_RING => \n Port: %d\n Ring Timeout: %d\n",
		ring.portNumber,ring.timeout), (DEBUG_COMMANDS | DEBUG_SLIC));
        }
#else 
	DEBUG(PRINT("DSP_SLIC_RING => SLIC Modules not enabled \n"), DEBUG_EXCEPTION);
#endif
        break;
      case SLIC_RING_USER_CONTROLLED:
        slicUserControlRing(param);
        DEBUG(PRINT("DSP_SLIC_RING => \n Port: %d\n",
                (UCHAR)param), (DEBUG_COMMANDS | DEBUG_SLIC));
        break;
      case SLIC_RING_OFF_USER_CONTROLLED:
        slicUserControlRingOff(param);
        DEBUG(PRINT("DSP_SLIC_RING => \n Port: %d\n",
                (UCHAR)param), (DEBUG_COMMANDS | DEBUG_SLIC));
        break;
      case DSP_SLIC_RING_OFF:
#if SLIC_MODULES
	DEBUG(PRINT("DSP_SLIC_RING_OFF => \n Port: %d\n",(int)param),
                                                (DEBUG_COMMANDS | DEBUG_SLIC));
        slicVoiceRingOff((UCHAR)param, FALSE);
#else 
	DEBUG(PRINT("DSP_SLIC_RING_OFF => SLIC Modules not enabled \n"), DEBUG_EXCEPTION);
#endif
        break;
      case DSP_SLIC_STAND_BY:
//        slicStandby((UCHAR) param);
        break;
      case DSP_SLIC_ACTIVATE:
#if SLIC_MODULES
	DEBUG(PRINT("DSP_SLIC_ACTIVATE=> \n Port: %d\n",(int)param), 
                                                (DEBUG_COMMANDS | DEBUG_SLIC));

// SAGEM LCe 03-01-07
//        slicVoiceRingOff((UCHAR) param,TRUE);
	slicIdlePol((UCHAR) param);
        slicActivate((UCHAR) param);
// SAGEM LCe 03-01-07
#else 
	DEBUG(PRINT("DSP_SLIC_ACTIVATE => SLIC Modules not enabled \n"), DEBUG_EXCEPTION);
#endif
        break;

// SAGEM LCe 03-01-07
      case DSP_SLIC_REVERSE_ACT:
#if SLIC_MODULES
	DEBUG(PRINT("DSP_SLIC_REVERSE_ACT=> \n Port: %d\n",(int)param), 
                                                (DEBUG_COMMANDS | DEBUG_SLIC));

        slicReverseActivate((UCHAR) param);
#else 
	DEBUG(PRINT("DSP_SLIC_REVERSE_ACT => SLIC Modules not enabled \n"), DEBUG_EXCEPTION);
#endif
        break;
// SAGEM LCe 03-01-07

      case DSP_SLIC_REC_GAIN:
	{
//        UCHAR aucTemp[2];
        unsigned char aucTemp[2];
        copy_from_user(aucTemp, (UCHAR *)param, 2);
	slicGainCtrl(aucTemp[0],aucTemp[1]);
	//si3216_gain_ctrl(param[0],level);
	}
	break;

      case DSP_FAR_END_TONE_GENERATE_ON:
	dspipcGenFarEndTone( (UCHAR)param);
        DEBUG(PRINT("DSP_FAR_END_TONE_GENERATE_ON => \n Port : %d\n",(int)param),
                                                                     DEBUG_COMMANDS);
        break;
      case DSP_FAR_END_TONE_GENERATE_OFF:
	dspipcStopFarEndTone( (UCHAR)param);
        DEBUG(PRINT("DSP_FAR_END_TONE_GENERATE_OFF => \n Port : %d\n",(int)param), 
                                                          DEBUG_COMMANDS);
        break; 
      case DSP_MEM_WRITE:
      {
        VoiceMem_t mem;
        copy_from_user((UCHAR*)&mem, (UCHAR*)param, sizeof(mem));
        dspipcMemWrite(&mem);
      }
        break;
      case DSP_MEM_READ:
      {
        VoiceMem_t mem;
        copy_from_user((UCHAR*)&mem, (UCHAR*)param, sizeof(mem));
        dspipcMemRead(&mem);
        copy_to_user((UCHAR*)param,(UCHAR*)&mem, sizeof(mem));
      }
        break;
      case DSP_SET_COUNTRYOPT:
      case DSP_SET_GCPARAM:
      case DSP_SET_DWNLOADOPTS:
      case DSP_SET_PCMMODE:
      case DSP_SET_VOICEOPT:
      case DSP_SET_PLC:
      case DSP_SET_VAD:
      case DSP_SET_EC:
      case DSP_SET_JBPARAMS:
      case DSP_SET_FAX_MODE:
        {
        VOICELS voice;
        memset(&voice,0,sizeof(voice));
        copy_from_user((UCHAR*)&voice,(UCHAR*)param,sizeof(voice));
      	voiceCliCmd(&voice,cmd);
        }
        break;
      case DSP_GET_COUNTRYOPT:
      case DSP_NEXT_GCPARAM:
      case DSP_GET_VOICEOPT:
      case DSP_GET_DWNLOADOPTS:
      case DSP_GET_PLC:
      case DSP_GET_VAD:
      case DSP_GET_EC:
      case DSP_GET_JBPARAMS:
      case DSP_GET_FAX_MODE:
#if THREEWAY_CONFERENCE
      case DSP_SET_CONFERENCE: /* Enable Conference */
      case DSP_GET_CONFERENCE: /* Enable Conference */
#endif
        {
        VOICELS voice;
        memset(&voice,0,sizeof(voice));
        copy_from_user((UCHAR*)&voice,(UCHAR*)param,sizeof(voice));
      	voiceCliCmd(&voice,cmd);
        copy_to_user((UCHAR*)param,&voice,sizeof(voice));
        
        }
        break;
      case DSP_SAVE_CONF:
        DEBUG(PRINT("DSP_SAVE_CONF\n"),DEBUG_CONF);
        return save_voice_conf();
      case DSP_GET_CONF:
        {
        VOICELS voice;
        copy_from_user(&voice,(UCHAR*)param,sizeof(voice));
        restore_voice_conf(&voice);
        copy_to_user((UCHAR*)param,(UCHAR*)&voice,sizeof(voice));
        DEBUG(PRINT("DSP_GET_CONF\n"),DEBUG_CONF);
        }
        break;
// Sagem ICC : Bug 3242 2006/01/09     
      case DSP_MWIIND :
        {
        UCHAR aucTemp[2];
        CidParams_t gMwiParams;   

        copy_from_user(aucTemp, (UCHAR *)param, 2);
        gMwiParams.ucVisualInd = aucTemp[1];
        mesgWaitingIndMain(aucTemp[0],&gMwiParams);  
        }
      break;
     
      case DSP_CALLERID:
      {     
#if CALLER_IDTx
        copy_from_user(&gDspCallerIdInfo, (UCHAR*)param, sizeof(CallerIdInfo_t));
	if (gDspCallerIdInfo.CidTxMode == FSK_CID)
	{
        if(gDspCallerIdInfo.CidType != 2)
        {
          dspipcSetDSPDataMode(gDspCallerIdInfo.uiCurChanNum,DSP_DATA_RECV_ONLY);
        }
        CidTxMain(gDspCallerIdInfo.uiCurChanNum,gDspCallerIdInfo.CidTxFlag,
           gDspCallerIdInfo.aucRemoteUserNum,gDspCallerIdInfo.aucRemoteUserName,
           gDspCallerIdInfo.aucDateBuffStr,gDspCallerIdInfo.CidType,
           gDspCallerIdInfo.bFirstRingOff);
        DEBUG(PRINT("FSK DSP_CALLERID => \nPort: %d\n CidTxFlag: %d\n aucRemoteUserNum: %s\n aucRemoteUserName: %s\n aucDateBuffStr: %s\n CidType: %d\n bFirstRingOff: %d\n",
	   gDspCallerIdInfo.uiCurChanNum,gDspCallerIdInfo.CidTxFlag,
           gDspCallerIdInfo.aucRemoteUserNum,gDspCallerIdInfo.aucRemoteUserName,
           gDspCallerIdInfo.aucDateBuffStr,gDspCallerIdInfo.CidType,
           gDspCallerIdInfo.bFirstRingOff), DEBUG_COMMANDS);
	}
	else 	// DTMF_CID
		{
		dtmfCidInit (gDspCallerIdInfo.aucRemoteUserNum, gDspCallerIdInfo.uiCurChanNum);

        DEBUG(PRINT("DTMF DSP_CALLERID => \nPort: %d\n  aucRemoteUserNum: %s\n",
	   gDspCallerIdInfo.uiCurChanNum, gDspCallerIdInfo.aucRemoteUserNum), DEBUG_COMMANDS);
		}
#else
        DEBUG(PRINT("DSP_CALLERID => Caller ID Module is not enabled\n"), DEBUG_EXCEPTION);
#endif
      break;
      }
      case DSP_SET_DYN_DTMF_REJ:
        dspipcSetDtmfRej((UCHAR)param);
        DEBUG(PRINT("DSP_SET_DYN_DTMF_REJ => \n Port: %d\n",(int)param), DEBUG_COMMANDS);
        break;
#if THREEWAY_CONFERENCE
      case DSP_START_CONF: /* Start Conference */
        {
        PortVoiceOpts_t voiceopts;
        copy_from_user(&voiceopts,(char*) param, sizeof(voiceopts));
        DEBUG(PRINT("DSP_START_CONF: %d\n",voiceopts.ucPort), DEBUG_COMMANDS);
        dspipcStartConference(voiceopts.ucPort, &voiceopts.voiceOptions);
        }
        break;
      case DSP_STOP_CONF: /* Stop Conference */
        {
        unsigned char ucPort;
        copy_from_user(&ucPort,(char*) param, sizeof(unsigned char));
        DEBUG(PRINT("DSP_STOP_CONF: %d\n",ucPort), DEBUG_COMMANDS);
        dspipcQuitConference(ucPort, 10);
        }
        break;
#endif
#if AJB_ENABLE_STATISTICS
     case AJB_DUMP_DEBUG_INFO:
        AJBDumpDebugInfo((UINT16*)param);
        break;
#endif
     default:
        DEBUG(PRINT("Unknown IOCTL Command %d\n",cmd), DEBUG_EXCEPTION);
	return -1;
    }
  }
#if T38
  else if(minor == T38_MINOR)
  {
    switch(cmd)
    {
      case DSP_SET_FAX_MODE:
	    {
        VOICELS voice;
        voice.ucPortNumber = param;
        voice.ucFaxMode = FAX_T38_METHOD;
        voiceCliCmd(&voice,cmd);
	    }
        break;

      case T38_RESET_QUEUE: /* Reset T38 Queue */
        dspipcResetT38Q((unsigned short)param);
        break;
      case DSP_SWITCHTO_T38:
#if DYNAMIC_CODEC_DOWNLOAD
        {
        UINT16  tempWord;
        /* To disable encoder path */
        tempWord = dspReadDataMem((unsigned short)param,0x3fb1);
        tempWord = tempWord | 0x0002; /* Set 2nd bit */
        dspWriteIntoDataMem((unsigned short)param,0x3fb1,tempWord);
        }
        dspipcChangeDSPImage((unsigned short)param, DSP_T38);
#endif
        dspipcSwitchToT38((unsigned short)param);
        break;
      default:
	return -1;
    }
  }
#endif
#if FXO_MODULE
  else if(minor == FXO_MINOR)
  {
    switch(cmd)
    {
      case FXO_LINE_STATUS:
      {
        int status = slicGetFXOState();
        copy_to_user((UCHAR*)param,&status,sizeof(int));
      }
      break;

      case FXO_GAIN_DECT:
      {
        int gain;
        copy_from_user(&gain, (int *)param, sizeof(int));
        si3050_Gain(gain);
      }
      break;

      case FXO_RESET_POLARITY:
	{
printk("FXO_RESET_POLARITY\n");
	si3050_reset_polarity();
	}
      break;

      case FXO_DSP_SET_CONNECTION:
      { 
        /* DSP connection between FXS-FXO */
        pstnDspCmdInfo_t  pstnDspInfo;
        copy_from_user((UCHAR*)&pstnDspInfo, (UCHAR *)param, sizeof(pstnDspCmdInfo_t));
        DEBUG(PRINT("FXO_DSP_SET_CONNECTION:PortNum%d:Len:%d \r\n",
                      pstnDspInfo.ucPortNo,pstnDspInfo.uiLen), DEBUG_COMMANDS);
        dspipcDSPConnection(&pstnDspInfo);
      }
      break;
      case FXO_DSP_QUIT_CONNECTION:
      {
        /* DSP disconnect between FXS-FXO */
        pstnDspCmdInfo_t  pstnDspInfo;
        copy_from_user(&pstnDspInfo, (UCHAR *)param, sizeof(pstnDspCmdInfo_t));
        if(pstnDspInfo.ucPortNo > MAX_VOICE_CHANNELS)
          return -1;
        DEBUG(PRINT("FXO_DSP_QUIT_CONNECTION:PortNum%d:Len:%d \r\n",
                      pstnDspInfo.ucPortNo,pstnDspInfo.uiLen), DEBUG_COMMANDS);
        dspipcQuitConnection(&pstnDspInfo);
      }
      break;
      case FXO_DSP_SLIC_CMD:
      {
        /* Commands to the FXO based on the FXS port */
        UCHAR aucTemp[2];
        copy_from_user(aucTemp, (UCHAR *)param, 2);
        if(aucTemp[0] > MAX_VOICE_CHANNELS)
          return -1;
        DEBUG(PRINT("FXO_DSP_SLIC_CMD:FXS-Port%d\n CMD: %d\n",
                      aucTemp[0], aucTemp[1]), DEBUG_COMMANDS);
        /* For FXO Caller ID, we always use DSP1 ir-respective what user has 
           provided the port */
        if(aucTemp[1] == FXO_ONHM_ENA || aucTemp[1] == FXO_ONHM_DIS)
          aucTemp[0] = 0;
        slicFXOCmd(aucTemp[0], aucTemp[1]);
      }
      break;
      case  FXS_GET_SLICSTATE:
      {
        /* Get State information for FXS connected to FXO Port */
        SlicStateInfo_t fxsState;
        copy_from_user((UCHAR*)&fxsState,(UCHAR*)param,sizeof(fxsState));
        if(fxsState.ucPortNum > MAX_VOICE_CHANNELS)
          return -1;
        slicGetHookStateInfo(&fxsState);
        DEBUG(PRINT("FXS_GET_SLICSTATE\n"),DEBUG_COMMANDS);
        copy_to_user((UCHAR*)param,&fxsState,sizeof(fxsState));
      }
      break;
      case FXO_DSP_DETECT_TONE:
      {
        /* Enable Tone Detection */
        UCHAR aucTemp[2];
        copy_from_user(aucTemp, (UCHAR *)param, 2);
        DEBUG(PRINT("FXO_DSP_DETECT_TONE: TONE=%d\r\n",aucTemp[1]), DEBUG_COMMANDS);
        confSetToneDetect(aucTemp[0],1);
        setToneDet(aucTemp[0],aucTemp[1]);
        ToneDetInit(aucTemp[0]);
      }
      break;
// BDt Boxer PSTN Call
      case FXO_FLASHHOOK:
      {
        /* Command a flash hook on FXO */
        UCHAR aucTemp[2];
        copy_from_user(aucTemp, (UCHAR *)param, 2);
        DEBUG(PRINT("FXO_FLASHHOOK: Delay=%d\r\n",aucTemp[1]), DEBUG_COMMANDS);
        /* already in communication FXS to FXO: don't need to command timeslot */
        si3050_FlashHook(aucTemp[1]);
      }
      break;
// BDt Boxer
#if CID_RX_DET
      case FXO_DSP_CALLERID_DETECT:
      {
        /* FXO Caller ID Detection Enable / Disable */
        UCHAR aucTemp[2];
        copy_from_user(aucTemp,(UCHAR *)param, 2);
        DEBUG(PRINT("FXO_DSP_CALLERID_DETECT: Port: %d\n  Enable : %d\n",
                       aucTemp[0],aucTemp[1]), DEBUG_COMMANDS);
       if(aucTemp[1])
        {
          cidrxAppInit(0);
          CidRxInit(0);
          Init_Pre_Process(0);
        }
#if 0
	else
		{ // stop CID detection so dump receive trame
		cidrx_dump();
		}
#endif
        confSetFXOCallerIdDetect(aucTemp[0],aucTemp[1]); 
      }
      break;
#endif
      default:
        return -1;
    }
  }
#endif
  else 
    return -1; /* Not a valid device */
  return 0;
}
/****************************************************************
 *  Function Name : dsp_write.                                  *
 *  Description   : DSP Module write Function.                  *
 *  Input         : file structure, size of output buffer,      *
 *                      Position.                               *
 *  Output        : Output data, if any                         *
 *  Return Value  : Number of bytes read.                       *
 ***************************************************************/
static ssize_t dsp_write(struct file * file,
                         const char * buffer, 
                         size_t count, 
                         loff_t *ppos)
{
#if (!KRTP || T38)
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
#endif

#if !KRTP
  /* Only for data */
  if(minor == VOICE_MINOR)
  {
    dspipcWriteVoicePkt((VoiceQ_t*)buffer);
    return count;
  }
  else
#endif
#if T38
  if(minor == T38_MINOR)
  {
    return dspipcWriteT38DataPkt((UCHAR*)buffer);
  }
#endif
  return 0; /* Did not write any thing  */
}

/****************************************************************
 *  Function Name : dsp_read.                                   *
 *  Description   : DSP Module read Function.                   *
 *  Input         : file structure, size of output buffer,      *
 *                      Position.                               *
 *  Output        : Output data, if any                         *
 *  Return Value  : Number of bytes read.                       *
 ***************************************************************/
static ssize_t dsp_read(struct file * file,
                        char * buffer, 
                        size_t count, 
                        loff_t *ppos)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
#if !KRTP
  if(minor == VOICE_MINOR)
  {
    /* Check if we have Voice data Packet */
    VoiceQ_t *pVoice;
    pVoice = dspipcGetVoicePkt(count,DSP_MEDIA_AUDIO);
    if(pVoice != NULL)
    {
      copy_to_user(buffer, (char *)pVoice, 24 + (pVoice->ucPktLen)); 
      DEBUG_ISR(PRINT("PKT-DRV: RD: %d\n",pVoice->ucChNum), DEBUGISR);
#if !KRTP
#if DRV_DEBUG_STAT
      VoiceDebugStat[pVoice->ucChNum].rtpPktSend++;
#endif
#endif
      return 24 + (pVoice->ucPktLen);
    }
  }
  else 
#endif
  if(minor == DSP_MINOR)
  {
    CmdDigitInfo_t *pCmd;
    /* Check if we have DSP Command Packet */
    if(dspipcGetDSPCmdCount() == 0)
      return 0;
    pCmd = dspipcGetCmdPkt();
    //printk("Read Command : %x  %x\n", pCmd->ConfirmCode, pCmd->ucChannelNo);
    DEBUG(PRINT("DSP_EVENT => \n Port: %d \nEvent: %d\n", pCmd->ucChannelNo,
			pCmd->ConfirmCode), DEBUG_EVENTS);
    copy_to_user(buffer, pCmd, sizeof(CmdDigitInfo_t)); 
    return sizeof(CmdDigitInfo_t);
  }
#if T38
  else if(minor == T38_MINOR)
  {
    t38Pkt_t *pT38;
    pT38 = dspipcGetT38Pkt(count);
    if(pT38 != NULL)
    {
      copy_to_user((UCHAR*)buffer,(UCHAR*)pT38,sizeof(t38Pkt_t));
      return 3 + pT38->ucPktLen;
    }
  }
#endif
#if FXO_MODULE
  else if(minor == FXO_MINOR)
  {
    CmdDigitInfo_t *pCmd;
    /* Check if we have FXO Event */
    if(dspipcGetFXOCmdCount() == 0)
      return 0;
    pCmd = dspipcGetFXOCmdPkt();
    DEBUG(PRINT("FXO-EVENT: %x  %x\n",pCmd->ConfirmCode,
              pCmd->ucChannelNo), DEBUG_EVENTS);
    copy_to_user(buffer, pCmd, sizeof(CmdDigitInfo_t));
    return sizeof(CmdDigitInfo_t);
  }
#endif
  return 0 ; /* We don't have any thing to do with RTP application */
}
/****************************************************************
 *  Function Name : dsp_poll.                                   *
 *  Description   : DSP Module poll Function.                   *
 *  Input         : poll table and file structure.              *
 *  Output        : None.                                       *
 *  Return Value  : 0 if no data otherwise mask.                *
 ***************************************************************/
static unsigned int dsp_poll(struct file *file, 
                             poll_table * wait)
{
  unsigned int mask = 0;
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  if(minor == DSP_MINOR)
  {
    /* Poll Only for DSP read */
    if(file->f_mode & FMODE_READ)
    {
      if(dspipcGetDSPCmdCount()  != 0)
      {
        mask |= POLLIN | POLLRDNORM;
      }
      else 
      {
        poll_wait(file, &dsp_wait_queue, wait);
        if(dspipcGetDSPCmdCount()  != 0)
        {
          mask |= POLLIN | POLLRDNORM;
        }
      }
    }
  }
#if !KRTP
  else if(minor == VOICE_MINOR)
  {
    /* Poll Only for DSP read */
    if(file->f_mode & FMODE_READ)
    {
      if(dspipcGetVoicePktCnt(1) != FAILURE)
      {
        mask |= POLLIN | POLLRDNORM;
      }
      else 
      {
#if DYNAMIC_CODEC_DOWNLOAD
        if(dspDynDownloadStart == 1)
          wakeupDynDownload();
#endif
        poll_wait(file, &voice_wait_queue, wait);
        if(dspipcGetVoicePktCnt(1) != FAILURE)
        {
          mask |= POLLIN | POLLRDNORM;
        }
      }
    }
  }
#endif
#if T38
  else if(minor == T38_MINOR)
  {
    if(dspipcGetT38PktCnt(1) != 0)
    {
      mask |= POLLIN | POLLRDNORM;
    }
    else 
    {
      poll_wait(file, &t38_wait_queue, wait);
      if(dspipcGetT38PktCnt(1) != 0)
      {
        mask |= POLLIN | POLLRDNORM;
      }
    }
  }
#endif
#if  FXO_MODULE
  else if(minor == FXO_MINOR)
  {
    if(dspipcGetFXOCmdCount()  != 0)
    {
      mask |= POLLIN | POLLRDNORM;
    }
    else
    {
      poll_wait(file, &fxo_wait_queue, wait);
      if(dspipcGetFXOCmdCount()  != 0)
      {
        mask |= POLLIN | POLLRDNORM;
      }
    }
  }
#endif
  return mask;
}
/****************************************************************
 *  Function Name : dsp_cmd_ready.                              *
 *  Description   : Function to invoke tasks, waiting for the   *
 *      DSP commands.                                           *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
void dsp_cmd_ready(void)
{
  /* If command is available, wakeup the tasks waiting for the 
     command */
  wake_up_interruptible(&dsp_wait_queue);
}
/****************************************************************
 *  Function Name : dsp_data_ready.                           *
 *  Description   : Function to invoke tasks, waiting for the   *
 *      DSP data.                                               *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
void dsp_data_ready(void)
{
  /* If data is available, wakeup the tasks waiting for the
     data */
  wake_up_interruptible(&voice_wait_queue);
}

#if FXO_MODULE
/****************************************************************
 *  Function Name : fxo_cmd_ready.                              *
 *  Description   : Function to invoke tasks, waiting for the   *
 *      FXO events.                                             *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
void fxo_cmd_ready(void)
{
  /* If command is available, wakeup the tasks waiting for the
     command */
  wake_up_interruptible(&fxo_wait_queue);
}
#endif
#if T38
/****************************************************************
 *  Function Name : t38_data_ready.                             *
 *  Description   : Function to invoke tasks, waiting for the   *
 *      T38 data.                                               *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
void t38_data_ready(void)
{
  /* If data is available, wakeup the tasks waiting for the
     data */
  wake_up_interruptible(&t38_wait_queue);
}
#endif
/****************************************************************
 *  Function Name : dsp_close.                                  *
 *  Description   : DSP Module Close Function.                  *
 *  Input         : inode and file structure.                   *
 *  Output        : None.                                       *
 *  Return Value  : 0 on Success else -EFAULT                   *
 ***************************************************************/
static int dsp_close(struct inode * inode, struct file * file)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev);
  if(minor == VOICE_MINOR || minor == DSP_MINOR
#if T38
    || minor == T38_MINOR
#endif
#if FXO_MODULE
    || minor == FXO_MINOR
#endif
                         )
  {
/* SAGEM -> [ Add reset port on close /Dev/dsp ] { */
	switch (minor) {
		case DSP_MINOR:
			/* Decrease Nb Open */
			g_nbDspOpen--;

			/* If Second part of Init completed and no more device open */
			if ( (g_initCompleted == 1) &&
				(g_nbDspOpen == 0) ) {
		 	 	UINT16 portNum = 0;

				/* Reset Fifo Event DSP */
				{
					gaDsp2HostQ.writeIdx = 0;
					gaDsp2HostQ.readIdx = 0;
				}

#if SLIC_MODULES
				/* Stop ring */
				for (portNum = 0; portNum < MAX_SLIC_CHANNELS; portNum++) {
					/* Stop ring */
    	    				slicVoiceRingOff(	portNum,
									TRUE);
				}
#endif

				/* Reset All Port */
				dspipcResetAllPorts();
	
#if SLIC_MODULES
				/* Insert in Event Fifo OFFHOOK */
				for (portNum = 0; portNum < MAX_SLIC_CHANNELS; portNum++) {
					/* If slic state stable */
					if (gaSlicInfo[portNum].slicStateChanged == FALSE) {
						/* OFFHOOK */
						if (gaSlicInfo[portNum].slicOffTheHook == SLIC_OFFHOOK) {
  							/* Call Handler */
							dspipcReportIndication(portNum,SLIC_OFFHOOK_EVENT);
						}
					}
				}
#endif
			}
			break;

#if FXO_MODULE
		case FXO_MINOR:
			/* Decrease Nb Open */
			g_nbFxoOpen--;

			/* If Second part of Init completed and no more device open */
			if ( (g_initCompleted == 1) &&
				(g_nbFxoOpen == 0) ) {
		 	 	UINT16 portNum = 0;

				/* Stop FXO Connection */
				for (portNum = 0; portNum < MAX_SLIC_CHANNELS; portNum++) {
					/* Stop FXO Connection */
					{
		        			pstnDspCmdInfo_t  _pstnDspInfo;
		
						memset(	&_pstnDspInfo, 0, sizeof(_pstnDspInfo));
						_pstnDspInfo.ConnectType	= TIMESLOT;
						_pstnDspInfo.ucPortNo	= portNum;
						_pstnDspInfo.uiLen		= 0;
		
						/* Stop FXO<->FXS connection */
    		    				dspipcQuitConnection(&_pstnDspInfo);
					}
	
					/* ON_HOOK FXO */
					slicFXOCmd(portNum, FXO_ONHOOK);
				}
	
				/* Reset Fifo Event FXO */
				dspipcResetFXOQ();
	
				/* If Silabs */
				if(slicHandlers.codecType == CODEC_SILABS) {
					si3050_initialize();
				}
			}
			break;
#endif

		case VOICE_MINOR:
			/* Decrease Nb Open */
			g_nbVoiceOpen--;

			/* If Second part of Init completed and no more device open */
			if ( (g_initCompleted == 1) &&
				(g_nbFxoOpen == 0) ) {
				printk("---> Not implemented yet !!!\n");
			}
			break;

		default:
			break;
	}
/* } */

#if  !LINUX_2_6
    MOD_DEC_USE_COUNT;
#endif
    return 0;
  }
  return -1;
}
    
/****************************************************************
 *  Function Name : dsp_open.                                   *
 *  Description   : DSP Module open Function.                   *
 *  Input         : inode and file structure.                   *
 *  Output        : None.                                       *
 *  Return Value  : 0 on Success else -EFAULT                   *
 ***************************************************************/
static int dsp_open(struct inode * inode, struct file * file)
{
  unsigned int minor = MINOR(file->f_dentry->d_inode->i_rdev); 

#ifndef DSP_INIT_NO_FRAGMENT
  if (!g_initCompleted) {
    dsp_init_part2();

    /* Init Completed */
    g_initCompleted=1;
  }

#endif
  if(minor == VOICE_MINOR || minor == DSP_MINOR 
#if T38
    || minor == T38_MINOR
#endif
#if FXO_MODULE
    || minor == FXO_MINOR
#endif
                         )
  {
/* SAGEM -> [ Free All ressource on close device ] { */
	switch (minor) {
		case DSP_MINOR:
			/* Increase Nb Open */
			g_nbDspOpen++;
			break;

		case VOICE_MINOR:
			/* Increase Nb Open */
			g_nbVoiceOpen++;
			break;

		case FXO_MINOR:
			/* Increase Nb Open */
			g_nbFxoOpen++;
			break;

		default:
			break;
	}
/* } */
#if !LINUX_2_6
    MOD_INC_USE_COUNT;
#endif

    /* dtr : Reset Fifo Event DSP */
    gaDsp2HostQ.writeIdx = 0;
    gaDsp2HostQ.readIdx = 0;
    return 0;
  }
  return -1;
}
/****************************************************************
 *  Function Name : dsp_driver_exit.                            *
 *  Description   : DSP Module exit Function.                   *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
void dsp_driver_exit(void)
{
  printk("\n");
  printk("##################################################\n");
  printk("# DSP Voice Module Unloading ...\n\n");
#ifdef VOICE_SYSCTL_STATS
  {
    printk("Unregister /sys/sagem/voice SysCtl ...");

    if (dsp_voiceSysCtl.head != (struct ctl_table_header *)0) {
      unregister_sysctl_table(dsp_voiceSysCtl.head);
    	 dsp_voiceSysCtl.head = (struct ctl_table_header *)0;
    }

    printk(" OK\n");
  }
#endif

  osDisableSystem();
  udelay(100); /* Just in case of pening Interrupts OR Tasklet */
  unregister_chrdev(DSP_MAJOR, DSP_DEVICE);

  printk("\n");
  printk("# DSP Voice Module Unloading Ok\n");
  printk("##################################################\n\n");

}
/****************************************************************
 *  Function Name : dsp_driver_init.                            *
 *  Description   : DSP Module Init Function.                   *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
extern int slic_init_procfs(void);	/* Sagem qualif only */

int dsp_driver_init(void)
{
  INT32 result;

  printk("\n");
  printk("##################################################\n");
  printk("# DSP Voice Module Part 1 Loading ...\n\n");

  result = register_chrdev(DSP_MAJOR, DSP_DEVICE,&dsp_fops);
  if(result != 0)
  {
    DEBUG(PRINT("Could not register the character device \n"),
          DEBUG_EXCEPTION);
    return -1;
  }

#ifdef VOICE_SYSCTL_STATS
  {
    printk("Register /sys/sagem/voice SysCtl ...");

    if ((dsp_voiceSysCtl.head = register_sysctl_table(	dsp_voiceSysCtl.sagem,
											0)) == 0) {
		printk("Error register_sysctl on /sys/sagem/voice\n");
		return -1;
    }

    printk(" OK\n");
  }
#endif

  slic_init_procfs();	/* Sagem qualif only */
  printk("Using: %s\n",RELEASE_INFO);
  gDebugLevel = ( ( 1 << DEBUG_INFO) | ( 1 << DEBUG_EXCEPTION));

  printk("\n");
  printk("# DSP Voice Module Part 1 Loading Ok\n");
  printk("##################################################\n\n");

#ifdef DSP_INIT_NO_FRAGMENT
  {
    /* Init Second Part */
    dsp_init_part2();

    /* Init Completed */
    g_initCompleted=1;
  }
#endif

  return 0;
}

void dsp_init_part2(void)
{
  printk("\n");
  printk("##################################################\n");
  printk("# DSP Voice Module Part 2 Loading ...\n\n");

    dspAssignIDMAReg();

  dsp_init_system();

#if DECT_TESTING
  dectSampleInit();
#endif

  printk("\n");
  printk("# DSP Voice Module Part 2 Loading Ok\n");
  printk("##################################################\n\n");
}

/****************************************************************
 *  Function Name : store_voice_conf.                           *
 *  Description   : configure DSP.                              *
 *  Input         : VOICELS Structure.                          *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
INT16 store_voice_conf(VOICELS *pVoice)
{
  /* Set Voice Options */
  voiceCliCmd(pVoice,DSP_SET_VOICEOPT);
  /* Set Country Option */
  voiceCliCmd(pVoice,DSP_SET_COUNTRYOPT); 
  /* Set Gain Control Parameter */
  voiceCliCmd(pVoice,DSP_SET_GCPARAM);
  /* Set DSP Download options */
  voiceCliCmd(pVoice,DSP_SET_DWNLOADOPTS);
  /* We have stored the configurations */
  /* Set Jitter Buffer Parameters */
  voiceCliCmd(pVoice,DSP_SET_JBPARAMS);
  voiceCliCmd(pVoice,DSP_SET_PCMMODE);
#if THREEWAY_CONFERENCE
  voiceCliCmd(pVoice, DSP_SET_CONFERENCE);
#endif
#if T38
  voiceCliCmd(pVoice,DSP_SET_FAX_MODE);
#endif
  voiceCliConfigStored();
  return 0;
}
/****************************************************************
 *  Function Name : restore_voice_conf.                         *
 *  Description   : Read the Configurations from DSP.           *
 *  Input         : VOICELS Structure.                          *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
INT16 restore_voice_conf(VOICELS *pVoice)
{
  /* Get all the Parameters from DSP */
  /* Get Voice Options */
  voiceCliCmd(pVoice,DSP_GET_VOICEOPT);
  /* Get SPORT codec type */
  voiceCliCmd(pVoice,DSP_GET_PCMMODE);
  /* Get Country Option */
  voiceCliCmd(pVoice,DSP_GET_COUNTRYOPT);
  /* Get Gain Control Parameter */
  voiceCliCmd(pVoice,DSP_NEXT_GCPARAM);
  /* Get DSP Download options */
  voiceCliCmd(pVoice,DSP_GET_JBPARAMS);
#if THREEWAY_CONFERENCE
  voiceCliCmd(pVoice, DSP_GET_CONFERENCE);
#endif
  return 0;
}
/****************************************************************
 *  Function Name : save_voice_con   f.                         *
 *  Description   : Write the Configurations to the file.       *
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : None.                                       *
 ***************************************************************/
INT16 save_voice_conf(void)
{
  VOICELS voice;
  int i;
  mm_segment_t old_fs;
  loff_t off = 0;
  struct file *fp;
  
  fp = filp_open(DSP_CONFIGURATION_FILE, O_CREAT|O_WRONLY, 0);
  if (IS_ERR(fp))
  {
    DEBUG(PRINT("Could not open DSP Configuration file [%s]\n",
		DSP_CONFIGURATION_FILE), DEBUG_EXCEPTION);
    return -1;
  }
#if FXO_MODULE
  for(i = 0; i< MAX_VOICE_CHANNELS + 1; i++)
#else
  for(i = 0; i< MAX_VOICE_CHANNELS; i++)
#endif
  {
    /* Get all the Parameters from DSP */
    voice.ucPortNumber = i;
    voice.lIndex = i;
    if((i&1) == 0)
      voice.ucDspNum = 0;
    if((i&1) == 1)
      voice.ucDspNum = 1;    
    /* Get Voice Options */
    voiceCliCmd(&voice,DSP_GET_VOICEOPT);
    /* Set Country Option */
    voiceCliCmd(&voice,DSP_GET_COUNTRYOPT);
    /* Set Gain Control Parameter */
    voiceCliCmd(&voice,DSP_NEXT_GCPARAM);
    /* Get DSP Download options */
    voiceCliCmd(&voice,DSP_GET_DWNLOADOPTS);
    /* Get PCM Mode */
    voiceCliCmd(&voice,DSP_GET_PCMMODE);
    voiceCliCmd(&voice,DSP_GET_JBPARAMS);
#if T38
    voiceCliCmd(&voice,DSP_GET_FAX_MODE);
#endif
#if THREEWAY_CONFERENCE
    voiceCliCmd(&voice, DSP_GET_CONFERENCE);
#endif

    old_fs = get_fs();
    set_fs(get_ds());
    
    if(fp->f_op->write(fp,(UCHAR*)&voice,sizeof(voice), &off) != sizeof(voice) )
    {
      DEBUG(PRINT("Could not write to file \n"), DEBUG_EXCEPTION);
      set_fs(old_fs);
      filp_close(fp,NULL);
      return -1;
    }
  }
  filp_close(fp,NULL);
  set_fs(old_fs);
  return 0;
}
/****************************************************************
 *  Function Name : dsp_config_default.                         *
 *  Description   : Read the DSP image file and downloads to DSP*
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : 0 on success else -1 on error               *
 ***************************************************************/
int dsp_config_default(void)
{
  VOICELS temp;
  int i;
  mm_segment_t old_fs;
  loff_t pos = 0;
  struct file *fp;

    old_fs = get_fs();
    set_fs(get_ds());
  fp = filp_open(DSP_CONFIGURATION_FILE, O_RDONLY, 0);
  if(IS_ERR(fp))
  {
    DEBUG(PRINT("Could not find DSP configuration file, setting to defaults\n"),
               DEBUG_INFO);
    goto set_default;
  }
  else 
  {
   // old_fs = get_fs();
   // set_fs(get_ds());
#if FXO_MODULE
    for(i=0; i<MAX_VOICE_CHANNELS + 1; i++)
#else
    for(i=0; i<MAX_VOICE_CHANNELS; i++)
#endif
    {
      pos = i*sizeof(temp);
      if(fp->f_op->read(fp,(UCHAR*)&temp, sizeof(temp), &pos) != sizeof(temp))
      {
        DEBUG(PRINT("File read error \n"),DEBUG_EXCEPTION);
        goto set_default;
      }
#if 0
      printk("Port Number : %d\n",temp.ucPortNumber);
      printk("VAD : %d\n",temp.bVad);
      printk("PLC : %d\n",temp.bPLC);
      printk("EC : %d\n",temp.bEC);
      printk("EC-Tail length : %d\n",temp.uiTailLen);
      printk("GTx : %d\n",temp.lGtx);
      printk("GRx : %d\n",temp.lGrx);
      printk("GEc : %d\n",temp.lGec);
      printk("Threshold : %d\n",temp.uiThreshold);
      printk("DSP Number : %d\n",temp.ucDspNum);
      printk("Default DSP Code : %x\n",temp.ucCountryOpt);
      printk("PCM Mode : %d\n",temp.ucPcmMode);
      printk("Pass Through Threshold: %d\n",temp.uiPassThrThreshold);
      printk("Country %d\n",temp.ucCountryOpt);
      printk("Download Opt: %x\n",temp.ucDownloadOpts);
#endif
      store_voice_conf(&temp);
      
    }
    set_fs(old_fs);
    if(fp != NULL)
      filp_close(fp, NULL);
    return 0;
  }

set_default:
#if FXO_MODULE
  for(i=0; i<MAX_VOICE_CHANNELS + 1; i++)
#else 
  for(i=0; i<MAX_VOICE_CHANNELS; i++)
#endif
  {
    memset(&temp,0,sizeof(temp));
    temp.ucPortNumber = i;
    temp.bPLC = FALSE;
    temp.bEC = TRUE;
    temp.bVad = FALSE;
    if((i&1) == 0) 
      temp.ucDspNum = 0;
    if((i&1) == 1) 
      temp.ucDspNum = 1;

    temp.ucDownloadOpts = DSP_PCMU;
    temp.uiTailLen = 30;
//VAD2    
	temp.ucVadType = 1;
//Sagem 20080226 : FXS_ToneDtmf_ACTIVE
#if !FXS_ToneDtmf_ACTIVE
    temp.ucCountryOpt = COUNTRY_NAME;
#else // FXS_ToneDtmf_ACTIVE
#ifdef FIRMWARE_CONF_COUNTRY_FR
    printk("**** Voice Configured for FRANCE Tones ****\n");
    temp.ucCountryOpt = FRANCE;
#elif defined(FIRMWARE_CONF_COUNTRY_NL)
    printk("**** Voice Configured for NETHERLANDS Tones ****\n");
    temp.ucCountryOpt = NETHERLANDS;
#elif defined(FIRMWARE_CONF_COUNTRY_POL)
    printk("**** Voice Configured for POLAND Tones ****\n");
    temp.ucCountryOpt = POLAND;
#elif defined(FIRMWARE_CONF_COUNTRY_SP)
    printk("**** Voice Configured for SPAIN Tones ****\n");
    temp.ucCountryOpt = SPAIN;
#else
    printk("**** Unknown Country ! --> Voice Configured for FRANCE Tones ****\n");
    temp.ucCountryOpt = FRANCE;
#endif
#endif // end FXS_ToneDtmf_ACTIVE

    temp.bMode = T38RELAY;
    temp.bRate = V27;
    temp.ucPcmMode = SPORT_CODEC;
    if(temp.ucCountryOpt == FRANCE)
/* SAGEM -> [ DIAL Tone gain Prb (power down) ] { */
#if 0
      temp.lGtx = -6;
#endif
      temp.lGtx = 0;
/* } */
    else
      temp.lGtx = 0;
    temp.lGrx = 0;
    temp.lGec = 0;
    temp.uiThreshold = 20;
    temp.bJBType = AJB;
#if THREEWAY_CONFERENCE
    temp.bConference = FALSE;
#endif
    temp.ucFaxMode = FAX_PASSTHRU_METHOD;
    store_voice_conf(&temp);
  }
  set_fs(old_fs);
  return 0;
}
/****************************************************************
 *  Function Name : dsp_init_system.                            *
 *  Description   : Read the DSP image file and downloads to DSP*
 *  Input         : None.                                       *
 *  Output        : None.                                       *
 *  Return Value  : 0 on success else -1 on error               *
 ***************************************************************/
int dsp_init_system(void)
{
  dsp_config_default();
#if DRV_DEBUG_STAT
  memset(VoiceDebugStat,0,sizeof(VoiceDebugStat));
#endif

#if DYNAMIC_CODEC_DOWNLOAD
  /* Store Dynamic Downloadable data */
  store_dynamic_image(&pFaxOnlyImage,  DSP_FAX_ONLY_IMAGE_NAME);
  store_dynamic_image(&pG723OnlyImage, DSP_G723_IMAGE_NAME);
  store_dynamic_image(&pG729OnlyImage, DSP_G729_IMAGE_NAME);
  store_dynamic_image(&pG711Vad2OnlyImage, DSP_G711VAD2_IMAGE_NAME);
  store_dynamic_image(&p3wayOnlyImage, DSP_3WAY_IMAGE_NAME);
#endif  
  /* Initialize DSP wait Queue */
  init_waitqueue_head(&dsp_wait_queue);
  init_waitqueue_head(&voice_wait_queue);

#if T38
  init_waitqueue_head(&t38_wait_queue);
  dspipcRegT38PacketFromDsp(t38_data_ready);
#endif

#if FXO_MODULE
  init_waitqueue_head(&fxo_wait_queue);
  dspipcRegFXOStateChangeHandler(fxo_cmd_ready);
  dspipcResetFXOQ();
#endif

  DEBUG(PRINT("\n Registering Call Back Handlers\n"),
                                        DEBUG_INFO);
                                                                                                 
  /* Register the function for Voice Data Packets */
  dspipcRegVoiceFromDsp(dsp_data_ready);
  dspipcRegCmdDigitFromDsp(dsp_cmd_ready);

  dsp_download_static_images();

  DEBUG(PRINT("\n Initializing SLIC and VOICE Wait Queues\n"),
                                        DEBUG_INFO);
  DEBUG(PRINT("\n Initializing Voice \n"), DEBUG_INFO);
  /* Initialize Voice Driver Module */
  dspipcSystemInit();
  DEBUG(PRINT("\n DSP System Initialized Successfully \n"), DEBUG_INFO);
#if KRTP
  rtpInit();
#endif
  return 0;
}

int parse_dsp_image_data(UCHAR *pData, int readCnt)
{
  int count;
  UINT16 *pDataOut = NULL;
  int temp;
  UCHAR *pBuf = pData;
  UCHAR *pTemp = pData;
  int total = 0;

  pDataOut = (UINT16*)pData;
  for(count =0; count < readCnt; count++)
  {
    if(pTemp[count] == '\n' || pTemp[count] == '\r')
    {
      pTemp[count] = '\0';
      temp = simple_strtoul(pBuf, NULL,16);
      if(pTemp[count+1] == '\r' || pTemp[count+1] == '\n')
        count++;
      pBuf = &pTemp[count+1];
      *pDataOut = temp;
      pDataOut++;
      total++;
    }
  }
  *(pDataOut-1) = 0xffff;
  return total;
}


int dsp_download_static_images(void)
{
  UCHAR *pBuffer;
  int read_cnt;
/* LUA MODIF FR Bug 1108
  int order,pos = 0;
*/
  int pos = 0;
  struct file *fp;

  fp = filp_open(DSP_G711_IMAGE_NAME, O_RDONLY, 0);
  if (IS_ERR(fp))
  {
    DEBUG(PRINT("Opening of DSP Image [%s] failed! Error: %ld\n",
                 DSP_G711_IMAGE_NAME,
                -PTR_ERR(fp)), DEBUG_EXCEPTION);
    return -1;
  }
  /* Look for the size of the file */
  read_cnt = fp->f_op->llseek(fp, 0, 2);
  fp->f_op->llseek(fp,0,0);

/* LUA MODIF FR Bug 1108
  order = 0;
  while ((PAGE_SIZE<<order) < read_cnt)
    order++;
	pBuffer = (UCHAR*)__get_free_pages(GFP_ATOMIC, order);
	
*/
/* Allocate the memory */
	pBuffer = (UCHAR*) vmalloc(read_cnt);
/* End modif FR Bug 1108 */

  printk("get buffer %p\n", pBuffer);

  if(pBuffer == NULL)
  {
    DEBUG(PRINT(" Could not allocate memory for DSP Image \n"), DEBUG_EXCEPTION);
    filp_close(fp, NULL);
    return 0;
  }
  /* Read the file */
  pos = kernel_read(fp,0,pBuffer,read_cnt);
  if(pos != read_cnt)
  {
    DEBUG(PRINT(" DSP Image read error \n"), DEBUG_EXCEPTION);
    filp_close(fp, NULL);
/* LUA MODIF FR Bug 1108
free_pages((unsigned long)pBuffer,order);
*/
    vfree(pBuffer);
/* End modif FR Bug 1108 */
    return 0;
  }
  filp_close(fp, NULL);
  /* Parse the data */
  parse_dsp_image_data(pBuffer,read_cnt);
  osLockInterrupts();

  /* Download to the DSP */
  *((UINT32 *)CPE_PER_INT) = 0x0;
  osUnlockInterrupts();

  for(pos = 0; pos<MAX_DSP_NUM; pos++)
  {
    osDownloadDSPImage((UINT16*)pBuffer,pos);
  }
  dspTimeSlot0 = DSP0_TIME_SLOT;
  dspTimeSlot1 = DSP1_TIME_SLOT;
  dspStartSPORT();
  /* Free Buffer */
/* LUA MODIF FR Bug 1108
free_pages((unsigned long)pBuffer,order);
*/
  vfree(pBuffer);
/* End modif FR Bug 1108 */
  return SUCCESS;
}

#if DYNAMIC_CODEC_DOWNLOAD
int  store_dynamic_image(UINT16 **pImageData, UCHAR *pImageName)
{
  UCHAR *pBuffer;
  int read_cnt;
  int order,dataOrder;
  struct file *fp;
  int total,pos;
  UINT16 *pDATA;
  UINT16 *pTemp;

  /* Open the DSP Image file */
  fp = filp_open(pImageName, O_RDONLY, 0);
  if (IS_ERR(fp))
  {
    DEBUG(PRINT("Opening of DSP Image [%s] failed! Error: %ld\n",
                 pImageName,
                -PTR_ERR(fp)), DEBUG_EXCEPTION);
    
    *pImageData = NULL;
    return FAILURE;
  }

  read_cnt = fp->f_op->llseek(fp, 0, 2);
  fp->f_op->llseek(fp,0,0);
  order = 0;

  /* Allocate page vector */
  while ((PAGE_SIZE<<order) < read_cnt)
    order++;

  /* Allocate the memory */
  pBuffer = (UCHAR*)__get_free_pages(GFP_ATOMIC, order);
  if(pBuffer == NULL)
  {
    DEBUG(PRINT(" Could not allocate memory for DSP Image \n"), DEBUG_EXCEPTION);
    filp_close(fp, NULL);
    *pImageData = NULL;
    return FAILURE;
  }
 
  /* Read the file */
  pos = kernel_read(fp,0,pBuffer,read_cnt);
  if(pos != read_cnt)
  {
    DEBUG(PRINT(" DSP Image read error \n"), DEBUG_EXCEPTION);
    filp_close(fp, NULL);
    free_pages((unsigned long)pBuffer,order);
    *pImageData = NULL;
    return FAILURE;
  }
  filp_close(fp, NULL);
  total = parse_dsp_image_data(pBuffer,read_cnt);
  DEBUG(PRINT("Total words found in %s Image %d\n",pImageName,total), DEBUG_INFO);

  read_cnt = total*2;
  /* Allocate page vector */
  dataOrder = 0;
  while ((PAGE_SIZE<<dataOrder) < read_cnt)
    dataOrder++;
  if(dataOrder == order)
  {
    *pImageData = (UINT16*)pBuffer;
    return 0;
  }
  pTemp = (UINT16*)__get_free_pages(GFP_ATOMIC, dataOrder);
  pDATA = (UINT16*)pBuffer;
  for(read_cnt = 0; read_cnt<total; read_cnt++)
  {
    pTemp[read_cnt] = pDATA[read_cnt];
  }
  *pImageData = pTemp;
  free_pages((unsigned long)pBuffer,order);
  return 0;
}
  
  
int dsp_download_image(int dspNum)
{
  int codec;
  UINT16 *pImage;
  codec = confGetDSPDownloadOpt(dspNum);
  switch(codec)
  {
    case DSP_PCMU:
    case DSP_PCMA:
      if(pG711Vad2OnlyImage == NULL)
      {
        DEBUG(PRINT("PCMU/PCMA DSP Image not found \n"), DEBUG_EXCEPTION);
        return -1;
      }
      pImage = pG711Vad2OnlyImage;
      break;
    case DSP_T38:

      if(pFaxOnlyImage == NULL)
      {
        DEBUG(PRINT("PCMU/PCMA DSP Image not found \n"), DEBUG_EXCEPTION);
        return -1;
      }
      pImage = pFaxOnlyImage;
      break;
    case DSP_G723_AH:
    case DSP_G723_AL:
      if(pG723OnlyImage == NULL)
      {
        DEBUG(PRINT("G-723 DSP Image not found \n"), DEBUG_EXCEPTION);
        return -1;
      }
      pImage = pG723OnlyImage;;
      break;
    case DSP_G729:
      if(pG729OnlyImage == NULL)
      {
        DEBUG(PRINT("G-729 DSP Image not found \n"), DEBUG_EXCEPTION);
        return -1;
      }
      pImage = pG729OnlyImage;;
      break;
     case DSP_3WAY:
      if(p3wayOnlyImage == NULL)
      {
        DEBUG(PRINT("G-729 DSP Image not found \n"), DEBUG_EXCEPTION);
        return -1;
      }
printk("Downloading Three way conference \n");
      pImage = p3wayOnlyImage;;
      break;
    default:
      DEBUG(PRINT("INVALID CODEC OPTION FOR IMAGE DOWNLOADd \n"), DEBUG_EXCEPTION);
      return FAILURE;
  }
  dspipcPollonDsp2completeIsr(dspNum);
  osDownloadDSPImage(pImage,dspNum);
  return 0;
}
#endif
module_init(dsp_driver_init);
module_exit(dsp_driver_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Linux Media Team ");
MODULE_DESCRIPTION("Driver Interface to the DSP and Slic ");
