
#include "include/si3216.h"
#include "include/france_const.h"

const ringStruct franceRingTable;
const toneCadenceTables franceToneCadenceTables [FR_TONE_END][FR_MAX_NUM_CADENCE];

/*
The following Array contains FRANCE Ring Parmeters
*/
const ringStruct  franceRingTable = {
	// TYPE_FRANCE : Ring 50Hz 35V 1.5s On 3.5s Off
	FR_RING_OSC_FREQ_50HZ_IR7,
	FR_RING_OSC_AMP_50HZ_IR8,
	FR_RING_OSC_INITPHASE_50HZ_IR9, 
	FR_RING_ON_1S5_LSB_DR48,
	FR_RING_ON_1S5_MSB_DR49,
	FR_RING_OFF_3S5_LSB_DR50,
	FR_RING_OFF_3S5_MSB_DR51,
	FR_RING_TRIP_THRESH_50HZ_IR16,
	FR_RING_TRIP_FILTER_50HZ_IR23,
	FR_RT_DEBOUNCE_50HZ_DR70
};
                                      
/*
The following Array contains FRANCE Tones Parmeters
*/
const toneCadenceTables  franceToneCadenceTables [FR_TONE_END][FR_MAX_NUM_CADENCE] =
{
	{ // FR_NULL_TONE
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_DIAL_TONE
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 440Hz -3.5dBm0 Continuous
				FR_DIAL_IR0,
				FR_DIAL_IR1,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_NULL_CALL_IN_PROGRESS
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_RING_BACK_TONE
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 440Hz -8dBm0 1.5s On 3.5s Off
				FR_RINGBACK_IR0,
				FR_RINGBACK_IR1,
				0x00,
				FR_RINGBACK_DR36,
				FR_RINGBACK_DR37,
				FR_RINGBACK_DR38,
				FR_RINGBACK_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_BUSY_TONE
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 440Hz -8dBm0 0.5s On 0.5s Off
				FR_BUSY_IR0,
				FR_BUSY_IR1,
				0x00,
				FR_BUSY_DR36,
				FR_BUSY_DR37,
				FR_BUSY_DR38,
				FR_BUSY_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_MSG_WAITING_TONE
		{
		0, // No Next Cadence Osc1
		1, // Next Cadence 1 Osc2
			{
				{ // OSC1 440Hz -8.5dBm0 Continuous
				FR_MSG_WAITING_IR0,
				FR_MSG_WAITING_IR1,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 330Hz 5dBm0 50ms On 150ms Off
				FR_MSG_WAITING_C0_IR3,
				FR_MSG_WAITING_C0_IR4,
				0x00,
				FR_MSG_WAITING_C0_DR40,
				FR_MSG_WAITING_C0_DR41,
				FR_MSG_WAITING_C0_DR42,
				FR_MSG_WAITING_C0_DR43
				},
			},
		},
		{
		0, // Next Cadence Osc1
		0, // Next Cadence 0 Osc2
			{
				{ // OSC1 440Hz -8.5dBm0 Continuous
				FR_MSG_WAITING_IR0,
				FR_MSG_WAITING_IR1,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 330Hz -5dBm0 50ms On 300ms Off
				FR_MSG_WAITING_C1_IR3,
				FR_MSG_WAITING_C1_IR4,
				0x00,
				FR_MSG_WAITING_C1_DR40,
				FR_MSG_WAITING_C1_DR41,
				FR_MSG_WAITING_C1_DR42,
				FR_MSG_WAITING_C1_DR43
				},
			},
		},
	},

	{ // FR_CALL_WAITING_TONE
		{
		1, // Next Cadence 1 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 440Hz -8dBm0 0.3s On 8s Off
				FR_CALL_WAIT_C0_IR0,
				FR_CALL_WAIT_C0_IR1,
				0x00,
				FR_CALL_WAIT_C0_DR36,
				FR_CALL_WAIT_C0_DR37,
				FR_CALL_WAIT_C0_DR38,
				FR_CALL_WAIT_C0_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		0, // Next Cadence 0 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 2s Off
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				FR_CALL_WAIT_C1_DR38,
				FR_CALL_WAIT_C1_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_ALERTING_TONE
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 2130Hz -9dBm0 80ms On
				FR_ALERTING_IR0,
				FR_ALERTING_IR1,
				0x00,
				FR_ALERTING_DR36,
				FR_ALERTING_DR37,
				0x00,
				0x00
				},
				{ // OSC2 2750Hz -9dBm0 80ms On
				FR_ALERTING_IR3,
				FR_ALERTING_IR4,
				0x00,
				FR_ALERTING_DR40,
				FR_ALERTING_DR41,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_VOIP_DIAL_TONE
		{
		1, // Next Cadence 1 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 440Hz -8.5dBm0 500ms On 20ms Off
				FR_VOIP_DIAL_C0_IR0,
				FR_VOIP_DIAL_C0_IR1,
				0x00,
				FR_VOIP_DIAL_C0_DR36,
				FR_VOIP_DIAL_C0_DR37,
				FR_VOIP_DIAL_C0_DR38,
				FR_VOIP_DIAL_C0_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		2, // Next Cadence 2 Osc1
		0, // No Next Cadence Osc2
			{
				{// OSC1	1000Hz -17dBm0 250ms On 20ms Off
				FR_VOIP_DIAL_C1_IR0,
				FR_VOIP_DIAL_C1_IR1,
				0x00,
				FR_VOIP_DIAL_C1_DR36,
				FR_VOIP_DIAL_C1_DR37,
				FR_VOIP_DIAL_C1_DR38,
				FR_VOIP_DIAL_C1_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		3, // Next Cadence 3 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	740Hz -17dBm0 250ms On 20ms Off
				FR_VOIP_DIAL_C2_IR0,
				FR_VOIP_DIAL_C2_IR1,
				0x00,
				FR_VOIP_DIAL_C2_DR36,
				FR_VOIP_DIAL_C2_DR37,
				FR_VOIP_DIAL_C2_DR38,
				FR_VOIP_DIAL_C2_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		1, // Next Cadence 1 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 440Hz -8.5dBm0 2s On 20ms Off
				FR_VOIP_DIAL_C3_IR0,
				FR_VOIP_DIAL_C3_IR1,
				0x00,
				FR_VOIP_DIAL_C3_DR36,
				FR_VOIP_DIAL_C3_DR37,
				FR_VOIP_DIAL_C3_DR38,
				FR_VOIP_DIAL_C3_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // FR_VOIP_MWI_TONE
		{
		1, // Next Cadence 1 Osc1
		1, // Next Cadence 1 Osc2
			{
				{ // OSC1 440Hz -8.5dBm0 550ms On 20ms Off
				FR_VOIP_MWI_C0_IR0,
				FR_VOIP_MWI_C0_IR1,
				0x00,
				FR_VOIP_MWI_C0_DR36,
				FR_VOIP_MWI_C0_DR37,
				FR_VOIP_MWI_C0_DR38,
				FR_VOIP_MWI_C0_DR39
				},
				{ // OSC2 330Hz -5dBm0 50ms On 150ms Off
				FR_VOIP_MWI_C0_IR3,
				FR_VOIP_MWI_C0_IR4,
				0x00,
				FR_VOIP_MWI_C0_DR40,
				FR_VOIP_MWI_C0_DR41,
				FR_VOIP_MWI_C0_DR42,
				FR_VOIP_MWI_C0_DR43
				},
			},
		},
		{
		2, // Next Cadence 2 Osc1
		2, // Next Cadence 2 Osc2
			{
				{ // OSC1	1000Hz -17dBm0 250ms On 20ms Off
				FR_VOIP_MWI_C1_IR0,
				FR_VOIP_MWI_C1_IR1,
				0x00,
				FR_VOIP_MWI_C1_DR36,
				FR_VOIP_MWI_C1_DR37,
				FR_VOIP_MWI_C1_DR38,
				FR_VOIP_MWI_C1_DR39
				},
				{ // OSC2 330Hz -5dBm0 50ms On 860ms Off
				FR_VOIP_MWI_C1_IR3,
				FR_VOIP_MWI_C1_IR4,
				0x00,
				FR_VOIP_MWI_C1_DR40,
				FR_VOIP_MWI_C1_DR41,
				FR_VOIP_MWI_C1_DR42,
				FR_VOIP_MWI_C1_DR43
				},
			},
		},
		{
		3, // Next Cadence 3 Osc1
		3, // Next Cadence 3 Osc2
			{
				{ // OSC1	740Hz -17dBm0 250ms On 20ms Off
				FR_VOIP_MWI_C2_IR0,
				FR_VOIP_MWI_C2_IR1,
				0x00,
				FR_VOIP_MWI_C2_DR36,
				FR_VOIP_MWI_C2_DR37,
				FR_VOIP_MWI_C2_DR38,
				FR_VOIP_MWI_C2_DR39
				},
				{ // OSC2 330Hz -5dBm0 50ms On 150ms Off
				FR_VOIP_MWI_C2_IR3,
				FR_VOIP_MWI_C2_IR4,
				0x00,
				FR_VOIP_MWI_C2_DR40,
				FR_VOIP_MWI_C2_DR41,
				FR_VOIP_MWI_C2_DR42,
				FR_VOIP_MWI_C2_DR43
				},
			},
		},
		{
		1, // Next Cadence 1 Osc1
		4, // Next Cadence 4 Osc2
			{
				{ // OSC1 440Hz -8.5dBm0 1650ms On 20ms Off
				FR_VOIP_MWI_C3_IR0,
				FR_VOIP_MWI_C3_IR1,
				0x00,
				FR_VOIP_MWI_C3_DR36,
				FR_VOIP_MWI_C3_DR37,
				FR_VOIP_MWI_C3_DR38,
				FR_VOIP_MWI_C3_DR39
				},
				{ // OSC2 330Hz -5dBm0 50ms On 300ms Off
				FR_VOIP_MWI_C3_IR3,
				FR_VOIP_MWI_C3_IR4,
				0x00,
				FR_VOIP_MWI_C3_DR40,
				FR_VOIP_MWI_C3_DR41,
				FR_VOIP_MWI_C3_DR42,
				FR_VOIP_MWI_C3_DR43
				},
			},
		},
		{
		0, // No Next Cadence Osc1
		5, // Next Cadence 5 Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 330Hz -5dBm0 50ms On 150ms Off
				FR_VOIP_MWI_C4_IR3,
				FR_VOIP_MWI_C4_IR4,
				0x00,
				FR_VOIP_MWI_C4_DR40,
				FR_VOIP_MWI_C4_DR41,
				FR_VOIP_MWI_C4_DR42,
				FR_VOIP_MWI_C4_DR43
				},
			},
		},
		{
		0, // No Next Cadence Osc1
		6, // Next Cadence 6 Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 330Hz -5dBm0 50ms On 300ms Off
				FR_VOIP_MWI_C5_IR3,
				FR_VOIP_MWI_C5_IR4,
				0x00,
				FR_VOIP_MWI_C5_DR40,
				FR_VOIP_MWI_C5_DR41,
				FR_VOIP_MWI_C5_DR42,
				FR_VOIP_MWI_C5_DR43
				},
			},
		},
		{
		0, // No Next Cadence Osc1
		7, // Next Cadence 7 Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 330Hz -5dBm0 50ms On 150ms Off
				FR_VOIP_MWI_C6_IR3,
				FR_VOIP_MWI_C6_IR4,
				0x00,
				FR_VOIP_MWI_C6_DR40,
				FR_VOIP_MWI_C6_DR41,
				FR_VOIP_MWI_C6_DR42,
				FR_VOIP_MWI_C6_DR43
				},
			},
		},
		{
		0, // No Next Cadence Osc1
		2, // Next Cadence 2 Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 330Hz -5dBm0 50ms On 860ms Off
				FR_VOIP_MWI_C7_IR3,
				FR_VOIP_MWI_C7_IR4,
				0x00,
				FR_VOIP_MWI_C7_DR40,
				FR_VOIP_MWI_C7_DR41,
				FR_VOIP_MWI_C7_DR42,
				FR_VOIP_MWI_C7_DR43
				},
			},
		},
	},

	{ // FR_DTMF_DIG_0_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				FR_DTMF_FREQ_941Hz,
				FR_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1336Hz,
				FR_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_1_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz 11dBm0 80ms On
				FR_DTMF_FREQ_697Hz,
				FR_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1209Hz,
				FR_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_2_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz -11dBm0 80ms On
				FR_DTMF_FREQ_697Hz,
				FR_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1336Hz,
				FR_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_3_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz -11dBm0 80ms On
				FR_DTMF_FREQ_697Hz,
				FR_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1477Hz,
				FR_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_4_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				FR_DTMF_FREQ_770Hz,
				FR_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1209Hz,
				FR_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_5_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				FR_DTMF_FREQ_770Hz,
				FR_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1336Hz,
				FR_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_6_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				FR_DTMF_FREQ_770Hz,
				FR_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1477Hz,
				FR_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_7_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				FR_DTMF_FREQ_852Hz,
				FR_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1209Hz,
				FR_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_8_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				FR_DTMF_FREQ_852Hz,
				FR_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1336Hz,
				FR_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_9_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				FR_DTMF_FREQ_852Hz,
				FR_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1477Hz,
				FR_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_A_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz -11dBm0 80ms On
				FR_DTMF_FREQ_697Hz,
				FR_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1633Hz,
				FR_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_B_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				FR_DTMF_FREQ_770Hz,
				FR_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1633Hz,
				FR_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_C_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				FR_DTMF_FREQ_852Hz,
				FR_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1633Hz,
				FR_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_D_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				FR_DTMF_FREQ_941Hz,
				FR_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1633Hz,
				FR_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_STAR_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				FR_DTMF_FREQ_941Hz,
				FR_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1209Hz,
				FR_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // FR_DTMF_DIG_HASH_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				FR_DTMF_FREQ_941Hz,
				FR_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				FR_DTMF_DR36,
				FR_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				FR_DTMF_FREQ_1477Hz,
				FR_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				FR_DTMF_DR40,
				FR_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
};


