/******************************************************************************
*
*   Copyright (C) 2002 Analog Devices India (P) Ltd. All rights are reserved.
*
*   The information and source code contained herein is the exclusive
*   property of Analog Devices India (P) Ltd. and may not be disclosed,
*   examined or reproduced in whole or in part without explicit written
*   authorization from the compnay.
*
******************************************************************************/

#ifndef __NETHERLANDS_CONFIG_H
#define __NETHERLANDS_CONFIG_H

//****************************************************************************
//		Plan de transmission du FXS NETHERLANDS : CPLX +5.5dBIn/-3.5dBout
//****************************************************************************
// SAGEM CFe : changement du plan de transmission du FXS : +5.5dBIn/-3.5dBout (au lieu de +3dBIn/-7dBout) 
// attnuation FXS totale en transmission de 3,5 dB avec les attnuations de 3.5 dB d'ATX, de 1.63 dB du mode linaire et 0.37 dB de ADCG
// attnuation FXS totale en rception de 5,5 dB avec les attnuations de 3.5 dB d'ARX, de 0.15 dB intrinsque et gain DACG de 0.15dB 

// Transmit Path Digital To Analog Converter Gain/Attenuation = -0.37dB.
#define NL_XMIT_DIGITAL_GAIN_CPLX_IR14 		0x3D55
// Receive Path Digital To Analog Converter Gain/Attenuation = +0.15dB.
#define NL_RECV_DIGITAL_GAIN_CPLX_IR13 		0x411D

#define NL_RECV_STD_GAIN_VERYSOFT 			0x20A0	// -5.85 dB
#define NL_RECV_STD_GAIN_SOFT 			0x2E10	// -2.85 dB
#define NL_RECV_STD_GAIN_NORMAL 			0x4110	//  0.15 dB
#define NL_RECV_STD_GAIN_LOUD 			0x5BF0	//  3.15 DB
#define NL_RECV_STD_GAIN_VERYLOUD 			0x7FF0	//  6 dB (max)

//****************************************************************************
//		NETHERLANDS Ring specifications : Ring 25Hz 35V 1s On 4s Off
//****************************************************************************
// Ringing Active Timer = 125uS * 0x1F40 = 1S.
#define NL_RING_ON_1S_LSB_DR48 			0x40
#define NL_RING_ON_1S_MSB_DR49 			0x1F
// Ringing Inactive Timer = 125uS * 0x7D00 = 4S
#define NL_RING_OFF_4S_LSB_DR50 			0x00
#define NL_RING_OFF_4S_MSB_DR51 			0x7D

// Ring Oscillator Frequency Coefficient for 25Hz.
#define NL_RING_OSC_FREQ_25HZ_IR7 			0x7E6C
// Ring Oscillator Amplitude Register for 50Hz and 35V.
#define NL_RING_OSC_AMP_25HZ_IR8 			0x0185
// Ring Oscillator Initial Phase Register for 50Hz and 35V.
#define NL_RING_OSC_INITPHASE_25HZ_IR9 		0x0000
// Data sheet SI3216.
#define NL_RING_TRIP_THRESH_25HZ_IR16 		0x3600
#define NL_RING_TRIP_FILTER_25HZ_IR23 		0x0350
// Ring Trip Detect Debounce Interval = 1.25mS * 0x0A = 12,5mS.
#define NL_RT_DEBOUNCE_25HZ_DR70 			0x0A


//****************************************************************************
//		NETHERLANDS Tones specifications
//****************************************************************************
// debut : alignement sur la BiOS (tone MWI NL = tone MWI FR)
#define FR_FREQ_330Hz 				0x7EED
#define FR_F330Hz_AMP_M5dBm0 			0x00D1

#define FR_FREQ_440Hz 				0x7E18
#define FR_F440Hz_AMP_M8dBm05 		0x00BA
// fin : alignement sur la BiOS (tone MWI NL = la tone MWI FR)

#define NL_FREQ_425Hz 				0x7E39
#define NL_F425Hz_AMP_M6dBm0 			0x00F0

#define NL_F425Hz_AMP_M11dBm5 		0x007F
#define NL_F425Hz_AMP_M13dBm0 		0x006B


// OSC1	425Hz -8.5dBm0 2s On 20ms Off
// OSC1	425Hz -11.5dBm0 Continuous : reduction du niveau de la frequence 425Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 425Hz
#define	NL_DIAL_IR0 				NL_FREQ_425Hz
#define	NL_DIAL_IR1 				NL_F425Hz_AMP_M11dBm5

// OSC1	425Hz -6dBm0 1s On 4s Off
#define	NL_RINGBACK_IR0 			NL_FREQ_425Hz
#define	NL_RINGBACK_IR1 			NL_F425Hz_AMP_M6dBm0
#define	NL_RINGBACK_DR36 			0x40
#define	NL_RINGBACK_DR37 			0x1F
#define	NL_RINGBACK_DR38 			0X00
#define	NL_RINGBACK_DR39 			0X7D

// OSC1	425Hz -6dBm0 0.5s On 0.5s Off
#define	NL_BUSY_IR0 				NL_FREQ_425Hz
#define	NL_BUSY_IR1 				NL_F425Hz_AMP_M6dBm0
#define	NL_BUSY_DR36 				0xA0
#define	NL_BUSY_DR37 				0x0F
#define	NL_BUSY_DR38 				0xA0
#define	NL_BUSY_DR39 				0x0F

// OSC1	425Hz -13dBm0 0.5s On 8s Off
#define	NL_CALL_WAIT_C0_IR0 		NL_FREQ_425Hz
#define	NL_CALL_WAIT_C0_IR1 		NL_F425Hz_AMP_M13dBm0
#define	NL_CALL_WAIT_C0_DR36 		0xA0	
#define	NL_CALL_WAIT_C0_DR37 		0x0F
#define	NL_CALL_WAIT_C0_DR38 		0x00
#define	NL_CALL_WAIT_C0_DR39 		0xFA
// OSC1 1.5s Off
#define	NL_CALL_WAIT_C1_IR0 		0x00
#define	NL_CALL_WAIT_C1_IR1 		0x00
#define	NL_CALL_WAIT_C1_DR36 		0x00	
#define	NL_CALL_WAIT_C1_DR37 		0x00
#define	NL_CALL_WAIT_C1_DR38 		0xE0
#define	NL_CALL_WAIT_C1_DR39 		0x2E

// OSC1	425Hz -8.5dBm0 2s On 20ms Off
// OSC1	425Hz -11.5dBm0 500ms On 20ms Off : reduction du niveau de la frequence 425Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 425Hz
#define	NL_VOIP_DIAL_C0_IR0 		NL_FREQ_425Hz
#define	NL_VOIP_DIAL_C0_IR1 		NL_F425Hz_AMP_M11dBm5
#define	NL_VOIP_DIAL_C0_DR36 		0xA0
#define	NL_VOIP_DIAL_C0_DR37 		0x0F
#define	NL_VOIP_DIAL_C0_DR38 		0xA0
#define	NL_VOIP_DIAL_C0_DR39 		0x00
// OSC1	1000Hz -8.5dBm0 250ms On 20ms Off
// OSC1	1000Hz -17dBm0 250ms On 20ms Off : reduction du niveau de la frequence 1000Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 1000Hz
#define	NL_VOIP_DIAL_C1_IR0 		0x7642
#define	NL_VOIP_DIAL_C1_IR1 		0x00A0 // 0x01AC -> -8.5dBm0
#define	NL_VOIP_DIAL_C1_DR36 		0xD0
#define	NL_VOIP_DIAL_C1_DR37 		0x07
#define	NL_VOIP_DIAL_C1_DR38 		0xA0
#define	NL_VOIP_DIAL_C1_DR39 		0x00
// OSC1	740Hz -8.5dBm0 250ms On 20ms Off
// OSC1	740Hz -17dBm0 250ms On 20ms Off : reduction du niveau de la frequence 740Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 740Hz
#define	NL_VOIP_DIAL_C2_IR0 		0x7AA2
#define	NL_VOIP_DIAL_C2_IR1 		0x0076 // 0x013B -> -8.5dBm0
#define	NL_VOIP_DIAL_C2_DR36 		0xD0
#define	NL_VOIP_DIAL_C2_DR37 		0x07
#define	NL_VOIP_DIAL_C2_DR38 		0xA0
#define	NL_VOIP_DIAL_C2_DR39 		0x00
// OSC1	425Hz -8.5dBm0 2s On 20ms Off
// OSC1	425Hz -11.5dBm0 2s On 20ms Off : reduction du niveau de la frequence 425Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 425Hz
#define	NL_VOIP_DIAL_C3_IR0 		NL_FREQ_425Hz
#define	NL_VOIP_DIAL_C3_IR1 		NL_F425Hz_AMP_M11dBm5
#define	NL_VOIP_DIAL_C3_DR36 		0x80
#define	NL_VOIP_DIAL_C3_DR37 		0x3E
#define	NL_VOIP_DIAL_C3_DR38 		0xA0
#define	NL_VOIP_DIAL_C3_DR39 		0x00



// OSC1	440Hz -8.5dBm0 550ms On 20ms Off
#define	NL_VOIP_MWI_C0_IR0         FR_FREQ_440Hz
#define	NL_VOIP_MWI_C0_IR1         FR_F440Hz_AMP_M8dBm05
#define	NL_VOIP_MWI_C0_DR36        0x30
#define	NL_VOIP_MWI_C0_DR37        0x11
#define	NL_VOIP_MWI_C0_DR38        0xA0
#define	NL_VOIP_MWI_C0_DR39        0x00
// OSC1	1000Hz -8.5dBm0 250ms On 20ms Off
// OSC1	1000Hz -17dBm0 250ms On 20ms Off : reduction du niveau de la frequence 1000Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 1000Hz
#define	NL_VOIP_MWI_C1_IR0         0x7642
#define	NL_VOIP_MWI_C1_IR1         0x00A0 // 0x01AC -> -8.5dBm0
#define	NL_VOIP_MWI_C1_DR36        0xD0
#define	NL_VOIP_MWI_C1_DR37        0x07
#define	NL_VOIP_MWI_C1_DR38        0xA0
#define	NL_VOIP_MWI_C1_DR39        0x00
// OSC1	740Hz -8.5dBm0 250ms On 20ms Off
// OSC1	740Hz -17dBm0 250ms On 20ms Off : reduction du niveau de la frequence 740Hz pour ameliorer
// la detection du premier digit DTMF en presence de la tonalite 740Hz
#define	NL_VOIP_MWI_C2_IR0         0x7AA2
#define	NL_VOIP_MWI_C2_IR1         0x0076 // 0x013B -> -8.5dBm0
#define	NL_VOIP_MWI_C2_DR36        0xD0
#define	NL_VOIP_MWI_C2_DR37        0x07
#define	NL_VOIP_MWI_C2_DR38        0xA0
#define	NL_VOIP_MWI_C2_DR39        0x00
// OSC1	440Hz -8.5dBm0 1650ms On 20ms Off
#define	NL_VOIP_MWI_C3_IR0         FR_FREQ_440Hz
#define	NL_VOIP_MWI_C3_IR1         FR_F440Hz_AMP_M8dBm05
#define	NL_VOIP_MWI_C3_DR36        0x90
#define	NL_VOIP_MWI_C3_DR37        0x33
#define	NL_VOIP_MWI_C3_DR38        0xA0
#define	NL_VOIP_MWI_C3_DR39        0x00
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	NL_VOIP_MWI_C0_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C0_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C0_DR40      0x90
#define	NL_VOIP_MWI_C0_DR41      0x01
#define	NL_VOIP_MWI_C0_DR42      0xB0
#define	NL_VOIP_MWI_C0_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 860ms Off
#define	NL_VOIP_MWI_C1_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C1_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C1_DR40      0x90
#define	NL_VOIP_MWI_C1_DR41      0x01
#define	NL_VOIP_MWI_C1_DR42      0xE0
#define	NL_VOIP_MWI_C1_DR43      0x1A
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	NL_VOIP_MWI_C2_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C2_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C2_DR40      0x90
#define	NL_VOIP_MWI_C2_DR41      0x01
#define	NL_VOIP_MWI_C2_DR42      0xB0
#define	NL_VOIP_MWI_C2_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 300ms Off
#define	NL_VOIP_MWI_C3_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C3_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C3_DR40      0x90
#define	NL_VOIP_MWI_C3_DR41      0x01
#define	NL_VOIP_MWI_C3_DR42      0x60
#define	NL_VOIP_MWI_C3_DR43      0x09
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	NL_VOIP_MWI_C4_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C4_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C4_DR40      0x90
#define	NL_VOIP_MWI_C4_DR41      0x01
#define	NL_VOIP_MWI_C4_DR42      0xB0
#define	NL_VOIP_MWI_C4_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 300ms Off
#define	NL_VOIP_MWI_C5_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C5_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C5_DR40      0x90
#define	NL_VOIP_MWI_C5_DR41      0x01
#define	NL_VOIP_MWI_C5_DR42      0x60
#define	NL_VOIP_MWI_C5_DR43      0x09
// OSC2	330Hz -5dBm0 50ms On 150ms Off
#define	NL_VOIP_MWI_C6_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C6_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C6_DR40      0x90
#define	NL_VOIP_MWI_C6_DR41      0x01
#define	NL_VOIP_MWI_C6_DR42      0xB0
#define	NL_VOIP_MWI_C6_DR43      0x04
// OSC2	330Hz -5dBm0 50ms On 860ms Off
#define	NL_VOIP_MWI_C7_IR3       FR_FREQ_330Hz
#define	NL_VOIP_MWI_C7_IR4       FR_F330Hz_AMP_M5dBm0
#define	NL_VOIP_MWI_C7_DR40      0x90
#define	NL_VOIP_MWI_C7_DR41      0x01
#define	NL_VOIP_MWI_C7_DR42      0xE0
#define	NL_VOIP_MWI_C7_DR43      0x1A


//****************************************************************************
//		NETHERLANDS Dtmf specifications
//****************************************************************************
#define	NL_DTMF_FREQ_697Hz 			0x7b3c 
#define	NL_DTMF_F697Hz_AMP_M11dBm0 	0xdd

#define	NL_DTMF_FREQ_770Hz 			0x7a31 
#define	NL_DTMF_F770Hz_AMP_M11dBm0 	0xf5

#define	NL_DTMF_FREQ_852Hz 			0x78e6 
#define	NL_DTMF_F852Hz_AMP_M11dBm0 	0x110

#define	NL_DTMF_FREQ_941Hz 			0x775c 
#define	NL_DTMF_F941Hz_AMP_M11dBm0 	0x12d

#define	NL_DTMF_FREQ_1209Hz 		0x71d7 
#define	NL_DTMF_F1209Hz_AMP_M9dBm0 	0x1ea

#define	NL_DTMF_FREQ_1336Hz 		0x6ec8 
#define	NL_DTMF_F1336Hz_AMP_M9dBm0 	0x220

#define	NL_DTMF_FREQ_1477Hz 		0x6b10 
#define	NL_DTMF_F1477Hz_AMP_M9dBm0 	0x25d

#define	NL_DTMF_FREQ_1633Hz 		0x6692 
#define	NL_DTMF_F1633Hz_AMP_M9dBm0 	0x2a1

// OSC1	80ms On
#define	NL_DTMF_DR36 				0x80
#define	NL_DTMF_DR37 				0x02
// OSC2	80ms On
#define	NL_DTMF_DR40 				0x80
#define	NL_DTMF_DR41 				0x02
                                                                 
typedef enum
{
	NL_NULL_TONE 					= 0,
	NL_DIAL_TONE 					= 1,
	NL_NULL_CALL_IN_PROGRESS 		= 2,
	NL_RING_BACK_TONE 				= 3,
	NL_BUSY_TONE 					= 4,
	NL_NULL_MSG_WAITING_TONE 		= 5,
	NL_CALL_WAITING_TONE 			= 6,
	NL_NULL_ALERTING_TONE 			= 7,                                                                                
	NL_VOIP_DIAL_TONE 				= 8,
	NL_VOIP_MWI_TONE 				= 9,
	NL_DTMF_DIG_0_TONE_LOCAL 		= 10,
	NL_DTMF_DIG_1_TONE_LOCAL 		= 11,
	NL_DTMF_DIG_2_TONE_LOCAL 		= 12,
	NL_DTMF_DIG_3_TONE_LOCAL 		= 13,
	NL_DTMF_DIG_4_TONE_LOCAL 		= 14,
	NL_DTMF_DIG_5_TONE_LOCAL 		= 15,
	NL_DTMF_DIG_6_TONE_LOCAL 		= 16,
	NL_DTMF_DIG_7_TONE_LOCAL 		= 17,
	NL_DTMF_DIG_8_TONE_LOCAL 		= 18,
	NL_DTMF_DIG_9_TONE_LOCAL 		= 19,
	NL_DTMF_DIG_A_TONE_LOCAL 		= 20,
	NL_DTMF_DIG_B_TONE_LOCAL 		= 21,
	NL_DTMF_DIG_C_TONE_LOCAL 		= 22,
	NL_DTMF_DIG_D_TONE_LOCAL 		= 23,
	NL_DTMF_DIG_STAR_TONE_LOCAL 		= 24,
	NL_DTMF_DIG_HASH_TONE_LOCAL 		= 25,
	NL_TONE_END 					= 26
} enumNetherlandsTone;

#define NL_MAX_NUM_CADENCE 			8


extern const ringStruct  netherlandsRingTable;
extern const toneCadenceTables  netherlandsToneCadenceTables [NL_TONE_END][NL_MAX_NUM_CADENCE];

#endif
