
/* Silab Driver File */
#include <oswrapper.h>
#include "include/si3216.h"
#include "include/france_const.h"
#include "include/netherlands_const.h"
#include "include/poland_const.h"
#include "include/spain_const.h"

unsigned short silabChipType[MAX_SLIC_CHANNELS];
unsigned short silabChipVersion[MAX_SLIC_CHANNELS];

chipStruct chipData[MAX_SLIC_CHANNELS];

extern int si3050_pola_pos;   // polarite ligne PSTN (driver si3050)
extern void si3216GoActive(unsigned char codecNo);

SilabIndReg_t silab_init_indir_reg[] = {
        /* Register                       Label                        Initial Value  */
        {IR0_OSC1_COEF                  , "OSC1_COEF",                  TONE_OSC_1_FREQ_COEFF           },
        {IR1_OSC1X                      , "OSC1X",                      TONE_OSC_1_AMP_REG              },
        {IR2_OSC1Y                      , "OSC1Y",                      TONE_OSC_1_INIT_PHASE_REG       },
        {IR3_OSC2_COEF                  , "OSC2_COEF",                  TONE_OSC_2_FREQ_COEFF           },
        {IR4_OSC2X                      , "OSC2X",                      TONE_OSC_2_AMP_REG              },
        {IR5_OSC2Y                      , "OSC2Y",                      TONE_OSC_2_INIT_PHASE_REG       },
        {IR6_RING_V_OFF                 , "RING_V_OFF",                 RING_OFFSET_IR6                 },
        {IR7_RING_OSC_COEF              , "RING_OSC_COEF",              RING_OSC_FREQ_50HZ_IR7          },
        {IR8_RING_X                     , "RING_X",                     RING_OSC_AMP_50HZ_IR8           },
        {IR9_RING_Y                     , "RING_Y",                     RING_OSC_INITPHASE_50HZ_IR9     },
        {IR13_RECV_DIGITAL_GAIN         , "RECV_DIGITAL_GAIN",          RECV_DIGITAL_GAIN_CPLX_IR13     },
        {IR14_XMIT_DIGITAL_GAIN         , "XMIT_DIGITAL_GAIN",          XMIT_DIGITAL_GAIN_CPLX_IR14     },
        {IR15_LOOP_CLOSE_THRESH         , "LOOP_CLOSE_THRESH",          LOOP_CLOSE_THRESH_IR15          },
        {IR16_RING_TRIP_THRESH          , "RING_TRIP_THRESH",           RING_TRIP_THRESH_50HZ_IR16      },
        {IR17_COMMON_MIN_THRESH         , "COMMON_MIN_THRESH",          COMMON_MODE_MIN_THRES           },
        {IR18_COMMON_MAX_THRESH         , "COMMON_MAX_THRESH",          COMMON_MODE_MAX_THRES           },
        {IR19_PWR_ALARM_Q1Q2            , "PWR_ALARM_Q1Q2",             PWR_ALARM_Q1Q2_SOT23_IR19       },
        {IR20_PWR_ALARM_Q3Q4            , "PWR_ALARM_Q3Q4",             PWR_ALARM_Q3Q4_SOT23_IR20       },
        {IR21_PWR_ALARM_Q5Q6            , "PWR_ALARM_Q5Q6",             PWR_ALARM_Q5Q6_SOT223_IR21      },
        {IR22_LOOP_CLOSURE_FILTER       , "LOOP_CLOSURE_FILTER",        LOOP_CLOSURE_FILTER             },
        {IR23_RING_TRIP_FILTER          , "RING_TRIP_FILTER",           RING_TRIP_FILTER_50HZ_IR23      },
        {IR24_THERM_LP_POLE_Q1Q2        , "THERM_LP_POLE_Q1Q2",         THERM_LP_POLE_Q1Q2_SOT23_IR24   },
        {IR25_THERM_LP_POLE_Q3Q4        , "THERM_LP_POLE_Q3Q4",         THERM_LP_POLE_Q3Q4_SOT23_IR25   },
        {IR26_THERM_LP_POLE_Q5Q6        , "THERM_LP_POLE_Q5Q6",         THERM_LP_POLE_Q5Q6_SOT223_IR26  },
        {IR27_CM_BIAS_RINGING           , "CM_BIAS_RINGING",            VCMR_IR27                       },
        {IR64_DCDC_MIN_V                , "DCDC_MIN_V",                 VOV_IR64                        },
        {IR66_LOOP_CLOSE_THRESH_LOW     , "LOOP_CLOSE_THRESH_LOW",      LOOP_CLOSE_THRESH_LOW_IR66      },
        {IR69_FSK_X_0                   , "FSK_X_0",                    0x01B4                          },
        {IR70_FSK_COEFF_0               , "FSK_COEFF_0",                0x6B60                          },
        {IR71_FSK_X_1                   , "FSK_X_1",                    0x00E9                          },
        {IR72_FSK_COEFF_1               , "FSK_COEFF_1",                0x79C0                          },
        {IR73_FSK_X_01                  , "FSK_X_01",                   0x1110                          },
        {IR74_FSK_X_10                  , "FSK_X_10",                   0x3C00                          }
};

/******************************************************************************/
/* writeDirectReg: Write one Direct Register                                  */
/*                                                                            */
/*       INPUTS: address : Adress of Direct Register                          */
/*               data    : Data to Write                                      */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void writeDirectReg(unsigned char codecNo, unsigned char address, 
                                                  unsigned char data)
{
  unsigned char value,dummy = 0;
  ASSERT_CHIPSELECT(0);
  osLockInterrupts();  
  //Step 1: select the codec
  value = 1<<codecNo;
  spiInitOperation(value,&dummy);
  //Step 2: select the register
  value = 0x7F & address;
  spiInitOperation(value,&dummy);
  spiInitOperation(data,&dummy);
  osUnlockInterrupts();  
  DEASSERT_CHIPSELECT(0);
}
/******************************************************************************/
/* readDirectReg: Read one Direct Register                                    */
/*                                                                            */
/*       INPUTS: address : Adress of Direct Register                          */
/*                                                                            */
/*       RETURNS: Value of Direct Register                                    */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
unsigned char readDirectReg(unsigned char codecNo, unsigned char address)
{
  unsigned char data,value,dummy = 0;
  ASSERT_CHIPSELECT(0);
  osLockInterrupts();  
  //Step 1: select the codec
  value = 1<<codecNo;
  spiInitOperation(value,&dummy);
  //Step 2: select the register
  value = 0x80 | address;
  spiInitOperation(value,&dummy);
  dummy = 0;
  spiInitOperation(dummy,&data);
  osUnlockInterrupts();  
  DEASSERT_CHIPSELECT(0);
  return data;
}
/******************************************************************************/
/* writeIndirectReg: Write one Indirect Register                              */
/*                                                                            */
/*       INPUTS: address : Adress of Indirect Register                        */
/*               data    : Data to Write                                      */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void writeIndirectReg(unsigned char codecNo, unsigned char address,
                                                      unsigned short data)
{
  volatile unsigned char wait_time;
  while(1)
  {
    wait_time = readDirectReg(codecNo, 31);
    if(wait_time == 0)
      break;
  }
  writeDirectReg(codecNo, 28, (unsigned char)(data & 0xFF));
  writeDirectReg(codecNo, 29, (unsigned char)((data & 0xFF00)>>8));
  writeDirectReg(codecNo, 30, address);
}

/******************************************************************************/
/* readIndirectReg: Read one Indirect Register                                */
/*                                                                            */
/*       INPUTS: address : Adress of Indirect Register                        */
/*                                                                            */
/*       RETURNS: Value of Indirect Register                                  */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
unsigned short readIndirectReg(unsigned char codecNo, unsigned char address)
{
  volatile unsigned char wait_time;
  volatile unsigned char temp1,temp2;
  while(1)
  {
    wait_time = readDirectReg(codecNo, 31);
    if(wait_time == 0)
      break;
  }
  writeDirectReg(codecNo, 30, address);
  while(1)
  {
    wait_time = readDirectReg(codecNo, 31);
    if(wait_time == 0)
      break;
  }
  temp1 = readDirectReg(codecNo, 28);
  temp2 = readDirectReg (codecNo, 29);
  return temp1 | (temp2 << 8);
}

// Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
void si3216Modify_country_params(int country, unsigned char codecNo){

	unsigned short uiCountryOpt;

	switch(country)
	{
		case FRANCE:
		{
			DEBUG(PRINT("\r\n FRANCE SI3216 Initialization. codecNo %d \r\n", codecNo),DEBUG_INFO);

			chipData[codecNo].chipPcm=TYPE_LINEAR;
			chipData[codecNo].country=TYPE_FRANCE;
			chipData[codecNo].ring=TYPE_RING_50HZ;
			chipData[codecNo].impedance=TYPE_CTR21;
			chipData[codecNo].flagSilabsToneGenerator=1;

			silab_init_indir_reg[IR7_RING_OSC_COEF].initial = franceRingTable.coeff;
			silab_init_indir_reg[IR8_RING_X].initial = franceRingTable.x;
			silab_init_indir_reg[IR9_RING_Y].initial = franceRingTable.y;
			silab_init_indir_reg[IR16_RING_TRIP_THRESH-3].initial = franceRingTable.rtThreshold;
			silab_init_indir_reg[IR23_RING_TRIP_FILTER-3].initial = franceRingTable.rtFilter;

			silab_init_indir_reg[IR13_RECV_DIGITAL_GAIN-3].initial =
				FR_RECV_DIGITAL_GAIN_CPLX_IR13;
			silab_init_indir_reg[IR14_XMIT_DIGITAL_GAIN-3].initial =
				FR_XMIT_DIGITAL_GAIN_CPLX_IR14;
		}
		break;

		case NETHERLANDS: /* PAYS BAS HOLLANDE */    
		{
			DEBUG(PRINT("\r\n NETHERLANDS SI3216 Initialization. codecNo %d \r\n", codecNo),DEBUG_INFO);

			chipData[codecNo].chipPcm=TYPE_LINEAR;
			chipData[codecNo].country=TYPE_NETHERLANDS;
			chipData[codecNo].ring=TYPE_RING_25HZ;
			chipData[codecNo].impedance=TYPE_CTR21;
			chipData[codecNo].flagSilabsToneGenerator=1;

			silab_init_indir_reg[IR7_RING_OSC_COEF].initial = netherlandsRingTable.coeff;
			silab_init_indir_reg[IR8_RING_X].initial = netherlandsRingTable.x;
			silab_init_indir_reg[IR9_RING_Y].initial = netherlandsRingTable.y;
			silab_init_indir_reg[IR16_RING_TRIP_THRESH-3].initial = netherlandsRingTable.rtThreshold;
			silab_init_indir_reg[IR23_RING_TRIP_FILTER-3].initial = netherlandsRingTable.rtFilter;

			silab_init_indir_reg[IR13_RECV_DIGITAL_GAIN-3].initial =
				NL_RECV_DIGITAL_GAIN_CPLX_IR13;
			silab_init_indir_reg[IR14_XMIT_DIGITAL_GAIN-3].initial =
				NL_XMIT_DIGITAL_GAIN_CPLX_IR14;
		}
		break;

		case POLAND: /* POLOGNE */
		{
			DEBUG(PRINT("\r\n POLAND SI3216 Initialization. codecNo %d \r\n", codecNo),DEBUG_INFO);

			chipData[codecNo].chipPcm=TYPE_LINEAR;
			chipData[codecNo].country=TYPE_POLAND;
			chipData[codecNo].ring=TYPE_RING_25HZ;
			chipData[codecNo].impedance=TYPE_600_OHMS;
			chipData[codecNo].flagSilabsToneGenerator=1;

			silab_init_indir_reg[IR7_RING_OSC_COEF].initial = polandRingTable.coeff;
			silab_init_indir_reg[IR8_RING_X].initial = polandRingTable.x;
			silab_init_indir_reg[IR9_RING_Y].initial = polandRingTable.y;
			silab_init_indir_reg[IR16_RING_TRIP_THRESH-3].initial = polandRingTable.rtThreshold;
			silab_init_indir_reg[IR23_RING_TRIP_FILTER-3].initial = polandRingTable.rtFilter;

			silab_init_indir_reg[IR13_RECV_DIGITAL_GAIN-3].initial =
				PL_RECV_DIGITAL_GAIN_600_IR13;
			silab_init_indir_reg[IR14_XMIT_DIGITAL_GAIN-3].initial =
				PL_XMIT_DIGITAL_GAIN_600_IR14;
		}
		break;
		
		case SPAIN:
		{
			DEBUG(PRINT("\r\n SPAIN SI3216 Initialization. codecNo %d \r\n", codecNo),DEBUG_INFO);

			chipData[codecNo].chipPcm=TYPE_LINEAR;
			chipData[codecNo].country=TYPE_SPAIN;
			chipData[codecNo].ring=TYPE_RING_25HZ;
			chipData[codecNo].impedance=TYPE_CTR21;
			chipData[codecNo].flagSilabsToneGenerator=1;

			silab_init_indir_reg[IR7_RING_OSC_COEF].initial = spainRingTable.coeff;
			silab_init_indir_reg[IR8_RING_X].initial = spainRingTable.x;
			silab_init_indir_reg[IR9_RING_Y].initial = spainRingTable.y;
			silab_init_indir_reg[IR16_RING_TRIP_THRESH-3].initial = spainRingTable.rtThreshold;
			silab_init_indir_reg[IR23_RING_TRIP_FILTER-3].initial = spainRingTable.rtFilter;

			silab_init_indir_reg[IR13_RECV_DIGITAL_GAIN-3].initial =
				SP_RECV_DIGITAL_GAIN_CPLX_IR13;
			silab_init_indir_reg[IR14_XMIT_DIGITAL_GAIN-3].initial =
				SP_XMIT_DIGITAL_GAIN_CPLX_IR14;
		}
		break;

		default:
		{
			DEBUG(PRINT("\r\n unknown country, default is France  \r\n"),DEBUG_INFO);
                        
			uiCountryOpt = FRANCE;
			confSetCountryOpts(&uiCountryOpt);
			
			chipData[codecNo].chipPcm=TYPE_LINEAR;
			chipData[codecNo].country=TYPE_FRANCE;
			chipData[codecNo].ring=TYPE_RING_50HZ;
			chipData[codecNo].impedance=TYPE_CTR21;
			chipData[codecNo].flagSilabsToneGenerator=1;

			silab_init_indir_reg[IR7_RING_OSC_COEF].initial = franceRingTable.coeff;
			silab_init_indir_reg[IR8_RING_X].initial = franceRingTable.x;
			silab_init_indir_reg[IR9_RING_Y].initial = franceRingTable.y;
			silab_init_indir_reg[IR16_RING_TRIP_THRESH-3].initial = franceRingTable.rtThreshold;
			silab_init_indir_reg[IR23_RING_TRIP_FILTER-3].initial = franceRingTable.rtFilter;

			silab_init_indir_reg[IR13_RECV_DIGITAL_GAIN-3].initial =
				FR_RECV_DIGITAL_GAIN_CPLX_IR13;
			silab_init_indir_reg[IR14_XMIT_DIGITAL_GAIN-3].initial =
				FR_XMIT_DIGITAL_GAIN_CPLX_IR14;
		}
	}       
}

/***************************************************************************
 * Function to initialize Indirect Registers .                             *
 **************************************************************************/
void initializeIndirectRegisters(int codecNo)                    
{
  int size = sizeof(silab_init_indir_reg) / sizeof(silab_init_indir_reg[0]);
  int i;
  for (i=0; i<size; i++)
  {
    writeIndirectReg(codecNo, silab_init_indir_reg[i].address,
                                       silab_init_indir_reg[i].initial);
  }
}  

/************************************************************************
 * Function to overwrite default coefficients.                          *
 ***********************************************************************/
void silabOverWriteDefault(int codecNo)
{
  /* Make VBat switch not automatic */
  /* The above is a saftey measure to prevent Q7 from accidentaly turning on
     and burning out */
  writeDirectReg(codecNo, 67,0x17);
  writeDirectReg(codecNo, 66,0x1);
  /* Step => 11, write battery voltages */
  /* High Battery Voltage */
  writeDirectReg(codecNo,74, 0x32);
  writeDirectReg(codecNo,75, 0x10);
  /* On HOOK Line Voltage */
//  writeDirectReg(codecNo, 72, 0x15);  /* 31.5 V */
  writeDirectReg(codecNo, 72, 0x1F);    /* Sagem - 20060201: 46.5 V */
  writeDirectReg(codecNo, 71, 0x06);    /* Sagem - 20060123 */
 // writeDirectReg(codecNo,10,0);
#if 1
  /* Put loss of Analof tx/rx as -3.5 db */
  writeDirectReg(codecNo, 9, 0x5);
  //writeIndirectReg(codecNo, 13, 0x6000);
  //writeIndirectReg(codecNo, 14, 0x6000);
#endif
}
/***************************************************************************
 * Function to Verify Power Leak .                                         *
 **************************************************************************/ 
int silabPowerLeakTest(int codecNo)
{
  unsigned char vBat; 
  writeDirectReg(codecNo, 64,0);
  /* Turn off DS-DC Convertor */
  writeDirectReg(codecNo, 14, 0x10); 
  mdelay(1000);  // one second
  if( (vBat = readDirectReg(codecNo, 82)) < 0x4)  // 6 volts
  {
    DEBUG(PRINT("si3216 Power leak test failed: %d\n",codecNo), 
                                                 DEBUG_EXCEPTION);
  }
  DEBUG(PRINT("\n vBat Power Leak Test = 0x%X \r\n",vBat), DEBUG_INFO);
  return vBat;
}
/***************************************************************************
 * Function to Power - UP si3216 device .                                  *
 **************************************************************************/
int silabPowerUp(int codecNo)
{
  int   vBat;
  int   i = 0;
  /* DC-DC Converter Period */
  writeDirectReg(codecNo,92,0xC8);	/* Sagem 20060210 */
  /* DC-DC Converter Minimum Off Time */
  writeDirectReg(codecNo,93,0x1F);	/* Sagem 20060210 */

  /* Turn On DC-DC Convertor */
  writeDirectReg(codecNo, 14, 0x00);
  
  while ((vBat = readDirectReg(codecNo,82)) < 0xAC)
  {
    mdelay(10);
    ++i;
    if (i > 200) 
    {
      DEBUG(PRINT("\n UNABLE TO DO VERIFY VBAT SI3216. codecNo %d \r\n",
			      codecNo), DEBUG_EXCEPTION);
      return 0;
    }
  }
  writeDirectReg(codecNo, 93, 0x9F);  /* DC-DC Calibration - Sagem 20060210 */
  i = 0;
  while(0x80 & readDirectReg(codecNo, 93))
  {
    mdelay(10);
    ++i;
    if (i > 200) 
    {
      DEBUG(PRINT("\n UNABLE TO CALIBRATE VBAT SI3216. codecNo %d \r\n",
			      codecNo), DEBUG_EXCEPTION);
      return 0;
    }
  }
  DEBUG(PRINT("\n VBAT Calibration = 0x%X \r\n", vBat), DEBUG_INFO);
  return vBat;
}        
/***************************************************************************
 * Function to Verify Indirect Registers writes .                          *
 **************************************************************************/
int silabVerifyIndirectRegisters(int codecNo)
{
  unsigned short  i,j,initial;
  int size = sizeof(silab_init_indir_reg) / sizeof(silab_init_indir_reg[0]);
  printk("Size : %d\n",size);
  for (i=0; i<size; i++)
  {
    j = readIndirectReg(codecNo, silab_init_indir_reg[i].address);
    initial = silab_init_indir_reg[i].initial;

    if(initial != j)
    {
      DEBUG(PRINT("si3216 Indirect Registers %d/%x /%xinitialization failed \n",
               silab_init_indir_reg[i].address,j,
                     silab_init_indir_reg[i].initial), DEBUG_EXCEPTION);
      return -1;
    }
  }
  return 0;
}
/***************************************************************************
 * Function to Calibrate si3216 .                                          *
 **************************************************************************/
int silabCalibrate(int codecNo)
{
  unsigned int i=0;
  unsigned char   DRvalue;
  /* Step => 14 */
  /* Do All calibrations */
  writeDirectReg(codecNo, 97,0x1E); 
  /* Start Calibration */ 
  writeDirectReg(codecNo, 96,0x47);
  /* Step => 15 */
  i = 0;
  while(i < 800)
  {
    DRvalue = readDirectReg(codecNo, 96);
    if(DRvalue == 0) 
      break;
    else if( i > 800)
      break; 
    i++;
#if LINUX_2_6
    msleep(120);
#else
    mdelay(120);
#endif
  }
  if (DRvalue != 0)
  {
    DEBUG(PRINT("UNABLE TO DO CALIBRATION for si3216 : %d\n",codecNo), 
		DEBUG_EXCEPTION);
    return -1;
  }
  /* Step => 16 */
  /* Manual Calibration */
  for ( i=0x1f; i>0; i--)
  {
    writeDirectReg(codecNo,98,i);
    mdelay(40);
    if((readDirectReg(codecNo,88)) == 0)
    {
       DEBUG(PRINT("\n RING Gain Calibration OK. \r\n"), DEBUG_INFO);
       break;
    }
  } 
 
  for(i=0x1f; i>0; i--)
  {
    writeDirectReg(codecNo,99,i);
    mdelay(40);
    if((readDirectReg(codecNo,89)) == 0)
    {
      DEBUG(PRINT("\n TIP Gain Calibration OK. \r\n"), DEBUG_INFO);
      break;
    }
  }
  /* Step => 18 & 19 */
//  writeDirectReg(codecNo, 97,0x01);
  /* Start Calibration */
//  writeDirectReg(codecNo, 96,0x40);

  i = 0;
  while(i < 800)
  {
    DRvalue = readDirectReg(codecNo, 96);
    if(DRvalue == 0)
      break;
    else if( i > 800)
      break;
    i++;
    mdelay(10);
  }
  if (DRvalue != 0)
  {
    DEBUG(PRINT("UNABLE TO DO CALIBRATION for si3216 : %d\n",codecNo),
                DEBUG_EXCEPTION);
    return -1;
  }
  return(0);

}


//#ifdef 0 //JUNGO /* { */
// AVX
/* Sagem 20070305 jfd: added 6061 switch support. */
/*extern unsigned short sierra_sw_reg_read(int addr, int reg);
extern void sierra_sw_reg_write(int addr, int reg, unsigned short val);

static void m88waitForSwitchBusy(void)
{
  unsigned int _register;

  do
	{
	  _register = 0;
	  mdelay(100);
	  _register = *(volatile unsigned int *)0xb911005c;
	  
	} while (_register & 0x0800);
}

#define PORT_CTRL_REG	0x4
#define PORT_BASED_VLAN_REG	0x6
#define DEF_PORT_VLANID_REG	0x7
typedef enum {
    PORT_UNKNOWN = -1,
    PORT0 = 0,
    PORT1 = 1,
    MII_PORT = 2
} port_num_t;

#define VISIO_VID 1
#define TV_VID 2
#define DEF_VID 100

#define PHY_REGS_ADDR 0
#define PORT_REGS_ADDR(port) (0x8 + (port))
#define GLOBAL_REGS_ADDR 0xF
#define MAX_PORTS_NUM LB1_MAX_PORTS_NUM
#define LB1_MAX_PORTS_NUM 3
#define UNSUPPORTED -1*/

/* Some switch port registers */
/*#define SWITCH_IDENTIFIER_REG 3
#define PORT_CONTROL_REG 4
#define PORT_VLAN_MAP_REG 6
#define PORT_DEF_VID_REG 7
#define PORT_CONTROL2_REG 8*/

/* Some switch global registers */
/*#define VTU_OP_REG 5
#define VTU_VID_REG 6
#define VTU_DATA_PORTS_REG 7
#define VTU_DATA_PORTS45_REG 8*/

/* Some registers bit offsets and values */
/*#define SWITCH_DEVID_BIT_N 4
#define SWITCH_DEVID_BIT_MASK ((1<<12) - 1)
#define SWITCH_REVID_BIT_N 3
#define SWITCH_REVID_BIT_MASK ((1<<4) - 1)

#define SWITCH_DEVID_6021 0x021
#define SWITCH_DEVID_6061 0x061

#define PORT_8021Q_MODE_BIT_N 10
#define PORT_8021Q_MODE_BIT_MASK (0x3<<10)
#define PORT_8021Q_MODE_DISABLED 0
#define PORT_8021Q_MODE_FALLBACK 1
#define PORT_8021Q_MODE_CHECK 2
#define PORT_8021Q_MODE_SECURE 3

#define PORT_DEF_VID_BIT_N 0
#define PORT_DEF_VID_BIT_MASK ((1<<12) - 1)

#define VTU_BUSY_BIT (1<<15)
#define VTU_OP_BIT_N 12
#define VTU_OP_BIT_MASK (0x7<<12)
#define VTU_OP_FLUSH 1
#define VTU_OP_LOAD 3
#define VTU_OP_GET_NEXT 4

#define VTU_VID_VALID_BIT (1<<12)
#define VTU_VID_BIT_N 0
#define VTU_VID_BIT_MASK ((1<<12) - 1)

#define VTU_DATA_REG_VAL(p0_memtag, p1_memtag, p2_memtag, p3_memtag) \
    ((p0_memtag) | ((p1_memtag)<<4) | ((p2_memtag)<<8) | ((p3_memtag)<<12))
#define VTU_DATA45_REG_VAL(p4_memtag, p5_memtag) \
    ((p4_memtag) | ((p5_memtag)<<4))

#define VAL_BITS_MODIFY(var, bitnum, bitmask, bitval) do { \
    var &= ~(bitmask); \
    var |= ((bitval) << (bitnum)) & (bitmask); \
} while (0)

static int tv_enabled;
static int visio_enabled;*/

/*
 * devid records the switch type.
 * This allow to adapt dynamically switch access functions to switch type.
 */
//static unsigned short devid = SWITCH_DEVID_6021;
/*
 * Port number conversion bitmaps.
 * From user pointer of view, port number is independant from switch (and board) type.
 * A table entry is the logical (user space) port number
 * and the entry content is the switch port number.
 * log_to_sw_port() is the conversion function.
 * Depending on the board type detected, sw_port_map will address one of the bitmaps.
 */
/*static int sw_port_map_lb1_6021[3] = {0, 1, 2};
static int sw_port_map_lb1_6061[3] = {0, 1, 5};
static int *sw_port_map;

#define VTU_VLAN_MEMBER_EGRESS_UNMODIFIED 0
#define VTU_VLAN_NOT_MEMBER \
    ((devid == SWITCH_DEVID_6021) ? 0x1 : 0x3)
#define VTU_VLAN_MEMBER_EGRESS_UNTAGGED \
    ((devid == SWITCH_DEVID_6021) ? 0x2 : 0x1)
#define VTU_VLAN_MEMBER_EGRESS_TAGGED \
    ((devid == SWITCH_DEVID_6021) ? 0x3 : 0x2)

extern unsigned short sierra_sw_reg_read(int addr, int reg);
extern void sierra_sw_reg_write(int addr, int reg, unsigned short val);*/

/* Conversion functions */

/*static int log_to_sw_port(port_num_t lport)
{
        if (lport >= MAX_PORTS_NUM || lport < 0)
                return UNSUPPORTED;
	return sw_port_map[lport];
}*/

/* Switch access functions: basic management */

/*static unsigned short switch_read_identifier(int sport)
{
    unsigned short val;

    val = sierra_sw_reg_read(PORT_REGS_ADDR(sport), SWITCH_IDENTIFIER_REG);
    VAL_BITS_MODIFY(val, SWITCH_REVID_BIT_N, SWITCH_REVID_BIT_MASK, 0);
    return val >> SWITCH_DEVID_BIT_N;
}

static void port_set_8021q_mode(int sport, int mode)
{
    unsigned short val;

    if (devid == SWITCH_DEVID_6021) {
	val = sierra_sw_reg_read(PORT_REGS_ADDR(sport), PORT_VLAN_MAP_REG);
	VAL_BITS_MODIFY(val, PORT_8021Q_MODE_BIT_N, PORT_8021Q_MODE_BIT_MASK, mode);
	sierra_sw_reg_write(PORT_REGS_ADDR(sport), PORT_VLAN_MAP_REG, val);
    } else {
	val = sierra_sw_reg_read(PORT_REGS_ADDR(sport), PORT_CONTROL2_REG);
	VAL_BITS_MODIFY(val, PORT_8021Q_MODE_BIT_N, PORT_8021Q_MODE_BIT_MASK, mode);
	sierra_sw_reg_write(PORT_REGS_ADDR(sport), PORT_CONTROL2_REG, val);
    }
}

static void port_set_default_vid(int sport, int vid)
{
    unsigned short val;

    val = sierra_sw_reg_read(PORT_REGS_ADDR(sport), PORT_DEF_VID_REG);
    VAL_BITS_MODIFY(val, PORT_DEF_VID_BIT_N, PORT_DEF_VID_BIT_MASK, vid);
    sierra_sw_reg_write(PORT_REGS_ADDR(sport), PORT_DEF_VID_REG, val);
}*/

/* Switch access functions: VTU management */

/*static void vtu_flush(void)
{
    unsigned short val;

    while ((val = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) &
	VTU_BUSY_BIT);
    
    VAL_BITS_MODIFY(val, VTU_OP_BIT_N, VTU_OP_BIT_MASK, VTU_OP_FLUSH);
    val |= VTU_BUSY_BIT;
    sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_OP_REG, val);

    while (sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG) & VTU_BUSY_BIT);
}

static void vtu_entry_set(int is_load, int vid, int vtu_data, int vtu_data45)
{
    unsigned short op, val;*/

    /* prepare 'op' register value */
/*    while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) &
	VTU_BUSY_BIT);
    VAL_BITS_MODIFY(op, VTU_OP_BIT_N, VTU_OP_BIT_MASK, VTU_OP_LOAD);
    op |= VTU_BUSY_BIT;*/
    
    /* write vtu vid register */
/*    val = vid & VTU_VID_BIT_MASK;
    if (is_load)
	val |= VTU_VID_VALID_BIT; *//* purge/load bit */
//    sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_VID_REG, val);
    /* write vtu data register */
/*    if (is_load) {
	sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_DATA_PORTS_REG, vtu_data);
	if (devid == SWITCH_DEVID_6061)
	    sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_DATA_PORTS45_REG, vtu_data45);
    }
*/
    /* execute the load/purge command */
/*    sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_OP_REG, op);
    while (sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG) & VTU_BUSY_BIT);
}

static void vtu_entry_load(int vid, int p0_memtag,int p1_memtag, int p2_memtag,
	int p3_memtag,int p4_memtag, int p5_memtag)
{
    unsigned short vtu_data, vtu_data45;

    if (devid == SWITCH_DEVID_6021) {
	vtu_data = VTU_DATA_REG_VAL(p0_memtag, p1_memtag, p2_memtag, 0);
    } else {
	vtu_data = VTU_DATA_REG_VAL(p0_memtag, p1_memtag, p2_memtag, p3_memtag);
	vtu_data45 = VTU_DATA_REG_VAL(p4_memtag, p5_memtag, 0, 0);
    }
    vtu_entry_set(1, vid, vtu_data, vtu_data45);
}

static void vtu_entry_purge(int vid)
{
    vtu_entry_set(0, vid, 0, 0);
}*/

/* Switch driver main functions */

/*static void malign_services_set(void)
{
    int memtag[6], i;
    for (i = 0; i < 6; i++)
	memtag[i] = VTU_VLAN_NOT_MEMBER;*/
    /* Set default VID per port */
/*    port_set_default_vid(log_to_sw_port(PORT0),
	visio_enabled ? VISIO_VID : DEF_VID);
    port_set_default_vid(log_to_sw_port(PORT1), tv_enabled ? TV_VID : DEF_VID);
    port_set_default_vid(log_to_sw_port(MII_PORT), DEF_VID);*/

    /* set vid 100 vtu entry:
     * phy port 0 - standard config: egress untagged. visio: not a member
     * phy port 1 - standard config: egress untagged. tv: not a member
     * mii port - egress tagged. */
/*    memtag[log_to_sw_port(PORT0)] = (visio_enabled ? VTU_VLAN_NOT_MEMBER
	: VTU_VLAN_MEMBER_EGRESS_UNTAGGED);
    memtag[log_to_sw_port(PORT1)] = (tv_enabled ? VTU_VLAN_NOT_MEMBER
	: VTU_VLAN_MEMBER_EGRESS_UNTAGGED);
    memtag[log_to_sw_port(MII_PORT)] = VTU_VLAN_MEMBER_EGRESS_TAGGED;
    vtu_entry_load(DEF_VID, memtag[0], memtag[1], memtag[2], memtag[3],
	memtag[4], memtag[5]);*/

    /* set vid 2 vtu entry (if tv enabled):
     * phy port 0 - not a member
     * phy port 1 - egress untagged
     * mii port - egress tagged. */
/*    if (tv_enabled)
    {
	memtag[log_to_sw_port(PORT0)] = VTU_VLAN_NOT_MEMBER;
	memtag[log_to_sw_port(PORT1)] = VTU_VLAN_MEMBER_EGRESS_UNTAGGED;
	memtag[log_to_sw_port(MII_PORT)] = VTU_VLAN_MEMBER_EGRESS_TAGGED;
	vtu_entry_load(TV_VID, memtag[0], memtag[1], memtag[2], memtag[3],
	    memtag[4], memtag[5]);
    }
    else
	vtu_entry_purge(TV_VID);
*/
    /* set vid 1 vtu entry (if visio enabled):
     * phy port 0 - egress untagged
     * phy port 1 - not a member
     * mii port - egress tagged. */
/*    if (visio_enabled)
    {
	memtag[log_to_sw_port(PORT0)] = VTU_VLAN_MEMBER_EGRESS_UNTAGGED;
	memtag[log_to_sw_port(PORT1)] = VTU_VLAN_NOT_MEMBER;
	memtag[log_to_sw_port(MII_PORT)] = VTU_VLAN_MEMBER_EGRESS_TAGGED;
	vtu_entry_load(VISIO_VID, memtag[0], memtag[1], memtag[2], memtag[3],
	    memtag[4], memtag[5]);
    }
    else
	vtu_entry_purge(VISIO_VID);
}

void m88e6021_init(void)
{
    devid = switch_read_identifier(0);
    if (devid == SWITCH_DEVID_6021)
	sw_port_map = sw_port_map_lb1_6021;
    else
	sw_port_map = sw_port_map_lb1_6061;*/
	/* enable ports 0, 1 */
/*	sierra_sw_reg_write(PORT_REGS_ADDR(0), PORT_CONTROL_REG, 0x107f);
	sierra_sw_reg_write(PORT_REGS_ADDR(1), PORT_CONTROL_REG, 0x107f);
    if (devid == SWITCH_DEVID_6021) {*/
	/* enable port 2 */
/*	sierra_sw_reg_write(PORT_REGS_ADDR(2), PORT_CONTROL_REG, 0x107f);
    } else if (devid == SWITCH_DEVID_6061) {*/
	/* enable port 5 */
/*	sierra_sw_reg_write(PORT_REGS_ADDR(5), PORT_CONTROL_REG, 0x107f);
    }*/
    /* Put switch ports into 8021Q mode */
/*    port_set_8021q_mode(log_to_sw_port(PORT0), PORT_8021Q_MODE_FALLBACK);
    port_set_8021q_mode(log_to_sw_port(PORT1), PORT_8021Q_MODE_FALLBACK);
    port_set_8021q_mode(log_to_sw_port(MII_PORT), PORT_8021Q_MODE_FALLBACK);*/
    /* flush all VTU entries */
//    vtu_flush();
    /* set default mode */
/*    malign_services_set();
}


void m88ReadConfiguration(void)
{
  unsigned int _vlan = 0;
  unsigned int op;


  tv_enabled = visio_enabled = 0;
  
  do {
	while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) & VTU_BUSY_BIT);
  
	VAL_BITS_MODIFY(op, VTU_OP_BIT_N, VTU_OP_BIT_MASK, VTU_OP_GET_NEXT);
	op |= VTU_BUSY_BIT;

	if (_vlan == 0) {
	  sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_VID_REG, 0x0fff);
	}
	sierra_sw_reg_write(GLOBAL_REGS_ADDR, VTU_OP_REG, op);
	while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) & VTU_BUSY_BIT);

	_vlan = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_VID_REG);
	if ((_vlan & ~(1<<12)) == VISIO_VID) {
	  visio_enabled = 1;
	}
	else if ((_vlan & ~(1<<12)) == TV_VID) {
	  tv_enabled = 1;
	}	  
  } while (_vlan & (1 << 12));
}*/
// End AVX  
//#endif /* JUNGO } */


/********************************************************************
 * Function Name : si3216_initialize.				    *
 * Desccription  : Function to initalize si3216.                    *
 *******************************************************************/
int si3216_initialize(unsigned char codecNo)
{
  unsigned char  dummy = 0;
  int i,j;
  unsigned short  temp[5];
  int timeSlotPcmPhone1,timeSlotPcmPhone2;
  unsigned short uiCountry;
  UINT16 slicGPIOReset;

  if (*(volatile UINT32*)0xb900003c == 0x6836) {
    slicGPIOReset = 0;	// Default to Fast@3202
#if TARGET_ALASKA
    slicGPIOReset = 11;
#endif
#ifdef CONFIG_SAGEM_BOARD_FAST_3202
    printk("SLIC board FAST_3202\n");
    slicGPIOReset = 0;
#endif
#ifdef CONFIG_SAGEM_FAST34XX
    printk("SLIC board FAST_34XX\n");
    slicGPIOReset = 10;
#endif
#ifdef CONFIG_SAGEM_BOXER
    printk("SLIC board BOXER_P1\n");
    slicGPIOReset = 3;
#endif
    DEBUG(PRINT("SLIC Initializaton for Silabs on VX160 Platform \n"),DEBUG_INFO);
  }
  else {
/* SAGEM -> [Hard V1] { */
    if (*(volatile UINT32*)0xb900003c == 0x6834) {
      DEBUG(PRINT("SLIC Initializaton for Silabs on VX150-0 Platform \n"),DEBUG_INFO);
      slicGPIOReset = 15;
    }
    else {
/* LUA -> [ Bug 1661 ] { */
      if (*(volatile UINT32*)0xb900003c == 0x6833) {
        DEBUG(PRINT("SLIC Initializaton for Silabs on VX150-1 Platform \n"),DEBUG_INFO);
        slicGPIOReset = 15;
      }
/* } */
      else {
        slicGPIOReset = 6;
        DEBUG(PRINT("SLIC Initializaton for Silabs on ???? Platform \n"),DEBUG_INFO);
      }
    }
/* } */
  }

  /* AN35 STEP -1 ,RESET THE CODEC */
  if(codecNo == 0)  
  {
//#ifdef 0 //JUNGO /* { */
//     m88ReadConfiguration();	//AVX
//#endif /* JUNGO } */

    //osLockInterrupts();
    
    if(slicGPIOReset > 15)
    {
      *(volatile UINT16*)GPIO_FLAG_EXT_RESET = (1 << (slicGPIOReset-16));
      /* Sleep for about 1000  ms */
      mdelay(1000);
      /* Release RESET */
      *(volatile UINT16*)GPIO_FLAG_EXT_SET = (1 << (slicGPIOReset-16));
    }
    else
    {
      *(volatile UINT16*)GPIO_FLAG_RESET = (1 << slicGPIOReset);
      /* Sleep for about 1000  ms */
      mdelay(1000);
      /* Release RESET */
      *(volatile UINT16*)GPIO_FLAG_SET = (1 << slicGPIOReset);
    }

//#ifdef 0 //JUNGO /* { */
//AVX
/*    {
     unsigned int op;

	spiInit(10);
      mdelay(1000);*/
	  // Wait while the busy bit is set
//	  while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) & VTU_BUSY_BIT);
		
	  /* re-init switch on RG1, for which phone and switch resets are common. */
//	  *(volatile unsigned int *)0xb9110058 = 0x107f;
//	  *(volatile unsigned int *)0xb911005c = 0x0f04;
	  // Wait while the busy bit is set
//	  while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) & VTU_BUSY_BIT);

//	  *(volatile unsigned int *)0xb9110058 = 0x107f;
//	  *(volatile unsigned int *)0xb911005c = 0x0f24;
	  // Wait while the busy bit is set
/*	  while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) & VTU_BUSY_BIT);

	  *(volatile unsigned int *)0xb9110058 = 0x107f;
	  *(volatile unsigned int *)0xb911005c = 0x0f44;*/
	  // Wait while the busy bit is set
//	  while ((op = sierra_sw_reg_read(GLOBAL_REGS_ADDR, VTU_OP_REG)) & VTU_BUSY_BIT);
		
	// Re-initialize the switch
//        m88e6021_init();
//    }
//endAVX
//#endif /* JUNGO } */

    mdelay(1000);
    ASSERT_CHIPSELECT(0);
    udelay(100);
    DEASSERT_CHIPSELECT(0);


    /***************enable daisy chaining***********/
    ASSERT_CHIPSELECT(0);
    spiInitOperation(0,&dummy);
    spiInitOperation(0x80,&dummy);
    DEASSERT_CHIPSELECT(0);
   // osUnlockInterrupts();
  }

  /* Step 8 , Do the sanity Check First */
  if ( (readDirectReg(codecNo, 8) != 2) ||
       (readDirectReg(codecNo, 64) != 0)    ||
       (readDirectReg(codecNo, 11) != 0x33) )
  {
    DEBUG(PRINT("\r\n UNABLE TO COMMUNICATE WITH SI3216. codecNo %d \r\n",
			    codecNo), DEBUG_EXCEPTION);
    return -1;
  }
  
  /* Read CHIP Type and CHIP Version */
  silabChipType[codecNo] = ((0x30 & readDirectReg(codecNo, 0)) >> 4);
  silabChipVersion[codecNo] = (0xf & readDirectReg(codecNo, 0));

  DEBUG(PRINT("Codec[%d] => Chip Type [%d]  Chip Version [%d]\n",codecNo, 
		silabChipType[codecNo],
		silabChipVersion[codecNo]), DEBUG_INFO);

// Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
  	confGetCountryOpts(&uiCountry);
	si3216Modify_country_params(uiCountry, codecNo);

  /* STEP => 9, Initialize Indirect Registers */
  initializeIndirectRegisters(codecNo);
  /* Step => 9, Write Indirect Registers 22-26 each to 0x8000
   * Save the defined values for later use */
  for ( i = 0 ; i <= 4 ; i++ )
  {
    j = i+22 ;
    temp[i] = readIndirectReg(codecNo, j);
    writeIndirectReg(codecNo, j, 0x8000);
  }

  /* STEP => 10,  Take si3216 out of loopback mode */
  writeDirectReg(codecNo,8,0x00);
  writeDirectReg(codecNo,108, 0xAB);
// UNDOCUMENTED Step of AN 35 Si321x	
  writeDirectReg(codecNo, 67, 0x17); // Make VBat switch not automatic
        // The above is a safety measure to prevent Q7 from accidentaly turning on and burning out.
        //  It works in combination with the statement below:
        //  Pin 34 DCDRV which is used for the battery switch on the Si3211 & Si3212
  writeDirectReg(codecNo, 66, 1);  // Q7 should be set to OFF for si3210

// Step 11 of AN 35 Si321x
        /*  Write Battery Voltages */
  writeDirectReg(codecNo, 74, 0x2F);  // High Batterie Voltage VBATH = 70,5V / 1.5V = 47 (0x2F).
  writeDirectReg(codecNo, 75, 0x14);  // Low Batterie Voltage VBATL = 30V / 1.5V = 20 (0x14).


  /* STEP => 12, perform DC-DC Calibration */
  silabPowerUp(codecNo);
//#if 0

  /* Do Power leak Test */
  silabPowerLeakTest(codecNo);
  /* Turn ON DC-DC convertor again */

  silabPowerUp(codecNo);
//#endif
  /* Step => 13 */
  writeDirectReg(codecNo, 64,0);
  /* Step => 14 */
  silabCalibrate(codecNo);

// Step 20 of AN 35 Si321x
  /* Flush out the energy accumulators */
        for ( i = 0 ; i <= 7 ; i++ ) // Zeroing Indirect Registers 75-82
  {
    j = i+75 ;
    writeIndirectReg(codecNo, j, 0x0);
  }
        writeIndirectReg(codecNo, 84, 0); // Scratch Pad zeroing
        for ( i = 0 ; i <= 3 ; i++ ) // Zeroing Indirect Registers 208-211
  {
    j = i+208 ;
    writeIndirectReg(codecNo, j, 0x0);
  }
  /* Step => 21 */
  /* Clear out the interrupts */
//// MOVED AFTER STEP 28
  /* Step => 22 */
  if(confGetPCMMode(codecNo) != LINEAR)
  {
    timeSlotPcmPhone1 = 16;
    timeSlotPcmPhone2 = 24;
  }
  else 
  {
    timeSlotPcmPhone1 = 32;
    timeSlotPcmPhone2 = 48;
  }
  /*  Initialize PCM and Clock Slot Assignment (Direct Registers 2-5 and 1) */
  if (codecNo == 0)
  {
    writeDirectReg(codecNo,2,timeSlotPcmPhone1);
    writeDirectReg(codecNo,3,0);
    writeDirectReg(codecNo,4,timeSlotPcmPhone1);
    writeDirectReg(codecNo,5,0);
  }
  else
  {
    writeDirectReg(codecNo,2,timeSlotPcmPhone2);
    writeDirectReg(codecNo,3,0);
    writeDirectReg(codecNo,4,timeSlotPcmPhone2);
    writeDirectReg(codecNo,5,0);
  }

  /* Reg->1 PCM Mode Select
     MU_LAW -> 0x28,
     A_LAW  -> 0x20
     LINEAR -> 0x3D
  */
  if(confGetPCMMode(codecNo) == MU_LAW)
  {
    // PCM Enable, U-Law, 8 kHz, 8-bit, 1 PCLK, tri-state bit 0 on positive.
    writeDirectReg(codecNo, 1, 0x28);
  }
  else if(confGetPCMMode(codecNo) == A_LAW)
  {
    // PCM Enable, A-Law, 8 kHz, 8-bit, 1 PCLK, tri-state bit 0 on positive.
    writeDirectReg(codecNo, 1, 0x20);
  }
  else
  {
    // Sagem 20080226
    // PCM Enable, Linear, 8 kHz, 16-bit, 1 PCLK, tri-state bit 0 on positive.
    //writeDirectReg(codecNo, 1, 0x3D);
    writeDirectReg(codecNo, 1, 0x3C);
  }

  /* Set the Impedence */
  /* Step => 23 */
  writeDirectReg(codecNo,8,0);
  writeDirectReg(codecNo,9,0x05);
  /* Two wire impedence synthesis enabled */
//  confGetCountryOpts(&country);

  /* Let's keep USA as country settings at SLIC level. */
//  country = USA;	/* Sagem 20060120 */
//  if(country == USA)
//    writeDirectReg(codecNo,10,0x8);
//  else if(country == FRANCE)
//    writeDirectReg(codecNo,10,0x2c);
// Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
  if(chipData[codecNo].impedance==TYPE_600_OHMS)
    writeDirectReg(codecNo, DR10_LINE_IMPEDANCE, LINE_IMPEDANCE_600_DR10);
  else if(chipData[codecNo].impedance==TYPE_CTR21)
    writeDirectReg(codecNo, DR10_LINE_IMPEDANCE, LINE_IMPEDANCE_CRT21_DR10);

  writeDirectReg(codecNo,11,0x33);

  /* Step => 24 */
        /*  Initialize OSC1,OSC2,Ringing,Pulse Metering (Direct Registers 32-51) */
// Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
//        writeDirectReg(codecNo, 34, 0x1A); // Enable Ringing Timer With Ringing Voltage Offset
//        writeDirectReg(codecNo, 48, 0xE0); // Ringing Active Timer = 125uS * 0x2EE0 = 1.5S
//        writeDirectReg(codecNo, 49, 0x2E);
//        writeDirectReg(codecNo, 50, 0x60); // Ringing Inactive Timer = 125uS * 0x6D60 = 3.5S
//        writeDirectReg(codecNo, 51, 0x6D);
  { // Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
	ringStruct* ptrRingTable=NULL;

	switch(chipData[codecNo].country)
	{
		case TYPE_FRANCE: ptrRingTable=&franceRingTable; break;
		case TYPE_NETHERLANDS: ptrRingTable=&netherlandsRingTable; break;
		case TYPE_POLAND: ptrRingTable=&polandRingTable; break;
		case TYPE_SPAIN: ptrRingTable=&spainRingTable; break;
	}
	writeDirectReg(codecNo, DR34_RING_OSC_CTL, 0x1A); // Enable Ringing Timer With Ringing Voltage Offset

	writeDirectReg(codecNo, DR48_RING_ON_LO, ptrRingTable->onLsbByte);
	writeDirectReg(codecNo, DR49_RING_ON_HI, ptrRingTable->onMsbByte);
	writeDirectReg(codecNo, DR50_RING_OFF_LO, ptrRingTable->offLsbByte);
	writeDirectReg(codecNo, DR51_RING_OFF_HI, ptrRingTable->offMsbByte);

// Step 25 of AN 35 Si321x
        /*  Initialize Detect Thresholds and Filters (Direct Registers 63 , 67 , 69  , 70) */
//// Note : Direct Registers 63 is unchanged (= 0x54)
//// Note : Direct Registers 67 is unchanged (= 0x17)
     //      Loop Closure Debounce Interval = 12 (0x0C) * 1.25mS = 15 mS.
     writeDirectReg(codecNo, 69, 0x0C); // Set the Loop Closure Debounce Interval

// Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
//	writeDirectReg(codecNo, 70, 0x05); // Ring Trip Detect Debounce Interval = 1.25mS * 0x05 = 6,25mS.
	writeDirectReg(codecNo, DR70_RT_DEBOUNCE, ptrRingTable->rtDebounce);
  }

  /* Step => 26 */
        /*  Initialize DC Feed Parameters (Direct Registers 65-67 , 71-73) */
//// Note : Direct Registers 65 is unchanged (= 0x61)
//// Note : Direct Registers 66 is unchanged (= 0x01)
//// Note : Direct Registers 67 is unchanged (= 0x17).

        /*  Write Loop Current Limit */
	writeDirectReg(codecNo, 71, 0x06); // Loop Current Limit = 20 mA + 0x06 * 3 mA/LSB = 38 mA.
// Sagem 20080226 : FRANCE/NETHERLANDS/POLAND
	writeDirectReg(codecNo, 72, 0x1F); // On-Hook Line Voltage VOC = 0x1F * 1,5 V = 46 V
// SAGEM LCe 16-11-05: tension raccroche a 42 V
//	writeDirectReg(codecNo, DR72_ON_HOOK_V, 0x1C); // On-Hook Line Voltage VOC = 0x1c * 1,5 V = 42 V

//// Note : Direct Registers 73 is unchanged (default value = 0x02 => VCM = -3V)

  /* Step => 27 */
        /* Flush Done restore coefficients */
  for ( i = 0 ; i <= 4 ; i++ )
  {
    j = i+22;
    writeIndirectReg(codecNo, j, temp[i]);
  }
  /* Step => 28 */
  silabVerifyIndirectRegisters(codecNo);

/* ############################################################################ */
/*Hta*/
/* Initialize the interrupt  registers */
/* Writing ones to the status registers clears out any pending interrupts */
        writeDirectReg(codecNo, DR18_INTRPT_STATUS1,0xFF);
        writeDirectReg(codecNo, DR19_INTRPT_STATUS2, 0xFF);
        writeDirectReg(codecNo, DR20_INTRPT_STATUS3, 0xFF);
        writeDirectReg(codecNo, DR21_INTRPT_MASK1, 0x00);
        writeDirectReg(codecNo, DR22_INTRPT_MASK2, 0x00);
        writeDirectReg(codecNo, DR23_INTRPT_MASK3, 0x00);

        si3216GoActive(codecNo);  // Forward Active Linefeed.
/* ############################################################################ */

  DEBUG(PRINT("\n End of SI3216 Initialization. codecNo %d \r\n", codecNo),
                                                                DEBUG_INFO);
/*  silabOverWriteDefault(codecNo); */

  return 0;
}


/*======================================================================
 * Function Name        : slicModeSetting
 * Description          : setting the slic mode
 *======================================================================
 */
void slicModeSetting_silabs(int codecNo, int chNo, int mode)
{
   unsigned char data;
   char pol_FXO;

// LCe OpenRG Hollande
#ifdef FIRMWARE_CONF_COUNTRY_NL
   pol_FXO = si3050_pola_pos;
#else
   pol_FXO = 0;
#endif


// LCe OpenRG Hollande

        switch(mode)
        {
                case SLIC_STANDBY:
                        writeDirectReg(codecNo, 64, FORWARD_ACTIVE_DR64);
//printk("\nSLIC_STANDBY\n");
                break;

                case SLIC_ACTIVE:
		   if (pol_FXO == 1)
                        writeDirectReg(codecNo, 64, REVERSE_ACTIVE_DR64);
		   else
                        writeDirectReg(codecNo, 64, FORWARD_ACTIVE_DR64);
printk("\nSLIC_ACTIVE %d\n", pol_FXO);
                break;

                case SLIC_RING:
		   data = readDirectReg(codecNo, 68);
		   // Apply SLIC_RING only if SLIC_ONHOOK
		   if ( (data & 0x03) == 0 ) 
			writeDirectReg(codecNo, 64, RINGING_DR64);
printk("\nSLIC_RING %d\n", pol_FXO);
                break;

                case SLIC_REVERSE_RINGOFF:
		  data = readDirectReg(codecNo, 68);
		  if (pol_FXO == 1)
			{
			if ( (data & 0x03) == 0 ) 
				writeDirectReg(codecNo, 64, OHT_DR64);
			else
				writeDirectReg(codecNo, 64, FORWARD_ACTIVE_DR64);
			}
		  else
			{
			if ( (data & 0x03) == 0 ) 
				writeDirectReg(codecNo, 64, REVERSE_OHT_DR64);
			else
				writeDirectReg(codecNo, 64, REVERSE_ACTIVE_DR64);
			}
printk("\nSLIC_REVERSE_RINGOFF %d\n", pol_FXO);
                break;

                case SLIC_RINGOFF:
		  data = readDirectReg(codecNo, 68);
		  if (pol_FXO == 0)
			{
			if ( (data & 0x03) == 0 ) 
				writeDirectReg(codecNo, 64, OHT_DR64);
			else
				writeDirectReg(codecNo, 64, FORWARD_ACTIVE_DR64);
			}
		  else
			{
			if ( (data & 0x03) == 0 ) 
				writeDirectReg(codecNo, 64, REVERSE_OHT_DR64);
			else
				writeDirectReg(codecNo, 64, REVERSE_ACTIVE_DR64);
			}
printk("\nSLIC_RINGOFF %d\n", pol_FXO);
                break;

                case SLIC_REVERSE_ACTIVE:
		  data = readDirectReg(codecNo, 68);
		  if ((data & 0x03) == 0)	// ON HOOK
			{
			if (pol_FXO == 1)
                        	writeDirectReg(codecNo, 64, OHT_DR64);
			else
	                        writeDirectReg(codecNo, 64, REVERSE_OHT_DR64);
			}
		  else				// OFF HOOK
			{
			if (pol_FXO == 1)
                        	writeDirectReg(codecNo, 64, FORWARD_ACTIVE_DR64);
			else
	                        writeDirectReg(codecNo, 64, REVERSE_ACTIVE_DR64);
			}

printk("\nSLIC_REVERSE_ACTIVE %d\n", pol_FXO);
                break;

                default:
                break;
    }
}

/*======================================================================
 * Function Name        : codecReadStatus
 * Description          : read the codec status register and return the value
 *======================================================================
 */
unsigned short codecReadStatus_silabs(int codecNo)
{
  unsigned short data;
  data = readDirectReg(codecNo, 68);
  if ( (data & 0x03) != 0 ) 
    data = 0x01 << codecNo;
  else 
    data = 0x0;
  return data;
}

/*======================================================================
 * Function Name        : si3216_gain_ctrl
 * Description          : modifie register IR13 (DAC)
 *======================================================================
 */
void si3216_gain_ctrl(unsigned char port, unsigned char option)
{
unsigned int level=NL_RECV_STD_GAIN_NORMAL;

switch (option)
	{
	case REC_GAIN_VERY_SOFT:	level = NL_RECV_STD_GAIN_VERYSOFT; 	break;
	case REC_GAIN_SOFT:		level = NL_RECV_STD_GAIN_SOFT; 		break;
	case REC_GAIN_NORMAL:	level = NL_RECV_STD_GAIN_NORMAL;		break;
	case REC_GAIN_LOUD:		level = NL_RECV_STD_GAIN_LOUD; 		break;
	case REC_GAIN_VERY_LOUD:	level = NL_RECV_STD_GAIN_VERYLOUD; 	break;
	}

//control for max value
level &= 0x7FF0;

writeIndirectReg(port, 13, level);
}
