/*======================================================================
 *
 * Copyright : (c) 2004 SAGEM SA - Site Velizy - URD2
 *
 * The information and source code contained herein is the exclusive
 * property of SAGEM and may not be disclosed, examined, or
 * reproduced in whole or in part without explicit written authorization
 * of the company.
 *
 *======================================================================
 */

/*======================================================================
 * File Name    :       si3216_func.c
 * Description  :
 * CVS ID       :
 *======================================================================
 */

/*======================================================================
 * History
 *
 * Date                 Author          Comment
 * Mars 2004            DTr             Initial version
 * Mars 2004            HTa             Next    version
 *======================================================================
 */


/* Silab Driver File */
#include <oswrapper.h>
#include "include/si3216.h"
#include "include/france_const.h"
#include "include/netherlands_const.h"
#include "include/poland_const.h"
#include "include/spain_const.h"

extern chipStruct chipData[MAX_SLIC_CHANNELS];  

extern unsigned short readIndirectReg(unsigned char codecNo, unsigned char address);
extern unsigned char readDirectReg(unsigned char codecNo, unsigned char address);
extern void writeIndirectReg(unsigned char codecNo, unsigned char address, unsigned short data);
extern void writeDirectReg(unsigned char codecNo, unsigned char address, unsigned char data);

/******************************************************************************/
/* goActive: Set the ProSlic Linefeed State to Forward Active                 */
/*                                                                            */
/*       INPUTS:                                                              */
/*                                                                            */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void si3216GoActive(unsigned char codecNo)
{
        writeDirectReg(codecNo, DR64_LINE_STATE, FORWARD_ACTIVE_DR64); /* LOOP STATE REGISTER SET TO ACTIVE */

        //hta tskSleep(100); // 100m second
         mdelay(100);
}

//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//		FUNCTIONS FOR SI3216 TONE
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
void si3216StopMute(unsigned char codecNo)
{
	writeDirectReg(codecNo, DR9_AUDIO_GAIN, 0x05); // AD & DA GAINS = -3.5dB
}
void si3216StartTxRxMute(unsigned char codecNo)
{
        writeDirectReg(codecNo, DR9_AUDIO_GAIN, 0x25);
}

void si3216StartMute(unsigned char codecNo)
{
	writeDirectReg(codecNo, DR9_AUDIO_GAIN, 0x15);
}

/******************************************************************************/
/* disableOscillators: Disable Oscillators Except Ringing                     */
/*                                                                            */
/*       INPUTS:                                                              */
/*                                                                            */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void si3216DisableOsc1(unsigned char codecNo)
{
	// Turns of OSC1
	writeDirectReg(codecNo, DR32_OSC1, DISABLE_ALL_DR32);
}

void si3216DisableOsc2(unsigned char codecNo)
{
	// Turns of OSC2
	writeDirectReg(codecNo, DR33_OSC2, DISABLE_ALL_DR33);
}

/******************************************************************************/
/* startTone: Stop Tone                                                       */
/*                                                                            */
/*       INPUTS: numtone : Num of Tone Selected                               */
/*                                                                            */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void si3216StopToneOsc1(unsigned char codecNo)
{
	//int intr;

	//hta intr = splnet();
	/* Disable Interrupts */
	osLockInterrupts();
  	
	chipData[codecNo].toneCadenceOsc1.tone = TYPE_NULL_TONE;
	chipData[codecNo].toneCadenceOsc1.time = 0;
	chipData[codecNo].toneCadenceOsc1.counter = 0;
	chipData[codecNo].toneCadenceOsc1.nextCadenceEntryIndex = 0;

	si3216DisableOsc1(codecNo);
	
	//hta splx(intr);
	 /* Enable Interrupts */
	osUnlockInterrupts();	
}

void si3216StopToneOsc2(unsigned char codecNo)
{
	//int intr;

	//hta intr = splnet();
	/* Disable Interrupts */
	osLockInterrupts();
        
	chipData[codecNo].toneCadenceOsc2.tone = TYPE_NULL_TONE;
	chipData[codecNo].toneCadenceOsc2.time = 0;
	chipData[codecNo].toneCadenceOsc2.counter = 0;
	chipData[codecNo].toneCadenceOsc2.nextCadenceEntryIndex = 0;
	
	si3216DisableOsc2(codecNo);

	//hta splx(intr);
	/* Enable Interrupts */
	osUnlockInterrupts();	 
}

void si3216StopTone(unsigned char codecNo)
{
	si3216StopToneOsc1(codecNo);
	si3216StopToneOsc2(codecNo);
}

/******************************************************************************/
/* genTone: Set the Tone Values                                               */
/*                                                                            */
/*       INPUTS: tone : Tone Characteristics Values                           */
/*                                                                            */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void si3216GenToneOsc1(unsigned char codecNo, Oscillator osc)
{
	// function to extract values for oscillators from toneStruct
	// place them in the registers, and enable the oscillators.
	unsigned char	osc1_ontimer_enable = 0;
	unsigned char	osc1_offtimer_enable = 0;
			   
	si3216DisableOsc1(codecNo); // Make sure the oscillators are not already on.
	if (osc.coeff == 0 || osc.x == 0)
	{
		// No Tone Defined
		return;
	}
	
	// Setup osc 1
	writeIndirectReg(codecNo, IR0_OSC1_COEF, osc.coeff);
	writeIndirectReg(codecNo, IR1_OSC1X, osc.x);
	writeIndirectReg(codecNo, IR2_OSC1Y, osc.y);
	// Active Timer
	if ( (osc.onLsbByte != 0) || (osc.onMsbByte != 0) )
	{
		writeDirectReg(codecNo, DR36_OSC1_ON_LO, osc.onLsbByte);
		writeDirectReg(codecNo, DR37_OSC1_ON_HI, osc.onMsbByte);
		osc1_ontimer_enable = ENABLE_OSC1_ACTIVE_DR32 ;
	}
			   
			    
	// Inactive Timer
	if ( (osc.offLsbByte != 0) ||(osc.offMsbByte != 0) )
	{
		writeDirectReg(codecNo, DR38_OSC1_OFF_LO, osc.offLsbByte);
		writeDirectReg(codecNo, DR39_OSC1_OFF_HI, osc.offMsbByte);
		osc1_offtimer_enable = ENABLE_OSC1_INACTIVE_DR32 ;
	}
			  
	writeDirectReg(codecNo, DR32_OSC1, (unsigned char)(START_OSC1_DR32 | osc1_ontimer_enable | osc1_offtimer_enable));
	return;
}

void si3216GenToneOsc2(unsigned char codecNo, Oscillator osc)
{
	// function to extract values for oscillators from toneStruct
	// place them in the registers, and enable the oscillators.
	unsigned char	osc2_ontimer_enable = 0;
	unsigned char	osc2_offtimer_enable = 0;

	si3216DisableOsc2(codecNo); // Make sure the oscillators are not already on.
	if (osc.coeff == 0 || osc.x == 0)
	{
		// No Tone Defined
		return;
	}

	// Setup OSC 2
	writeIndirectReg(codecNo, IR3_OSC2_COEF, osc.coeff);
	writeIndirectReg(codecNo, IR4_OSC2X, osc.x);
	writeIndirectReg(codecNo, IR5_OSC2Y, osc.y);

	// Active Timer
	if ( (osc.onLsbByte != 0) || (osc.onMsbByte != 0) )
	{
		writeDirectReg(codecNo, DR40_OSC2_ON_LO, osc.onLsbByte);
		writeDirectReg(codecNo, DR41_OSC2_ON_HI, osc.onMsbByte);
		osc2_ontimer_enable = ENABLE_OSC2_ACTIVE_DR33 ;
	}

	// Inactive Timer
	if ( (osc.offLsbByte != 0) ||(osc.offMsbByte != 0) )
	{
		writeDirectReg(codecNo, DR42_OSC2_OFF_LO, osc.offLsbByte);
		writeDirectReg(codecNo, DR43_OSC2_OFF_HI, osc.offMsbByte);
		osc2_offtimer_enable = ENABLE_OSC2_INACTIVE_DR33 ;
	}

	writeDirectReg(codecNo, DR33_OSC2, (unsigned char)(START_OSC2_DR33 | osc2_ontimer_enable | osc2_offtimer_enable));
	return;
}

void si3216debugFxsGenTone(unsigned char country, unsigned char tone)
{
	unsigned char * ptrCountry=NULL;

	switch(country)
	{
		case TYPE_FRANCE: 		ptrCountry="FRANCE"; break;
		case TYPE_NETHERLANDS: 	ptrCountry="NETHERLANDS"; break;
		case TYPE_POLAND: 		ptrCountry="POLAND"; break;
		case TYPE_SPAIN: 		ptrCountry="SPAIN"; break;
	}

	if (ptrCountry==NULL) return;

	switch(tone)
	{
		// TONES
		case 1 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DIAL_TONE");break;

		case 2 :
			if(strcmp(ptrCountry,"POLAND")==0)
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"CALL_IN_PROGRESS");
			else
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"NULL_CALL_IN_PROGRESS");
		break;

		case 3 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"RING_BACK_TONE");break;
		case 4 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"BUSY_TONE");break;

		case 5 :
			if(strcmp(ptrCountry,"NETHERLANDS")==0)
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"NULL_MSG_WAITING_TONE");
			else
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"MSG_WAITING_TONE");
			break;

		case 6 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"CALL_WAITING_TONE");break;

		case 7 :
			if(strcmp(ptrCountry,"NETHERLANDS")==0)
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"NULL_ALERTING_TONE");
			else
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"ALERTING_TONE");
			break;

		// VOIP TONES
		case 8 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"VOIP_DIAL_TONE");break;

		case 9 :
			if(strcmp(ptrCountry,"FRANCE")==0)
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"VOIP_MWI_TONE");
			else
				printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"NULL_VOIP_MWI_TONE");
		break;

		// DTMF_DIG_0-DIG_HASH_TONE
		case 10 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_0");break;
		case 11 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_1");break;
		case 12 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_2");break;
		case 13 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_3");break;
		case 14 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_4");break;
		case 15 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_5");break;
		case 16 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_6");break;
		case 17 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_7");break;
		case 18 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_8");break;
		case 19 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_9");break;
		case 20 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_A");break;
		case 21 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_B");break;
		case 22 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_C");break;
		case 23 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DTMF_DIG_D");break;
		case 24 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DIG_STAR_TONE");break;
		case 25 : printk("\n ***fxsGenTone %s : %s*** \n", ptrCountry,"DIG_HASH_TONE");break;
	}
}

/******************************************************************************/
/* startTone: Start Tone Selected                                             */
/*                                                                            */
/*       INPUTS: numTone : Num of Tone Selected                               */
/*                                                                            */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void si3216StartTone(unsigned char codecNo, unsigned short numTone)
{
	int x;
	int y;
	unsigned int toneOn, toneOff;
	toneCadenceTables *prtToneCadenceTables;

	chipData[codecNo].toneCadenceOsc1.tone = numTone;
	chipData[codecNo].toneCadenceOsc1.time = 0;
	chipData[codecNo].toneCadenceOsc1.counter = 0;
	chipData[codecNo].toneCadenceOsc1.nextCadenceEntryIndex = 0;

	chipData[codecNo].toneCadenceOsc2.tone = numTone;
	chipData[codecNo].toneCadenceOsc2.time = 0;
	chipData[codecNo].toneCadenceOsc2.counter = 0;
	chipData[codecNo].toneCadenceOsc2.nextCadenceEntryIndex = 0;
	
	x = 0;
	y = chipData[codecNo].toneCadenceOsc1.tone;

#if 0 // debug
si3216debugFxsGenTone(chipData[codecNo].country, numTone);
#endif

	switch (chipData[codecNo].country)
	{
		case TYPE_FRANCE:
			if (numTone >= FR_TONE_END) return;
			prtToneCadenceTables = &franceToneCadenceTables[y][x];
		break;
		case TYPE_NETHERLANDS:
			if (numTone >= NL_TONE_END) return;
			prtToneCadenceTables = &netherlandsToneCadenceTables[y][x];
		break;
		case TYPE_POLAND:
			if (numTone >= PL_TONE_END) return;
			prtToneCadenceTables = &polandToneCadenceTables[y][x];
		break;
		case TYPE_SPAIN:
			if (numTone >= SP_TONE_END) return;
			prtToneCadenceTables = &spainToneCadenceTables[y][x];
		break;
		default :
			DEBUG(PRINT("UNKNOWN CountryType for TONES\r\n"),DEBUG_INFO);
			return;
		break;
	}

	if (prtToneCadenceTables->nextOsc1 != 0)
	{
		toneOn = 0;
		toneOff = 0;
		toneOn = (prtToneCadenceTables->toneRegs.osc1.onMsbByte << 8) +
				prtToneCadenceTables->toneRegs.osc1.onLsbByte;
		toneOff = (prtToneCadenceTables->toneRegs.osc1.offMsbByte << 8) +
				prtToneCadenceTables->toneRegs.osc1.offLsbByte;
		chipData[codecNo].toneCadenceOsc1.time = ((toneOn + toneOff) * 125) / 10000;
		chipData[codecNo].toneCadenceOsc1.counter = 0;

		/* Start current coordinate in the table */
		si3216GenToneOsc1(codecNo, prtToneCadenceTables->toneRegs.osc1);

		/* change coordinate */
		chipData[codecNo].toneCadenceOsc1.nextCadenceEntryIndex =
							prtToneCadenceTables->nextOsc1;
	}
	else
	{
		si3216GenToneOsc1(codecNo, prtToneCadenceTables->toneRegs.osc1);
	}

	if (prtToneCadenceTables->nextOsc2 != 0)
	{
		toneOn = 0;
		toneOff = 0;
		toneOn = (prtToneCadenceTables->toneRegs.osc2.onMsbByte << 8) +
				prtToneCadenceTables->toneRegs.osc2.onLsbByte;
		toneOff = (prtToneCadenceTables->toneRegs.osc2.offMsbByte << 8) +
				prtToneCadenceTables->toneRegs.osc2.offLsbByte;
		chipData[codecNo].toneCadenceOsc2.time = ((toneOn + toneOff) * 125) / 10000;
		chipData[codecNo].toneCadenceOsc2.counter = 0;

		/* Start current coordinate in the table */
		si3216GenToneOsc2(codecNo, prtToneCadenceTables->toneRegs.osc2);

		/* change coordinate */
		chipData[codecNo].toneCadenceOsc2.nextCadenceEntryIndex =
							prtToneCadenceTables->nextOsc2;
	}
	else
	{
		si3216GenToneOsc2(codecNo, prtToneCadenceTables->toneRegs.osc2);
	}
}

/******************************************************************************/
/* nextTone: Go to the Next Cadence for Tone                                  */
/*                                                                            */
/*       INPUTS:                                                              */
/*                                                                            */
/*       RETURNS:                                                             */
/*       OUTPUTS:                                                             */
/*       NOTE(S):                                                             */
/******************************************************************************/
void si3216NextToneOsc1(unsigned char codecNo)
{
	int x;
	int y;
	unsigned int toneOn, toneOff;
	toneCadenceTables *prtToneCadenceTables;

	x = chipData[codecNo].toneCadenceOsc1.nextCadenceEntryIndex;
	y = chipData[codecNo].toneCadenceOsc1.tone;

	switch (chipData[codecNo].country)
	{
		case TYPE_FRANCE:
			prtToneCadenceTables = &franceToneCadenceTables[y][x];
		break;
		case TYPE_NETHERLANDS:
			prtToneCadenceTables = &netherlandsToneCadenceTables[y][x];
		break;
		case TYPE_POLAND:
			prtToneCadenceTables = &polandToneCadenceTables[y][x];
		break;
		case TYPE_SPAIN:
			prtToneCadenceTables = &spainToneCadenceTables[y][x];
		break;
		default:
			DEBUG(PRINT("UNKNOWN CountryType for TONES\r\n"),DEBUG_INFO);
			return;
		break;
	}

	if (chipData[codecNo].toneCadenceOsc1.time != 0)
	{
		chipData[codecNo].toneCadenceOsc1.counter++;

		if (chipData[codecNo].toneCadenceOsc1.counter >= chipData[codecNo].toneCadenceOsc1.time)
		{
			toneOn = 0;
			toneOff = 0;
			toneOn = (prtToneCadenceTables->toneRegs.osc1.onMsbByte << 8) +
					prtToneCadenceTables->toneRegs.osc1.onLsbByte;
			toneOff = (prtToneCadenceTables->toneRegs.osc1.offMsbByte << 8) +
					prtToneCadenceTables->toneRegs.osc1.offLsbByte;
			chipData[codecNo].toneCadenceOsc1.time = ((toneOn + toneOff) * 125) / 10000;
			chipData[codecNo].toneCadenceOsc1.counter = 0;

			/* Start current coordinate in the table */
			si3216GenToneOsc1(codecNo, prtToneCadenceTables->toneRegs.osc1);

			/* change coordinate */
			chipData[codecNo].toneCadenceOsc1.nextCadenceEntryIndex =
								prtToneCadenceTables->nextOsc1;
		}
	}
}

void si3216NextToneOsc2(unsigned char codecNo)
{
	int x;
	int y;
	unsigned int toneOn, toneOff;
	toneCadenceTables *prtToneCadenceTables;
	
	x = chipData[codecNo].toneCadenceOsc2.nextCadenceEntryIndex;
	y = chipData[codecNo].toneCadenceOsc2.tone;

	switch (chipData[codecNo].country)
	{
		case TYPE_FRANCE:
			prtToneCadenceTables = &franceToneCadenceTables[y][x];
		break;
		case TYPE_NETHERLANDS:
			prtToneCadenceTables = &netherlandsToneCadenceTables[y][x];
		break;
		case TYPE_POLAND:
			prtToneCadenceTables = &polandToneCadenceTables[y][x];
		break;
		case TYPE_SPAIN:
			prtToneCadenceTables = &spainToneCadenceTables[y][x];
		break;
		default:
			DEBUG(PRINT("UNKNOWN CountryType for TONES\r\n"),DEBUG_INFO);
			return;
		break;
	}

	if (chipData[codecNo].toneCadenceOsc2.time != 0)
	{
		chipData[codecNo].toneCadenceOsc2.counter++;

		if (chipData[codecNo].toneCadenceOsc2.counter >= chipData[codecNo].toneCadenceOsc2.time)
		{
			toneOn = 0;
			toneOff = 0;
			toneOn = (prtToneCadenceTables->toneRegs.osc2.onMsbByte << 8) +
					prtToneCadenceTables->toneRegs.osc2.onLsbByte;
			toneOff = (prtToneCadenceTables->toneRegs.osc2.offMsbByte << 8) +
					prtToneCadenceTables->toneRegs.osc2.offLsbByte;
			chipData[codecNo].toneCadenceOsc2.time = ((toneOn + toneOff) * 125) / 10000;
			chipData[codecNo].toneCadenceOsc2.counter = 0;

			/* Start current coordinate in the table */
			si3216GenToneOsc2(codecNo, prtToneCadenceTables->toneRegs.osc2);

			/* change coordinate */
			chipData[codecNo].toneCadenceOsc2.nextCadenceEntryIndex =
							prtToneCadenceTables->nextOsc2;
		}
	}
}

void si3216NextTone(unsigned char codecNo)
{
	si3216NextToneOsc1(codecNo);
	si3216NextToneOsc2(codecNo);
}

void si3216ClearTimerTonesIT(unsigned char codecNo)
{
	writeDirectReg(codecNo, DR18_INTRPT_STATUS1, 0x03);
}

void si3216ITGenTones(unsigned char codecNo, unsigned char data)
{
	writeDirectReg(codecNo, DR21_INTRPT_MASK1, data);
}
unsigned char si3216ReadOsc1ITTones(unsigned char codecNo)
{
	unsigned char itTonesStat = 0x00;
	itTonesStat = readDirectReg(codecNo, DR18_INTRPT_STATUS1) & 0x03;
	return itTonesStat;
}
