
#include <linux/module.h>
#include <linux/tty.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>


#include <oswrapper.h>
#include "include/slic_proc.h"
//#include "include/si3216.h"

extern void osLockInterrupts(void);
extern void osUnlockInterrupts(void);
extern void writeDirectReg(unsigned char codecNo, unsigned char address, unsigned char data);
extern void writeIndirectReg(unsigned char codecNo, unsigned char address,unsigned short data);
extern unsigned char readDirectReg(unsigned char codecNo, unsigned char address);
extern unsigned short readIndirectReg(unsigned char codecNo, unsigned char address);

#define BOARD_2FXS	0

// name for the file;
#define si3216_proc_filename0 "si3216-0"
static struct proc_dir_entry * si3216_dir_entry0; // procfs filename 

#if BOARD_2FXS
#define si3216_proc_filename1 "si3216-1"
static struct proc_dir_entry * si3216_dir_entry1; // procfs filename 
#endif

// read  a register, if indirect != 0 => indirect register 
static unsigned short si3216_read_register(int regn, int indirect, unsigned char num)
{
unsigned short ret;
  osLockInterrupts();	/* Sagem qualif only */

    if (indirect) ret = readIndirectReg(num, regn);
    else ret = readDirectReg(num, regn);
  osUnlockInterrupts(); /* Sagem qualif only */
	return ret;
}

// write a register, if indirect != 0 => indirect register
static void si3216_write_register(int regn, unsigned short value, int indirect, unsigned char num)
{
  osLockInterrupts();	/* Sagem qualif only */

    if (indirect) writeIndirectReg(num, regn, value);
    else writeDirectReg(num,regn, (char)value);
  osUnlockInterrupts(); /* Sagem qualif only */

}

// procfs write function :

static int si3216_procfs_write (struct file * file, const char * buffer, unsigned long count, void * data, unsigned char num)
{
    int reg;
    unsigned int val;
    int indirect;
#ifdef FIRMWARE_CONF_COUNTRY_NL
    int i;
    unsigned char codec_pol;
#endif

    // warning data comes from userland !
    char cmdbuf [40] = {0}; // this is too much, but that's ok at least
    
    if ( count > 39 ) return -EFAULT;

    if (copy_from_user (cmdbuf, buffer, count))
    {
	printk("SLIC: si3216 can't get copy_from_user !!\n");
	return -EFAULT;
    }
    
    switch (slic_proc_decode_cmd(cmdbuf, count,  &reg, &val, &indirect))
    {
	case SLIC_PROC_WRITE_REG:
		si3216_write_register(reg, (short)val,  indirect, num);
		return count;
		break;

#ifdef FIRMWARE_CONF_COUNTRY_NL
	case SLIC_PROC_LINE_MONITORING:
		// save returned value
		voltmonitor = val;
		osLockInterrupts();	/* Sagem qualif only */
		for (i=0 ; i<MAX_SLIC_CHANNELS ; i++)
			{
			if (val == LINE_MONITORING_YES)
				{ // only if codec is ONHOOK
				if (codecReadStatus_silabs(i) == SLIC_ONHOOK)
					{
					codec_pol = (readDirectReg(i, 72) & 0x40);
					if (codec_pol == 0x40)
						{
						writeDirectReg(i, 72, 0x5C);
						}
					else
						{
						writeDirectReg(i, 72, 0x1C);
						}
					}
				}
			else
				{ // do it every time
				codec_pol = (readDirectReg(i, 72) & 0x40);
				if (codec_pol == 0x40)
					{
					writeDirectReg(i, 72, 0x55);
					}
				else
					{
					writeDirectReg(i, 72, 0x15);
					}
				}
			}
		osUnlockInterrupts(); /* Sagem qualif only */
		return count;
		break;
#endif
	default:
		printk("error\n");
		return -EFAULT;
    } 

    return -EFAULT;
}


// structure used to define all registers :

struct register_info 
{
    int number; 	// the register number
    char * name;	// the name of the register
    int indirect;		// is the register direct or not (1 => indirect)
};

// big register information  ! funcky tabs make viewing aligned, don't touch it, or you will regret it
static struct register_info register_description [] = 
{
	{0,  "(spi md)		",  0},
	{1,  "(pcm md)		",  0},
	{2,  "(pcm xmit lsb)	",  0},
	{3,  "(pcm xmit msb)	",  0},
	{4,  "(pcm rcv lsb)	",  0},
	{5,  "(pcm rcv msb)	",  0},
	{6,  "(pni)		",  0},
	{8,  "(audio lpbk)	",  0},
	{9,  "(audio gain)	",  0},
	{10, "(line imped)	",  0},
	{11, "(hybr)		",  0},
	{14, "(pwr dn1)	",  0},
	{15, "(pwr dn2)		",  0},
	{18, "(intrpt stat1)	",  0},
	{19, "(intrpt stat2)	",  0},
	{20, "(intrpt stat3)	",  0},
	{21, "(intrpt msk1)	",  0},
	{22, "(intrpt msk2)	",  0},
	{23, "(intrpt msk3)	",  0},
	{28, "(i data low)	",  0},
	{29, "(i data hgh)	",  0},
	{30, "(i addr)		",  0},
	{31, "(i stat)		",  0},
	{32, "(osc1)		",  0},
	{33, "(osc2)		",  0},
	{34, "(ring osc ctl)	",  0},
	{36, "(osc1 on lo)	",  0},
	{37, "(osc1 on hi)	",  0},
	{38, "(osc1 off lo)	",  0},
	{39, "(osc1 off hi)	",  0},
	{40, "(osc2 on lo)	",  0},
	{41, "(osc2 on hi)	",  0},
	{42, "(osc2 off lo)	",  0},
	{43, "(osc2 off hi)	",  0},
	{48, "(ring on lo)	",  0},
	{49, "(ring on hi)	",  0},
	{50, "(ring off lo)	",  0},
	{51, "(ring off hi)	",  0},
	{52, "(fsk data  )	",  0},
	{63, "(loop closure)	",  0},
	{64, "(ln stat   )	",  0},
	{65, "(bias squelch)	",  0},
	{66, "(bat feed  )	",  0},
	{67, "(auto state)	",  0},
	{68, "(lp stat   )	",  0},
	{69, "(lp debnce )	",  0},
	{70, "(rt debnce )	",  0},
	{71, "(lp i limit)	",  0},
	{72, "(on hook v )	",  0},
	{73, "(cmn v)		",  0},
	{74, "(bat v hi  )	",  0},
	{75, "(bat v lo)	",  0},
	{76, "(pwr stat dev)	",  0},
	{77, "(pwr stat  )	",  0},
	{78, "(lp v sns  )	",  0},
	{79, "(lp i sns)	",  0},
	{80, "(tip v sns )	",  0},
	{81, "(ring v sns)	",  0},
	{82, "(bat v hi sns)	",  0},
	{83, "(bat v lo sns)	",  0},
	{84, "(iq1)		",  0},
	{85, "(iq2)		",  0},
	{86, "(iq3)		",  0},
	{87, "(iq4)		",  0},
	{88, "(iq5)		",  0},
	{89, "(iq6)		",  0},
	{92, "(dcdc pwm prd)	",  0},
	{93, "(dcdc pwm time)	",  0},
	{94, "(dcdc pw off)	",  0},
	{96, "(calibr1   )	",  0},
	{97, "(calibr2   )	",  0},
	{98, "(ring gn cal)	",  0},
	{99, "(tip gn cal)	",  0},
	{100,"(diff i cal)	",  0},
	{101,"(cmn i cal)	",  0},
	{102,"(i lmt gn cal)	",  0},
	{103,"(adc off cal)	",  0},
	{104,"(dac adc off)	",  0},
	{105,"(dac off cal)	",  0},
	{107,"(dc peak cal)	",  0},
	{108,"(enhce enabl)	",  0},
/* SI3216 INDIRECT REGISTER ADDRESS (decimal) */
	{0, "(osc1 coef  )	",  1},/* Oscillators */
	{1, "(osc1x)		",  1},
	{2, "(osc1y)		",  1},
	{3, "(osc2 coef  )	",  1},
	{4, "(osc2x)		",  1},
	{5, "(osc2y)		",  1},
	{6, "(ring v off)	",  1},
	{7, "(ring osc coef)	",  1},
	{8, "(ring x)		",  1},
	{9, "(ring y)		",  1},
	{13, "(recv digt gn)	",  1},/* Digital Programmable Gain/attenuation */
	{14, "(xmit digt gn)	",  1},
	{15, "(lp close thrsh)	",  1},/* SLIC Control */
	{16, "(ring trip thrsh	",  1},
	{17, "(cmn min thresh)	",  1},
	{18, "(cmn max thresh)	",  1},
	{19, "(pwr alrm q1q2)	",  1},
	{20, "(pwr alrm q3q4)	",  1},
	{21, "(pwr alrm q5q6)	",  1},
	{22, "(lp closr fltr)	",  1},
	{23, "(ring trp fltr)	",  1},
	{24, "(thm lp pol q1q2)	",  1},
	{25, "(thm lp pol q3q4)",  1},
	{26, "(thm lp pol q5q6)	",  1},
	{27, "(cm bias ringng)	",  1},
	{64, "(dcdc min v)	",  1},
	{66, "(lp cls thsh low)",  1},
	{69, "(fsk x0)		",  1}, /* FSK Control */
	{70, "(fsk coef 0)	",  1},
	{71, "(fsk x1)		",  1},
	{72, "(fsk coef 1)	",  1},
	{73, "(fsk x01)		",  1},
	{74, "(fsk x10)	",  1},
};

static int register_number = 114;	// number of elements of the last array


#define N_REG_PER_LINE 2

// Read method used by user to read the procfs file :
int si3216_procfs_read(char * buff, char **buffchar, off_t offset, int buffer_length, int * eof, void * data, unsigned char num)
{
    static char buffer [PAGE_SIZE]; // this is huge, for test only !
    static int n;
    int ret = 0;

    // first reset the packet ...
    if (offset > 0 ) 
    {
	// Now we are here, is there still some stuffs to do ?
	if (offset >= n )
	{
	    *eof = 1;
	    ret = 0;
	} else {
	    // Stills some to do ... alrigth.
	    if (buffer_length > (n - offset ))
	    {
		memcpy(buff + offset, buffer + offset, (n - offset));
		ret = n; // return total read from begin
		*eof = 1;
	    } else {
		// still some more
		
		memcpy(buff + offset, buffer + offset, buffer_length);
		ret = buffer_length + offset; // must return total read from the beginning
	    }
	    
	}

    } else {

	// if we are at zero, then generate the buffer ...
	int i;
	
	n = 0;

	memset (buffer, 0, 3*1024);

	if (num == 0)
		n += sprintf(buffer + n, "# Slic si3216 FXS 0, SPI Bus 1:0\n");
#if BOARD_2FXS
	if (num == 1)
		n += sprintf(buffer + n, "# Slic si3216 FXS 1, SPI Bus 1:1\n");
#endif

	for (i = 0; i < register_number ; i++)
	{
	    n += sprintf(buffer + n, "r%.2d %s = %.4X %s", 
			  register_description[i].number,
			  register_description[i].name,
			  si3216_read_register(register_description[i].number, register_description[i].indirect, num),
			  (register_description[i].indirect) ? "I" : "D"
		    );
	    // pagination 
	    if (  (((i+1) % N_REG_PER_LINE) == 0 ) && ( (i!=0) && (N_REG_PER_LINE != 1))  ) 
	    {
		n += sprintf(buffer + n, "\n");
	    }
	    else 
	    {
		n += sprintf(buffer + n, "  ");
	    }
	}
	if ((i+1) % N_REG_PER_LINE != 0) n += sprintf(buffer + n, "\n");
/*
	if (num == 0)
		n += sprintf(buffer + n, "# Use 'echo rN=0xXXXX > /proc/bus/spi/slic/" si3216_proc_filename0 "' to modify a direct register\n");
#if BOARD_2FXS
	if (num == 1)
		n += sprintf(buffer + n, "# Use 'echo rN=0xXXXX > /proc/bus/spi/slic/" si3216_proc_filename1 "' to modify a direct register\n");
#endif

	n += sprintf(buffer + n, "#  - where N is the number of the register\n");
	n += sprintf(buffer + n, "#  - and XX the hexadecimal value you want to write in\n");
	if (num == 0)
		n += sprintf(buffer + n, "# or  'echo rN=0xXXXX indirect > /proc/bus/spi/slic/" si3216_proc_filename0 "' to modify an indirect register\n");
#if BOARD_2FXS
	if (num == 1)
		n += sprintf(buffer + n, "# or  'echo rN=0xXXXX indirect > /proc/bus/spi/slic/" si3216_proc_filename1 "' to modify an indirect register\n");
#endif

	n ++; // 0 character
	n += sprintf(buffer + n, "# %d bytes read\n", n - 1); //Use this to tune the buffer size
*/
	// buffer is now ready, and n contains the size of it ...
	// return the first buffer_length buffer ..
	if (n >  buffer_length) 
	{
	    memcpy(buff, buffer, buffer_length);
	    //ret = buffer_length;
	    ret = buffer_length;
	} else {
	    memcpy(buff, buffer, n);
	    ret = n;
	    *eof = 1;
	}
    }
    return ret;
}

int si3216_procfs_read0(char * buff, char **buffchar, off_t offset, int buffer_length, int * eof, void * data)
{
int ret;
ret = si3216_procfs_read( buff, buffchar, offset, buffer_length, eof, data, 0);
return ret;
}

static int si3216_procfs_write0 (struct file * file, const char * buffer, unsigned long count, void * data)
{
int ret;
ret = si3216_procfs_write( file, buffer, count, data, 0);
return ret;
}

#if BOARD_2FXS
int si3216_procfs_read1(char * buff, char **buffchar, off_t offset, int buffer_length, int * eof, void * data)
{
int ret;
ret = si3216_procfs_read( buff, buffchar, offset, buffer_length, eof, data, 1);
return ret;
}

static int si3216_procfs_write1 (struct file * file, const char * buffer, unsigned long count, void * data)
{
int ret;
ret = si3216_procfs_write( file, buffer, count, data, 1);
return ret;
}

#endif

// Called to init the procfs ( to create a file ... in the proc) 
int slic_si3216_proc_init(void)
{
    si3216_dir_entry0 = create_proc_entry(si3216_proc_filename0, 0644, proc_bus_spi_slic);
    if (!si3216_dir_entry0)
    {
	printk("SLIC : can't create si3216-0 proc entry\n");
	return 1;
    }

    si3216_dir_entry0->read_proc = si3216_procfs_read0;
    si3216_dir_entry0->write_proc = si3216_procfs_write0;
    si3216_dir_entry0->owner = THIS_MODULE;
    si3216_dir_entry0->mode = S_IFREG | 0644;
    si3216_dir_entry0->uid = 0;
    si3216_dir_entry0->gid = 0;
    si3216_dir_entry0->size = 2305; // change the file size !! 

#if BOARD_2FXS
    si3216_dir_entry1 = create_proc_entry(si3216_proc_filename1, 0644, proc_bus_spi_slic);
    if (!si3216_dir_entry1)
		{
		printk("SLIC : can't create si3216 proc entry\n");
		return 1;
		}

    si3216_dir_entry1->read_proc = si3216_procfs_read1;
    si3216_dir_entry1->write_proc = si3216_procfs_write1;
    si3216_dir_entry1->owner = THIS_MODULE;
    si3216_dir_entry1->mode = S_IFREG | 0644;
    si3216_dir_entry1->uid = 0;
    si3216_dir_entry1->gid = 0;
    si3216_dir_entry1->size = 2305; // change the file size !! 
#endif

    return 0;
}

// Called to cleanup the module procfs entry
void slic_si3216_proc_cleanup(void)
{
    remove_proc_entry(si3216_proc_filename0, proc_bus_spi_slic);

#if BOARD_2FXS
    remove_proc_entry(si3216_proc_filename1, proc_bus_spi_slic);
#endif
}
