
#include "include/si3216.h"
#include "include/spain_const.h"

const ringStruct spainRingTable;
const toneCadenceTables spainToneCadenceTables [SP_TONE_END][SP_MAX_NUM_CADENCE];

/*
The following Array contains SPAIN Ring Parmeters
*/
const ringStruct  spainRingTable = {
	// TYPE_SPAIN : Ring 25Hz 35V 1,5s On 3s Off
	SP_RING_OSC_FREQ_25HZ_IR7,
	SP_RING_OSC_AMP_25HZ_IR8,
	SP_RING_OSC_INITPHASE_25HZ_IR9, 
	SP_RING_ON_1S5_LSB_DR48,
	SP_RING_ON_1S5_MSB_DR49,
	SP_RING_OFF_3S_LSB_DR50,
	SP_RING_OFF_3S_MSB_DR51,
	SP_RING_TRIP_THRESH_25HZ_IR16,
	SP_RING_TRIP_FILTER_25HZ_IR23,
	SP_RT_DEBOUNCE_25HZ_DR70
};

/*
The following Array contains SPAIN Tones Parmeters
*/
const toneCadenceTables  spainToneCadenceTables [SP_TONE_END][SP_MAX_NUM_CADENCE] =
{
	{ // SP_NULL_TONE
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_DIAL_TONE	PGn: application norme espagnole "ITE-CA-001-v05"
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -10,5dBm0 continuous
				SP_DIAL_IR0,
				SP_DIAL_IR1,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_CALL_IN_PROGRESS	PGn:"ITE-CA-001-v05" ne specifie pas de tonalite speciale --> on conserve celle de Poland !...
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 425Hz -10,5dBm0 200ms On 200ms Off
				SP_CALL_IN_PROGRESS_C0_IR0,
				SP_CALL_IN_PROGRESS_C0_IR1,
				0x00,
				SP_CALL_IN_PROGRESS_C0_DR36,
				SP_CALL_IN_PROGRESS_C0_DR37,
				SP_CALL_IN_PROGRESS_C0_DR38,
				SP_CALL_IN_PROGRESS_C0_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_RING_BACK_TONE	PGn: application norme espagnole "ITE-CA-001-v05"
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -10,5dBm0 1,5s On 3s Off
				SP_RINGBACK_IR0,
				SP_RINGBACK_IR1,
				0x00,
				SP_RINGBACK_DR36,
				SP_RINGBACK_DR37,
				SP_RINGBACK_DR38,
				SP_RINGBACK_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_BUSY_TONE	PGn: application norme espagnole "ITE-CA-001-v05"
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 425Hz 10,5dBm0 0.2s On 0.2s Off
				SP_BUSY_IR0,
				SP_BUSY_IR1,
				0x00,
				SP_BUSY_DR36,
				SP_BUSY_DR37,
				SP_BUSY_DR38,
				SP_BUSY_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_MSG_WAITING_TONE	PGn:"ITE-CA-001-v05" ne specifie pas de tonalite speciale --> on conserve celle de Poland !...
		{
		1, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.2s Off
				SP_MSG_WAITING_C013_IR0,
				SP_MSG_WAITING_C013_IR1,
				0x00,
				SP_MSG_WAITING_C013_DR36,
				SP_MSG_WAITING_C013_DR37,
				SP_MSG_WAITING_C013_DR38,
				SP_MSG_WAITING_C013_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
			{
		2, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.2s Off
				SP_MSG_WAITING_C013_IR0,
				SP_MSG_WAITING_C013_IR1,
				0x00,
				SP_MSG_WAITING_C013_DR36,
				SP_MSG_WAITING_C013_DR37,
				SP_MSG_WAITING_C013_DR38,
				SP_MSG_WAITING_C013_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		3, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.4s On 0.4s Off
				SP_MSG_WAITING_C2_IR0,
				SP_MSG_WAITING_C2_IR1,
				0x00,
				SP_MSG_WAITING_C2_DR36,
				SP_MSG_WAITING_C2_DR37,
				SP_MSG_WAITING_C2_DR38,
				SP_MSG_WAITING_C2_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		4, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.2s Off
				SP_MSG_WAITING_C013_IR0,
				SP_MSG_WAITING_C013_IR1,
				0x00,
				SP_MSG_WAITING_C013_DR36,
				SP_MSG_WAITING_C013_DR37,
				SP_MSG_WAITING_C013_DR38,
				SP_MSG_WAITING_C013_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.5s Off
				SP_MSG_WAITING_C4_IR0,
				SP_MSG_WAITING_C4_IR1,
				0x00,
				SP_MSG_WAITING_C4_DR36,
				SP_MSG_WAITING_C4_DR37,
				SP_MSG_WAITING_C4_DR38,
				SP_MSG_WAITING_C4_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_CALL_WAITING_TONE	PGn: application norme espagnole "ITE-CA-001-v05"
		{
		1, // Next Cadence 1 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 425Hz -14,5dBm0 0.175s On 0.175s Off
				SP_CALL_WAIT_C0_IR0,
				SP_CALL_WAIT_C0_IR1,
				0x00,
				SP_CALL_WAIT_C0_DR36,
				SP_CALL_WAIT_C0_DR37,
				SP_CALL_WAIT_C0_DR38,
				SP_CALL_WAIT_C0_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		0, // Next Cadence 0 Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 425Hz -14,5dBm0 0.175s On 3s Off
				SP_CALL_WAIT_C1_IR0,
				SP_CALL_WAIT_C1_IR1,
				0x00,
				SP_CALL_WAIT_C1_DR36,
				SP_CALL_WAIT_C1_DR37,
				SP_CALL_WAIT_C1_DR38,
				SP_CALL_WAIT_C1_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_ALERTING_TONE	Kesako ?
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1 2130Hz -9dBm0 80ms On
				SP_ALERTING_IR0,
				SP_ALERTING_IR1,
				0x00,
				SP_ALERTING_DR36,
				SP_ALERTING_DR37,
				0x00,
				0x00
				},
				{ // OSC2 2750Hz -9dBm0 80ms On
				SP_ALERTING_IR3,
				SP_ALERTING_IR4,
				0x00,
				SP_ALERTING_DR40,
				SP_ALERTING_DR41,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_VOIP_DIAL_TONE	PGn: (= SP_DIAL_TONE) application norme espagnole "ITE-CA-001-v05"
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -10,5dBm0 continuous
				SP_VOIP_DIAL_IR0,
				SP_VOIP_DIAL_IR1,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_VOIP_MSG_WAITING_TONE 	PGn:"ITE-CA-001-v05" ne specifie pas de tonalite speciale --> on conserve celle de Poland !...
		{
		1, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.2s Off
				SP_MSG_WAITING_C013_IR0,
				SP_MSG_WAITING_C013_IR1,
				0x00,
				SP_MSG_WAITING_C013_DR36,
				SP_MSG_WAITING_C013_DR37,
				SP_MSG_WAITING_C013_DR38,
				SP_MSG_WAITING_C013_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
			{
		2, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.2s Off
				SP_MSG_WAITING_C013_IR0,
				SP_MSG_WAITING_C013_IR1,
				0x00,
				SP_MSG_WAITING_C013_DR36,
				SP_MSG_WAITING_C013_DR37,
				SP_MSG_WAITING_C013_DR38,
				SP_MSG_WAITING_C013_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		3, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.4s On 0.4s Off
				SP_MSG_WAITING_C2_IR0,
				SP_MSG_WAITING_C2_IR1,
				0x00,
				SP_MSG_WAITING_C2_DR36,
				SP_MSG_WAITING_C2_DR37,
				SP_MSG_WAITING_C2_DR38,
				SP_MSG_WAITING_C2_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		4, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.2s Off
				SP_MSG_WAITING_C013_IR0,
				SP_MSG_WAITING_C013_IR1,
				0x00,
				SP_MSG_WAITING_C013_DR36,
				SP_MSG_WAITING_C013_DR37,
				SP_MSG_WAITING_C013_DR38,
				SP_MSG_WAITING_C013_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
		{
		0, // No Next Cadence Osc1
		0, // No Next Cadence Osc2
			{
				{ // OSC1	425Hz -6dBm0 0.2s On 0.5s Off
				SP_MSG_WAITING_C4_IR0,
				SP_MSG_WAITING_C4_IR1,
				0x00,
				SP_MSG_WAITING_C4_DR36,
				SP_MSG_WAITING_C4_DR37,
				SP_MSG_WAITING_C4_DR38,
				SP_MSG_WAITING_C4_DR39
				},
				{ // OSC2 Not Used
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00,
				0x00
				},
			},
		},
	},

	{ // SP_DTMF_DIG_0_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				SP_DTMF_FREQ_941Hz,
				SP_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1336Hz,
				SP_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_1_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz 11dBm0 80ms On
				SP_DTMF_FREQ_697Hz,
				SP_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1209Hz,
				SP_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_2_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz -11dBm0 80ms On
				SP_DTMF_FREQ_697Hz,
				SP_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1336Hz,
				SP_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_3_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz -11dBm0 80ms On
				SP_DTMF_FREQ_697Hz,
				SP_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1477Hz,
				SP_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_4_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				SP_DTMF_FREQ_770Hz,
				SP_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1209Hz,
				SP_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_5_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				SP_DTMF_FREQ_770Hz,
				SP_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1336Hz,
				SP_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_6_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				SP_DTMF_FREQ_770Hz,
				SP_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1477Hz,
				SP_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_7_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				SP_DTMF_FREQ_852Hz,
				SP_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1209Hz,
				SP_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_8_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				SP_DTMF_FREQ_852Hz,
				SP_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1336Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1336Hz,
				SP_DTMF_F1336Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_9_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				SP_DTMF_FREQ_852Hz,
				SP_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1477Hz,
				SP_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_A_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 697Hz -11dBm0 80ms On
				SP_DTMF_FREQ_697Hz,
				SP_DTMF_F697Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1633Hz,
				SP_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_B_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 770Hz -11dBm0 80ms On
				SP_DTMF_FREQ_770Hz,
				SP_DTMF_F770Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1633Hz,
				SP_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_C_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 852Hz -11dBm0 80ms On
				SP_DTMF_FREQ_852Hz,
				SP_DTMF_F852Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1633Hz,
				SP_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_D_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				SP_DTMF_FREQ_941Hz,
				SP_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1633Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1633Hz,
				SP_DTMF_F1633Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_STAR_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				SP_DTMF_FREQ_941Hz,
				SP_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1209Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1209Hz,
				SP_DTMF_F1209Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
	
	{ // SP_DTMF_DIG_HASH_TONE_LOCAL
		{
		0,  // No Next Cadence Osc1
		0,  // No Next Cadence Osc1
			{
				{ // OSC1 941Hz -11dBm0 80ms On
				SP_DTMF_FREQ_941Hz,
				SP_DTMF_F941Hz_AMP_M11dBm0,
				0x00,
				SP_DTMF_DR36,
				SP_DTMF_DR37,
				0x00,
				0x00
				},
				{ // OSC2 1477Hz -9dBm0 80ms On
				SP_DTMF_FREQ_1477Hz,
				SP_DTMF_F1477Hz_AMP_M9dBm0,
				0x00,
				SP_DTMF_DR40,
				SP_DTMF_DR41,
				0x00,
				0x00
				},
			},
		},
	},
};


