/* ============================================================================
 *
 * Copyright (C) 2001 Analog Devices Inc. All rights reserved
 *
 * The information and source code contained herein is the exclusive property
 * of Analog Devices and may not be disclosed, examined or reproduced in whole
 * or in part without the explicit written authorization from Analog Devices.
 *
 * ============================================================================
 */
#include<oswrapper.h>
#include<voicespi.h>

/********************************************************************
 *  Function to read SPI Status.                                    *
 *******************************************************************/
static UINT16 spiGetStatus(void)
{
  UINT16 status = *((volatile UINT16 *)SPI_STATUS_ADDRESS);
  return status;
}

static void spiGetData(UCHAR  *data )
{
  UINT16 value = (*(volatile UINT16 *)(SPI_BASE_ADDRESS + 0x10));;
  *data = value;
}
/********************************************************************
 *  Function Name : spiEnable.                                      *
 *  Description   : This function enables SPI module.               *
 *  Input         : None.                                           *
 *  Output        : None.                                           *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiEnable(void)
{
  volatile UINT16 spi_value = *(volatile UINT16*)SPI_CTRL_ADDRESS;
  spi_value |= (0x1 << SPI_SPE_BIT);
  /* Enable SPI Module. */
  *(volatile UINT16*)SPI_CTRL_ADDRESS = spi_value;
}
/********************************************************************
 *  Function Name : spiDisable.                                     *
 *  Description   : This function disables SPI module.              *
 *  Input         : None.                                           *
 *  Output        : None.                                           *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiDisable(void)
{
  volatile UINT16 spi_value = *(volatile UINT16*)SPI_CTRL_ADDRESS;
  spi_value &= ~(0x1 << SPI_SPE_BIT);
  /* Disable Interrupts */
  osLockInterrupts();
  /* Disable SPI Module. */
  *(volatile UINT16*)SPI_CTRL_ADDRESS = spi_value;
  /* Enable Interrupts */
  osUnlockInterrupts();
}
/********************************************************************
 *  Function Name : spiControlChipSelect.                           *
 *  Description   : This function enables/disables SPI Chip select. *
 *  Input         : bEnable => 1 -> Enable , 0 -> Disable.          *
 *                  Codec.                                          *
 *  Output        : None.                                           *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiControlChipSelect(UCHAR bEnable, int codec)
{
  volatile UINT16 spi_value = *(volatile UINT16*)SPI_FLAG_ADDRESS;
  if(bEnable)
  {
    if(codec == 0)
    {
      /* enable codec 0 */
      spi_value &= (~ (0x1 << (CODEC_CS0_GPIO+8)));
    }
    else if(codec == 1)
    {
      /* Enable Codec 1 */
      spi_value &=  (~ (0x1 << (CODEC_CS1_GPIO+8)));
    }
    else if(codec == 2)
    {
      /* Enable Codec 2 */
      spi_value &=  (~ (0x1 << (CODEC_CS2_GPIO+8)));
    }
  }
  else
  {
    /* disable codec  */
    if(codec == 0)
      spi_value |=  (0x1 << (CODEC_CS0_GPIO+8));
    else if(codec == 1)
      spi_value |= (0x1 << (CODEC_CS1_GPIO+8));
    else if(codec == 2)
      spi_value |= (0x1 << (CODEC_CS2_GPIO+8));
  }
  *(volatile UINT16*)SPI_FLAG_ADDRESS = spi_value;
}
/********************************************************************
 *  Function Name : spiSetMode.                                     *
 *  Description   : This function configures SPI module.            *
 *  Input         : Baud Rate.                                      *
 *  Output        : None.                                           *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiSetMode(int baudRate)
{
  UINT16 temp = 0;
  /* Configure Baud Rate Register */
  *(volatile UINT16*)SPI_BAUD_ADDRESS = baudRate;
  /* Configure SPI Flag register */
  temp = ( 0x1 << CODEC_CS0_GPIO ) ;
  temp |= (0x1 << (CODEC_CS0_GPIO+8));
  temp |= (0x1 << CODEC_CS1_GPIO ) ;
  temp |= (0x1 << (CODEC_CS1_GPIO+8));
  temp |= (0x1 << CODEC_CS2_GPIO ) ;
  temp |= (0x1 << (CODEC_CS2_GPIO+8));
  *(volatile UINT16*)SPI_FLAG_ADDRESS = temp;
  /* Configure SPI Control Register */
  temp = 0;
  temp |= ( SPI_INIT_XFER << SPI_TIMOD_FST);
  temp |= ( SPI_SEND_ZEROS << SPI_SZ_BIT );
  temp |= ( SPI_OVERWRITE << SPI_GM_BIT );
  temp |= ( SPI_EN_SLAVE_SEL << SPI_PSSE_BIT );
  temp |= ( SPI_EN_MISO_OUT << SPI_EMISO_BIT );
  temp |= ( SPI_WORD_LENGTH << SPI_SIZE_BIT );
  temp |= ( SPI_LSBF << SPI_LSBF_BIT );
  temp |= ( SPI_CLOCK_PHASE << SPI_CPHA_BIT );
  temp |= ( SPI_CLOCK_POLAR << SPI_CPOL_BIT );
  temp |= ( SPI_IS_MASTER << SPI_MSTR_BIT );
  temp |= ( SPI_OPEN_DRAIN << SPI_WOM_BIT );
  *(volatile UINT16*)SPI_CTRL_ADDRESS = temp;
  /* Finally Enable SPI */
  spiEnable();
}
/********************************************************************
 *  Function Name : spiInit.                                        *
 *  Description   : This function initializes SPI Module.           *
 *  Input         : Baud Rate.                                      *
 *  Output        : None.                                           *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiInit(int baudRate)
{
  volatile UINT16 gpio_value = *(volatile UINT16*)GPIO_DIR_REG;
  /* set direction to output for codec chipselect */
  gpio_value = gpio_value | ( 0x1 << CODEC_CS0_GPIO );
  if( *(volatile UINT32*)0xb900003c == 0x6836)
  gpio_value = gpio_value | ( 0x1 << CODEC_CS1_GPIO );
//  gpio_value = gpio_value | ( 0x1 << CODEC_CS2_GPIO );
  *(volatile UINT16*)GPIO_DIR_REG = gpio_value;
  /* Configure SPI Module */
  spiSetMode(baudRate);
}
/********************************************************************
 *  Function Name : spiSendByte.                                    *
 *  Description   : Function to send byte on SPI                    *
 *  Input         : data => byte to send.                           *
 *  Output        : None.                                           *
 *  Return Value  : 0 on Success else 1 if can not transmit.        *
 *******************************************************************/
int spiSendByte(unsigned char data)
{
  /* Get for SPI Tx Status */
  volatile UINT16 spi_status = spiGetStatus();
  /* If Empty, write the byte */
  if( 0 == GETBIT( spi_status, SPI_TXS ) )
  {
    *(volatile UINT16*)SPI_XMIT_BUF_ADDRESS = data;
    return 0;
  }
  return 1;
}
/********************************************************************
 *  Function Name : spiSendData.                                    *
 *  Description   : Function to send data on SPI                    *
 *  Input         : data => Data to send, len => length of the data *
 *  Output        : None.                                           *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiSendData(unsigned char *data,int len)
{
  int i;
  volatile int result;
  int safeguard = 0;
  for(i = 0; i < len; i++)
  {
    /* Wait until we dont transmit the byte */
    while ( 1 )
    {
      result = spiSendByte( data[i] );
      if(!result)
        break;
      else 
      {
        safeguard ++;
        if(safeguard == 1000)
        {
           printk("Could not write data on SPI \n");
           break;
        }
      }
    }
  }
}
/********************************************************************
 *  Function Name : spiRecvByte.                                    *
 *  Description   : Function to receive byte on SPI                 *
 *  Input         : None.                                           *
 *  Output        : Data .                                          *
 *  Return Value  : 0 on Success else 1 if can not transmit.        *
 *******************************************************************/
int spiRecvByte(unsigned char *data)
{
  /* Get for SPI Tx Status */
  volatile INT16 spi_status = spiGetStatus();
  if( 1 == GETBIT( spi_status, SPI_RXS ) )
  {
    spiGetData(data);
    return 0;
  }
  return 1;
}
/********************************************************************
 *  Function Name : spiRecvData.                                    *
 *  Description   : Function to receive data on SPI.                *
 *  Input         : length of the data.                             *
 *  Output        : data => buffer to store the data.               *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiRecvData(unsigned char *data,int len)
{
  int i;
  volatile int result;
  int safeguard = 0;
  for ( i = 0; i < len; i++ )
  {
    while ( 1 )
    {
      result = spiRecvByte( &(data[i]) );
      if(!result)
        break;
      else
      {
        safeguard ++;
        if(safeguard == 1000)
        {
           printk("Could not read data on SPI \n");
           break;
        }
      }
    }
  }
}
/********************************************************************
 *  Function Name : spiInitOperation.                               *
 *  Description   : Function to start SPI operation.                *
 *  Input         : data to send.                                   *
 *  Output        : recvData => data received.                      *
 *  Return Value  : None.                                           *
 *******************************************************************/
void spiInitOperation(unsigned char data,unsigned char *recvData)
{
  spiSendData(&data,1);
  spiRecvData(recvData,1);
}

void ASSERT_CHIPSELECT(int codec)
{
  spiControlChipSelect(1,codec);
}

void DEASSERT_CHIPSELECT(int codec)
{
  unsigned char dummy = 0;
  spiControlChipSelect(0,codec);
  spiInitOperation(dummy,&dummy);
}

